@@ -1,6 +1,9 @@
-static const char *TagName   = "$Name:  $";
-static const char *HttpServer= "http://www.esrf.fr/computing/cs/tango/tango_doc/ds_doc/";
-static const char *RcsId = "$Header: /cvsroot/tango-cs/tango/cppserver/dbase/DataBaseClass.cpp,v 2.34 2007/04/25 06:01:03 pascal_verdier Exp $";
+static const char *ClassId    = "$Id: DataBaseClass.cpp,v 2.38 2009/04/06 13:11:08 pascal_verdier Exp $";
+static const char *CvsPath    = "$Source: /cvsroot/tango-cs/tango/cppserver/dbase/DataBaseClass.cpp,v $";
+static const char *SvnPath    = "$HeadURL: $";
+static const char *TagName    = "$Name:  $";
+static const char *HttpServer = "http://www.esrf.fr/computing/cs/tango/tango_doc/ds_doc/";
+static const char *RcsId = "$Header: /cvsroot/tango-cs/tango/cppserver/dbase/DataBaseClass.cpp,v 2.38 2009/04/06 13:11:08 pascal_verdier Exp $";
 //+=============================================================================
 //
 // file :        DataBaseClass.cpp
@@ -17,6 +20,30 @@ static const char *RcsId = "$Header: /cvsroot/tango-cs/tango/cppserver/dbase/Dat
 // $Version$
 //
 // $Log: DataBaseClass.cpp,v $
+// Revision 2.38  2009/04/06 13:11:08  pascal_verdier
+// DbMySqlSelect command added.
+//
+// Revision 2.37  2009/01/30 08:17:51  taurel
+// - Added DbGetDeviceWideList command
+//
+// Revision 2.36  2008/03/26 17:31:00  taurel
+// - Committed after merge with the Release 4 branch
+//
+// Revision 2.35.2.3  2008/03/26 17:15:36  taurel
+// - Add some code in the device-factory to set the info returned by the Tango "info" call
+//
+// Revision 2.35.2.2  2008/02/28 17:07:37  taurel
+// - Added the StoredProcedureRelease attribute
+//
+// Revision 2.35.2.1  2008/01/25 12:29:50  taurel
+// - Add a new command DbDeleteAllDeviceAttributeProperty
+// - The DbImportDevice now also returns the device class
+// - Fix bug in the DbGetHostServerList command
+//
+// Revision 2.35  2007/11/06 07:24:06  taurel
+// - Added the DbGetDataForServerCache command (with timing stats)
+// - Add timing stats for the DbPutClassProperty command
+//
 // Revision 2.34  2007/04/25 06:01:03  pascal_verdier
 // IOR NULL checked in DbGetDeviceInfo() and DbImportDevice().
 //
@@ -188,6 +215,128 @@ static const char *RcsId = "$Header: /cvsroot/tango-cs/tango/cppserver/dbase/Dat
 
 namespace DataBase_ns {
 
+
+//+----------------------------------------------------------------------------
+//
+// method : 		DbMySqlSelectCmd::execute()
+// 
+// description : 	method to trigger the execution of the command.
+//                PLEASE DO NOT MODIFY this method core without pogo   
+//
+// in : - device : The device on which the command must be executed
+//		- in_any : The command input data
+//
+// returns : The command output data (packed in the Any object)
+//
+//-----------------------------------------------------------------------------
+CORBA::Any *DbMySqlSelectCmd::execute(Tango::DeviceImpl *device,const CORBA::Any &in_any)
+{
+
+	cout2 << "DbMySqlSelectCmd::execute(): arrived" << endl;
+
+	Tango::DevString	argin;
+	extract(in_any, argin);
+
+	return insert((static_cast<DataBase *>(device))->db_my_sql_select(argin));
+}
+
+
+//+----------------------------------------------------------------------------
+//
+// method : 		DbGetDeviceWideListCmd::execute()
+// 
+// description : 	method to trigger the execution of the command.
+//                PLEASE DO NOT MODIFY this method core without pogo   
+//
+// in : - device : The device on which the command must be excuted
+//		- in_any : The command input data
+//
+// returns : The command output data (packed in the Any object)
+//
+//-----------------------------------------------------------------------------
+CORBA::Any *DbGetDeviceWideListCmd::execute(Tango::DeviceImpl *device,const CORBA::Any &in_any)
+{
+
+	cout2 << "DbGetDeviceWideListCmd::execute(): arrived" << endl;
+
+	Tango::DevString	argin;
+	extract(in_any, argin);
+
+	return insert((static_cast<DataBase *>(device))->db_get_device_wide_list(argin));
+}
+
+//+----------------------------------------------------------------------------
+//
+// method : 		DbGetDeviceListCmd::execute()
+// 
+// description : 	method to trigger the execution of the command.
+//                PLEASE DO NOT MODIFY this method core without pogo   
+//
+// in : - device : The device on which the command must be excuted
+//		- in_any : The command input data
+//
+// returns : The command output data (packed in the Any object)
+//
+//-----------------------------------------------------------------------------
+CORBA::Any *DbGetDeviceListCmd::execute(Tango::DeviceImpl *device,const CORBA::Any &in_any)
+{
+
+	cout2 << "DbGetDeviceListCmd::execute(): arrived" << endl;
+
+	const Tango::DevVarStringArray	*argin;
+	extract(in_any, argin);
+
+	return insert((static_cast<DataBase *>(device))->db_get_device_list(argin));
+}
+
+//+----------------------------------------------------------------------------
+//
+// method : 		DbDeleteAllDeviceAttributePropertyClass::execute()
+// 
+// description : 	method to trigger the execution of the command.
+//                PLEASE DO NOT MODIFY this method core without pogo   
+//
+// in : - device : The device on which the command must be executed
+//		- in_any : The command input data
+//
+// returns : The command output data (packed in the Any object)
+//
+//-----------------------------------------------------------------------------
+CORBA::Any *DbDeleteAllDeviceAttributePropertyClass::execute(Tango::DeviceImpl *device,const CORBA::Any &in_any)
+{
+
+	cout2 << "DbDeleteAllDeviceAttributePropertyClass::execute(): arrived" << endl;
+
+	const Tango::DevVarStringArray	*argin;
+	extract(in_any, argin);
+
+	((static_cast<DataBase *>(device))->db_delete_all_device_attribute_property(argin));
+	return new CORBA::Any();
+}
+
+//+----------------------------------------------------------------------------
+//
+// method : 		DbGetDataForServerCacheCmd::execute()
+// 
+// description : 	method to trigger the execution of the command.
+//                PLEASE DO NOT MODIFY this method core without pogo   
+//
+// in : - device : The device on which the command must be excuted
+//		- in_any : The command input data
+//
+// returns : The command output data (packed in the Any object)
+//
+//-----------------------------------------------------------------------------
+CORBA::Any *DbGetDataForServerCacheCmd::execute(Tango::DeviceImpl *device,const CORBA::Any &in_any)
+{
+
+	cout2 << "DbGetDataForServerCacheCmd::execute(): arrived" << endl;
+
+	const Tango::DevVarStringArray	*argin;
+	extract(in_any, argin);
+
+	return insert((static_cast<DataBase *>(device))->db_get_data_for_server_cache(argin));
+}
 //+----------------------------------------------------------------------------
 //
 // method : 		DbGetClassInheritanceForDeviceClass::execute()
@@ -1488,28 +1637,6 @@ CORBA::Any *DbInfo::execute(Tango::DeviceImpl *device,const CORBA::Any &in_any)
 }
 
 
-//+----------------------------------------------------------------------------
-//
-// method : 		DbGetDeviceList::execute()
-// 
-// description : 	method to trigger the execution of the command
-//
-// in : - device : The device on which the command must be excuted
-//		- in_any : The command input data
-//
-// returns : The command output data (packed in the Any object)
-//
-//-----------------------------------------------------------------------------
-CORBA::Any *DbGetDeviceList::execute(Tango::DeviceImpl *device,const CORBA::Any &in_any)
-{
-
-	cout2 << "DbGetDeviceList::execute(): arrived" << endl;
-
-	const Tango::DevVarStringArray	*argin;
-	extract(in_any, argin);
-
-	return insert((static_cast<DataBase *>(device))->db_get_device_list(argin));
-}
 
 
 //+----------------------------------------------------------------------------
@@ -2284,10 +2411,15 @@ void DataBaseClass::command_factory()
 		"Device name",
 		"Info from DbImportDevice and started/stopped dates.",
 		Tango::OPERATOR));
-	command_list.push_back(new DbGetDeviceList("DbGetDeviceList",
+	command_list.push_back(new DbGetDeviceListCmd("DbGetDeviceList",
 		Tango::DEVVAR_STRINGARRAY, Tango::DEVVAR_STRINGARRAY,
-		"",
-		"",
+		"argin[0] : server name\nargin[1] : class name",
+		"The list of devices for specified server and class.",
+		Tango::OPERATOR));
+	command_list.push_back(new DbGetDeviceWideListCmd("DbGetDeviceWideList",
+		Tango::DEV_STRING, Tango::DEVVAR_STRINGARRAY,
+		"filter",
+		"list of exported devices",
 		Tango::OPERATOR));
 	command_list.push_back(new DbGetDeviceMemberList("DbGetDeviceMemberList",
 		Tango::DEV_STRING, Tango::DEVVAR_STRINGARRAY,
@@ -2459,6 +2591,21 @@ void DataBaseClass::command_factory()
 		"",
 		"",
 		Tango::OPERATOR));
+	command_list.push_back(new DbGetDataForServerCacheCmd("DbGetDataForServerCache",
+		Tango::DEVVAR_STRINGARRAY, Tango::DEVVAR_STRINGARRAY,
+		"Elt[0] = DS name (exec_name/inst_name), Elt[1] = Host name",
+		"All the data needed by the device server during its startup sequence. Precise list depend on the device server",
+		Tango::OPERATOR));
+	command_list.push_back(new DbDeleteAllDeviceAttributePropertyClass("DbDeleteAllDeviceAttributeProperty",
+		Tango::DEVVAR_STRINGARRAY, Tango::DEV_VOID,
+		"str[0] = device name, str[1]...str[n] = attribute name(s)",
+		"",
+		Tango::OPERATOR));
+	command_list.push_back(new DbMySqlSelectCmd("DbMySqlSelect",
+		Tango::DEV_STRING, Tango::DEVVAR_LONGSTRINGARRAY,
+		"MySql Select command",
+		"MySql Select command result\n - svalues : select results\n - lvalue[n] : =0 if svalue[n] is null else =1\n (last lvalue -1) is number of rows, (last lvalue) is number of fields",
+		Tango::OPERATOR));
 
 	//	add polling if any
 	for (unsigned int i=0 ; i<command_list.size(); i++)
@@ -2477,7 +2624,7 @@ void DataBaseClass::command_factory()
 //+----------------------------------------------------------------------------
 Tango::DbDatum DataBaseClass::get_class_property(string &prop_name)
 {
-	for (int i=0 ; i<cl_prop.size() ; i++)
+	for (unsigned int i=0 ; i<cl_prop.size() ; i++)
 		if (cl_prop[i].name == prop_name)
 			return cl_prop[i];
 	//	if not found, return  an empty DbDatum
@@ -2492,7 +2639,7 @@ Tango::DbDatum DataBaseClass::get_class_property(string &prop_name)
 //-----------------------------------------------------------------------------
 Tango::DbDatum DataBaseClass::get_default_device_property(string &prop_name)
 {
-	for (int i=0 ; i<dev_def_prop.size() ; i++)
+	for (unsigned int i=0 ; i<dev_def_prop.size() ; i++)
 		if (dev_def_prop[i].name == prop_name)
 			return dev_def_prop[i];
 	//	if not found, return  an empty DbDatum
@@ -2508,7 +2655,7 @@ Tango::DbDatum DataBaseClass::get_default_device_property(string &prop_name)
 //-----------------------------------------------------------------------------
 Tango::DbDatum DataBaseClass::get_default_class_property(string &prop_name)
 {
-	for (int i=0 ; i<cl_def_prop.size() ; i++)
+	for (unsigned int i=0 ; i<cl_def_prop.size() ; i++)
 		if (cl_def_prop[i].name == prop_name)
 			return cl_def_prop[i];
 	//	if not found, return  an empty DbDatum
@@ -2526,18 +2673,23 @@ Tango::DbDatum DataBaseClass::get_default_class_property(string &prop_name)
 //-----------------------------------------------------------------------------
 void DataBaseClass::device_factory(const Tango::DevVarStringArray *devlist_ptr)
 {
+
+
 	cout << "DataBaseClass::DataBaseClass(): create database device " << DataBase::db_name.c_str() << endl;
 	device_list.push_back(new DataBase(this, DataBase::db_name.c_str(),
                                     	"TANGO database device server"));
 
 	export_device(device_list[0],"database");
 }
-
 //+----------------------------------------------------------------------------
 //	Method: DataBaseClass::attribute_factory(vector<Tango::Attr *> &att_list)
 //-----------------------------------------------------------------------------
 void DataBaseClass::attribute_factory(vector<Tango::Attr *> &att_list)
 {
+	//	Attribute : StoredProcedureRelease
+	StoredProcedureReleaseAttrib	*stored_procedure_release = new StoredProcedureReleaseAttrib();
+	att_list.push_back(stored_procedure_release);
+
 	//	Attribute : Timing_average
 	Timing_averageAttrib	*timing_average = new Timing_averageAttrib();
 	att_list.push_back(timing_average);
@@ -2562,6 +2714,8 @@ void DataBaseClass::attribute_factory(vector<Tango::Attr *> &att_list)
 	Timing_infoAttrib	*timing_info = new Timing_infoAttrib();
 	att_list.push_back(timing_info);
 
+	//	End of Automatic code generation
+	//-------------------------------------------------------------
 }
 
 //+----------------------------------------------------------------------------
@@ -2578,12 +2732,11 @@ void DataBaseClass::get_class_property()
 
 	//	Read class properties from database.(Automatic code generation)
 	//------------------------------------------------------------------
-	if (Tango::Util::instance()->_UseDb==false)
-		return;
 
 	//	Call database and extract values
 	//--------------------------------------------
-	get_db_class()->get_property(cl_prop);
+	if (Tango::Util::instance()->_UseDb==true)
+		get_db_class()->get_property(cl_prop);
 	Tango::DbDatum	def_prop;
 	int	i = -1;
 
@@ -2645,17 +2798,23 @@ void DataBaseClass::write_class_property()
 	description << str_desc;
 	data.push_back(description);
 		
-	//	put cvs location
-	string	rcsId(RcsId);
+	//	put cvs or svn location
 	string	filename(classname);
-	start = rcsId.find("/");
+	filename += "Class.cpp";
+	
+	// Create a string with the class ID to
+	// get the string into the binary
+	string	class_id(ClassId);
+	
+	// check for cvs information
+	string	src_path(CvsPath);
+	start = src_path.find("/");
 	if (start!=string::npos)
 	{
-		filename += "Class.cpp";
-		end   = rcsId.find(filename);
+		end   = src_path.find(filename);
 		if (end>start)
 		{
-			string	strloc = rcsId.substr(start, end-start);
+			string	strloc = src_path.substr(start, end-start);
 			//	Check if specific repository
 			start = strloc.find("/cvsroot/");
 			if (start!=string::npos && start>0)
@@ -2669,20 +2828,67 @@ void DataBaseClass::write_class_property()
 			data.push_back(cvs_loc);
 		}
 	}
+	// check for svn information
+	else
+	{
+		string	src_path(SvnPath);
+		start = src_path.find("://");
+		if (start!=string::npos)
+		{
+			end = src_path.find(filename);
+			if (end>start)
+			{
+				header = "$HeadURL: ";
+				start = header.length();
+				string	strloc = src_path.substr(start, (end-start));
+				
+				Tango::DbDatum	svn_loc("svn_location");
+				svn_loc << strloc;
+				data.push_back(svn_loc);
+			}
+		}
+	}
 
-	//	Get CVS tag revision
+	//	Get CVS or SVN revision tag
+	
+	// CVS tag
 	string	tagname(TagName);
 	header = "$Name: ";
 	start = header.length();
 	string	endstr(" $");
+	
 	end   = tagname.find(endstr);
-	if (end>start)
+	if (end!=string::npos && end>start)
 	{
 		string	strtag = tagname.substr(start, end-start);
 		Tango::DbDatum	cvs_tag("cvs_tag");
 		cvs_tag << strtag;
 		data.push_back(cvs_tag);
 	}
+	
+	// SVN tag
+	string	svnpath(SvnPath);
+	header = "$HeadURL: ";
+	start = header.length();
+	
+	end   = svnpath.find(endstr);
+	if (end!=string::npos && end>start)
+	{
+		string	strloc = svnpath.substr(start, end-start);
+		
+		string tagstr ("/tags/");
+		start = strloc.find(tagstr);
+		if ( start!=string::npos )
+		{
+			start = start + tagstr.length();
+			end   = strloc.find(filename);
+			string	strtag = strloc.substr(start, end-start-1);
+			
+			Tango::DbDatum	svn_tag("svn_tag");
+			svn_tag << strtag;
+			data.push_back(svn_tag);
+		}
+	}
 
 	//	Get URL location
 	string	httpServ(HttpServer);
@@ -2693,6 +2899,13 @@ void DataBaseClass::write_class_property()
 		data.push_back(db_doc_url);
 	}
 
+	//  Put inheritance
+	Tango::DbDatum	inher_datum("InheritedFrom");
+	vector<string> inheritance;
+	inheritance.push_back("Device_4Impl");
+	inher_datum << inheritance;
+	data.push_back(inher_datum);
+
 	//	Call database and and values
 	//--------------------------------------------
 	get_db_class()->put_property(data);
