@@ -10,11 +10,17 @@
 //			
 // project :      TANGO Device Server
 //
-// $Author: taurel $
+// $Author: pascal_verdier $
 //
-// $Revision: 2.31 $
+// $Revision: 2.33 $
 //
 // $Log: DataBaseClass.h,v $
+// Revision 2.33  2009/04/06 13:11:08  pascal_verdier
+// DbMySqlSelect command added.
+//
+// Revision 2.32  2009/01/30 08:17:51  taurel
+// - Added DbGetDeviceWideList command
+//
 // Revision 2.31  2008/03/26 17:31:00  taurel
 // - Committed after merge with the Release 4 branch
 //
@@ -224,6 +230,30 @@ public:
 //=========================================
 //	Define classes for commands
 //=========================================
+class DbMySqlSelectCmd : public Tango::Command
+{
+public:
+	DbMySqlSelectCmd(const char   *name,
+	               Tango::CmdArgType in,
+				   Tango::CmdArgType out,
+				   const char        *in_desc,
+				   const char        *out_desc,
+				   Tango::DispLevel  level)
+	:Command(name,in,out,in_desc,out_desc, level)	{};
+
+	DbMySqlSelectCmd(const char   *name,
+	               Tango::CmdArgType in,
+				   Tango::CmdArgType out)
+	:Command(name,in,out)	{};
+	~DbMySqlSelectCmd() {};
+	
+	virtual CORBA::Any *execute (Tango::DeviceImpl *dev, const CORBA::Any &any);
+	virtual bool is_allowed (Tango::DeviceImpl *dev, const CORBA::Any &any)
+	{return (static_cast<DataBase *>(dev))->is_DbMySqlSelect_allowed(any);}
+};
+
+
+
 class DbDeleteAllDeviceAttributePropertyClass : public Tango::Command
 {
 public:
@@ -1088,10 +1118,10 @@ public:
 
 
 
-class DbGetDeviceList : public Tango::Command
+class DbGetDeviceWideListCmd : public Tango::Command
 {
 public:
-	DbGetDeviceList(const char   *name,
+	DbGetDeviceWideListCmd(const char   *name,
 	               Tango::CmdArgType in,
 				   Tango::CmdArgType out,
 				   const char        *in_desc,
@@ -1099,11 +1129,35 @@ public:
 				   Tango::DispLevel  level)
 	:Command(name,in,out,in_desc,out_desc, level)	{};
 
-	DbGetDeviceList(const char   *name,
+	DbGetDeviceWideListCmd(const char   *name,
 	               Tango::CmdArgType in,
 				   Tango::CmdArgType out)
 	:Command(name,in,out)	{};
-	~DbGetDeviceList() {};
+	~DbGetDeviceWideListCmd() {};
+	
+	virtual CORBA::Any *execute (Tango::DeviceImpl *dev, const CORBA::Any &any);
+	virtual bool is_allowed (Tango::DeviceImpl *dev, const CORBA::Any &any)
+	{return (static_cast<DataBase *>(dev))->is_DbGetDeviceWideList_allowed(any);}
+};
+
+
+
+class DbGetDeviceListCmd : public Tango::Command
+{
+public:
+	DbGetDeviceListCmd(const char   *name,
+	               Tango::CmdArgType in,
+				   Tango::CmdArgType out,
+				   const char        *in_desc,
+				   const char        *out_desc,
+				   Tango::DispLevel  level)
+	:Command(name,in,out,in_desc,out_desc, level)	{};
+
+	DbGetDeviceListCmd(const char   *name,
+	               Tango::CmdArgType in,
+				   Tango::CmdArgType out)
+	:Command(name,in,out)	{};
+	~DbGetDeviceListCmd() {};
 	
 	virtual CORBA::Any *execute (Tango::DeviceImpl *dev, const CORBA::Any &any);
 	virtual bool is_allowed (Tango::DeviceImpl *dev, const CORBA::Any &any)
@@ -2109,6 +2163,8 @@ protected:
 	void attribute_factory(vector<Tango::Attr *> &);
 	void write_class_property();
 	void set_default_property();
+	string get_cvstag();
+	string get_cvsroot();
 
 private:
 	void device_factory(const Tango::DevVarStringArray *);
