@@ -1,4 +1,4 @@
-static const char *RcsId = "$Header: /cvsroot/tango-cs/tango/cppserver/dbase/DataBaseUtils.cpp,v 2.7 2008/03/26 17:31:01 taurel Exp $";
+static const char *RcsId = "$Header: /cvsroot/tango-cs/tango/cppserver/dbase/DataBaseUtils.cpp,v 2.10 2009/04/27 08:01:40 taurel Exp $";
 //+=============================================================================
 //
 // file :         DataBaseUtils.cpp
@@ -18,7 +18,7 @@ static const char *RcsId = "$Header: /cvsroot/tango-cs/tango/cppserver/dbase/Dat
 #include <tango.h>
 #include <DataBase.h>
 #include <DataBaseClass.h>
-#include <mysql.h>
+
 #include <stdio.h>
 
 using namespace std;
@@ -361,7 +361,7 @@ void DataBase::init_timing_stats()
 
 //+----------------------------------------------------------------------------
 //
-// method : 		DataBase::get_id()
+// method : 		DataBase::check_history_tables()
 // 
 // description : 	Return history id
 //
@@ -723,12 +723,15 @@ void DataBase::purge_att_property(const char *table,const char *field,const char
  */
 //+------------------------------------------------------------------
 
-void DataBase::create_connection_pool(char *mysql_user,char *mysql_password)
+void DataBase::create_connection_pool(const char *mysql_user,const char *mysql_password)
 {
 	char *database = (char *)"tango";
-	
-	WARN_STREAM << "DataBase::create_connection_pool(): mysql database user =  " << mysql_user 
-	            << " , password = " << mysql_password << endl;
+
+	if (mysql_user != NULL && mysql_password != NULL)
+	{	
+		WARN_STREAM << "DataBase::create_connection_pool(): mysql database user =  " << mysql_user 
+	           	 << " , password = " << mysql_password << endl;
+	}
 					
 	for (int loop = 0;loop < conn_pool_size;loop++)
 	{
