@@ -6,6 +6,8 @@
 #include <sys/resource.h>
 #endif
 
+int DataBase_ns::DataBase::conn_pool_size;
+
 int main(int argc,char *argv[])
 {
 
@@ -43,8 +45,8 @@ int main(int argc,char *argv[])
 	  limit.rlim_cur = flimit;
 	  limit.rlim_max = flimit;
 	}
-	
-	// Apply the limit
+
+	// Apply the max open file limit
 	if( setrlimit(RLIMIT_NOFILE,&limit) != 0 ) {	
 	  cout << "setrlimit(RLIMIT_NOFILE," << (int)limit.rlim_cur << ") failed." << endl;	  
 	  if(errno==EPERM) {
@@ -54,8 +56,33 @@ int main(int argc,char *argv[])
 	  }
 	  return -1;
 	}
+#endif	
+	
+	// Browse argv and try to find the -connPoolSize option
+	int j = 0;
+	int found_conn = 0;
+	int conn_size = DEFAULT_CONN_POOL_SIZE;
+	while(j<argc && !found_conn) {
+#ifndef WIN32
+	  found_conn = (strcasecmp(argv[j],"-poolSize")==0);
+#else
+	  found_conn = (stricmp(argv[j],"-poolSize")==0);
+#endif
+	  if(!found_conn) j++;
+	}
+	if( found_conn ) {
+	  if(j>=argc-1) {
+	    cout << "Invalid poolSize parameter." << endl;
+	    return -1;
+	  }
+	  conn_size = atoi(argv[j+1]);
+	  if(conn_size<=0) {
+	    cout << "Invalid poolSize parameter." << endl;
+	    return -1;
+	  }
+	}
+	DataBase_ns::DataBase::set_conn_pool_size(conn_size);
 	
-#endif		
 
 	try
 	{
@@ -71,6 +98,19 @@ int main(int argc,char *argv[])
 		DataBase_ns::DataBase::db_name.append(argv[1]);
 		cout1 << "main(): create DataBase " << DataBase_ns::DataBase::db_name << endl;
 
+//
+// Create and install interceptors
+//
+
+		DataBase_ns::DbInter *dbi = new DataBase_ns::DbInter();
+		tango_util->set_interceptors(dbi);
+
+//
+// Set the serialization method
+//
+
+		tango_util->set_serial_model(Tango::NO_SYNC);
+
 //
 // Create the device server singleton which will create everything
 //
@@ -83,7 +123,7 @@ int main(int argc,char *argv[])
 // database
 //
 		Tango::DeviceImpl *dbase, *dserver;
-        	Tango::DevVarStringArray *export_parms = new Tango::DevVarStringArray();
+        Tango::DevVarStringArray *export_parms = new Tango::DevVarStringArray();
 
 		dserver = tango_util->get_dserver_device();
 		dbase = tango_util->get_device_by_name(DataBase_ns::DataBase::db_name);
@@ -97,20 +137,18 @@ int main(int argc,char *argv[])
 
 		Tango::Device_var d = dserver->_this();
 		dserver->set_d_var(Tango::Device::_duplicate(d));
-        	(*export_parms)[0] = CORBA::string_dup(dserver->get_name().c_str());
-        	(*export_parms)[1] = CORBA::string_dup((Tango::Util::instance()->get_orb()->object_to_string(d)));
-        	(*export_parms)[2] = CORBA::string_dup(tango_util->get_host_name().c_str());
-        	(*export_parms)[3] = CORBA::string_dup(tango_util->get_pid_str().c_str());
-        	(*export_parms)[4] = CORBA::string_dup(tango_util->get_version_str().c_str());
-		//((DataBase_ns::DataBase*)dbase)->db_export_device(export_parms);
+		(*export_parms)[0] = CORBA::string_dup(dserver->get_name().c_str());
+		(*export_parms)[1] = CORBA::string_dup((Tango::Util::instance()->get_orb()->object_to_string(d)));
+		(*export_parms)[2] = CORBA::string_dup(tango_util->get_host_name().c_str());
+		(*export_parms)[3] = CORBA::string_dup(tango_util->get_pid_str().c_str());
+		(*export_parms)[4] = CORBA::string_dup(tango_util->get_version_str().c_str());
+
 		(static_cast<DataBase_ns::DataBase *>(dbase))->db_export_device(export_parms);
 
 // export database object to TANGO database
-			//CORBA::string_free((*export_parms)[0]);
-			//CORBA::string_free((*export_parms)[1]);
 
-        	(*export_parms)[0] = CORBA::string_dup(dbase->get_name().c_str());
-        	(*export_parms)[1] = CORBA::string_dup((Tango::Util::instance()->get_orb()->object_to_string(dbase->get_d_var())));
+		(*export_parms)[0] = CORBA::string_dup(dbase->get_name().c_str());
+		(*export_parms)[1] = CORBA::string_dup((Tango::Util::instance()->get_orb()->object_to_string(dbase->get_d_var())));
 		((DataBase_ns::DataBase*)dbase)->db_export_device(export_parms);
 
 //
