@@ -1,4 +1,4 @@
-static const char *RcsId = "$Header: /cvsroot/tango-cs/tango/cppserver/starter/StartProcessThread.cpp,v 3.15 2008/04/04 14:30:51 jensmeyer Exp $";
+static const char *RcsId = "$Header: /cvsroot/tango-cs/tango/cppserver/starter/StartProcessThread.cpp,v 3.17 2008/12/12 13:29:56 pascal_verdier Exp $";
 //+=============================================================================
 //
 // file :         Starter.cpp
@@ -7,11 +7,17 @@ static const char *RcsId = "$Header: /cvsroot/tango-cs/tango/cppserver/starter/S
 //
 // project :      TANGO Device Server
 //
-// $Author: jensmeyer $
+// $Author: pascal_verdier $
 //
-// $Revision: 3.15 $
+// $Revision: 3.17 $
 //
 // $Log: StartProcessThread.cpp,v $
+// Revision 3.17  2008/12/12 13:29:56  pascal_verdier
+// Log in file start and stop for servers and itself.
+//
+// Revision 3.16  2008/09/23 14:19:40  pascal_verdier
+// Log files history added.
+//
 // Revision 3.15  2008/04/04 14:30:51  jensmeyer
 // Added compile option for FreeBSD and MacOSX.
 // They have a different signal handling.
@@ -172,7 +178,13 @@ void *StartProcessThread::run_undetached(void *ptr)
 			}
 		}
 		if (started)
+		{
+			TangoSys_OMemStream out_stream;
+			out_stream << servname << " started";
+			starter->util->log_starter_info(out_stream.str());
+
 			cout << "----- " << servname << " started in " << (t1-t0) << " sec." << endl;
+		}
 		else
 		if (failed)
 			cout << "----- " << servname << " failed in " << (t1-t0) << " sec." << endl;
@@ -249,7 +261,9 @@ void StartProcessThread::start_process(NewProcess *process)
 				//	Close the stderr and re-open it on a log file.
 				//-------------------------------------------------
 				close(2);
-				remove(process->logfile);
+				starter->util->manage_log_file_history(
+						process->logfile, starter->keepLogFiles);
+				//remove(process->logfile);
 				open(process->logfile, O_RDWR | O_CREAT, 0664);
 
 				//	Start the execution of the device server
@@ -287,7 +301,14 @@ void StartProcessThread::start_process(NewProcess *process)
 //+----------------------------------------------------------
 void StartProcessThread::start_process(NewProcess *process)
 {
-	StartWinThread	*win_thread = new StartWinThread(process);
+	/*****
+	On Win32 problem with permission denied !
+	It seems that this not necessary because there is nothing in file.
+	
+	starter->util->manage_log_file_history(
+			process->logfile, starter->keepLogFiles);
+	*/
+	StartWinThread	*win_thread = new StartWinThread(process, starter);
 	win_thread->start();
 }
 
