@@ -8,9 +8,21 @@
 //
 // $Author: pascal_verdier $
 //
-// $Revision: 3.21 $
+// $Revision: 3.25 $
 //
 // $Log: Starter.h,v $
+// Revision 3.25  2009/04/06 13:57:57  pascal_verdier
+// IDL-4 (Tango-7) release.
+//
+// Revision 3.24  2008/09/23 14:19:40  pascal_verdier
+// Log files history added.
+//
+// Revision 3.23  2008/04/09 14:46:11  pascal_verdier
+// Bug in init of NotifdState attribute fixed.
+//
+// Revision 3.22  2008/04/07 08:54:55  pascal_verdier
+// Check if this starter instance is able to run on this host.
+//
 // Revision 3.21  2008/02/29 15:15:05  pascal_verdier
 // Checking running processes by system call added.
 //
@@ -138,7 +150,7 @@
 
 /**
  * @author	$Author: pascal_verdier $
- * @version	$Revision: 3.21 $
+ * @version	$Revision: 3.25 $
  */
 
  //	Add your own constants definitions here.
@@ -181,7 +193,7 @@ class StartProcessShared;
  */
 
 
-class Starter: public Tango::Device_3Impl
+class Starter: public Tango::Device_4Impl
 {
 public :
 	//	Add your own data members here
@@ -278,6 +290,10 @@ public :
  *	Timeout on device server startup in seconds.
  */
 	Tango::DevLong	serverStartupTimeout;
+/**
+ *	Number of log file kept.
+ */
+	Tango::DevLong	keepLogFiles;
 //@}
 
 /**@name Constructors
@@ -527,6 +543,11 @@ NewProcess *processCouldStart(char *);
  *	Start a thread to start processes
  */
 void startProcesses(vector<NewProcess *>, int level);
+
+/**
+ *	check if instance and host name are coherent
+ */
+void check_host();
 //@}
 };
 
@@ -566,9 +587,11 @@ public:
 #ifdef WIN32
 class StartWinThread: public omni_thread
 {
-	NewProcess* process;;
+	NewProcess	*process;
+	Starter		*starter;
 public:
-	StartWinThread(NewProcess* proc) { process = proc; };
+	StartWinThread(NewProcess* proc, Starter *st) 
+		{ process = proc; starter=st;};
 
 	void *run_undetached(void *);
 	void start() {start_undetached();}
