@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: attr_proxy.cpp,v 1.8 2008/01/18 14:44:40 jensmeyer Exp $\n$Name:  $";
+static const char *RcsId = "$Id: attr_proxy.cpp,v 1.14 2009/01/29 15:28:06 taurel Exp $\n$Name:  $";
 //
 // attr_proxy.cpp 	- C++ source code file for TANGO attribute proxy api 
 //
@@ -6,6 +6,26 @@ static const char *RcsId = "$Id: attr_proxy.cpp,v 1.8 2008/01/18 14:44:40 jensme
 //
 // original 		- July 2003
 //
+// Copyright (C) :      2003,2004,2005,2006,2007,2008,2009
+//						European Synchrotron Radiation Facility
+//                      BP 220, Grenoble 38043
+//                      FRANCE
+//
+// This file is part of Tango.
+//
+// Tango is free software: you can redistribute it and/or modify
+// it under the terms of the GNU Lesser General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+// 
+// Tango is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU Lesser General Public License for more details.
+// 
+// You should have received a copy of the GNU Lesser General Public License
+// along with Tango.  If not, see <http://www.gnu.org/licenses/>.
+//
 // version 		- $Version$
 //
 
@@ -115,7 +135,7 @@ void AttributeProxy::real_constructor (string &name)
 		if (strcmp(dfe.errors[0].reason.in(),"API_AttrNotFound") == 0)
 		{
 			TangoSys_OMemStream desc;
-			desc << "Attribute" << attr_name << " is not supported by device " << device_name << ends;
+			desc << "Attribute " << attr_name << " is not supported by device " << device_name << ends;
 			
 			ApiWrongNameExcept::throw_exception((const char*)"API_UnsupportedAttribute",
 						desc.str(),
@@ -1221,6 +1241,18 @@ void AttributeProxy::write(DeviceAttribute& attr_value)
 	dev_proxy->write_attribute(attr_value);
 }
 
+//-----------------------------------------------------------------------------
+//
+// AttributeProxy::write_read() - write then read attribute
+//
+//-----------------------------------------------------------------------------
+ 
+DeviceAttribute AttributeProxy::write_read(DeviceAttribute& attr_value) 
+{
+	attr_value.set_name(attr_name);
+	return dev_proxy->write_read_attribute(attr_value);
+}
+
 //-----------------------------------------------------------------------------
 //
 // AttributeProxy::history() - get attribute history
@@ -1320,5 +1352,26 @@ int AttributeProxy::subscribe_event (EventType event, CallBack *callback,
 	                           attr_name, event, callback, filters, stateless);
 }
 
+//-----------------------------------------------------------------------------
+//
+// AttributeProxy::subscribe_event - Subscribe to an event with an event queue
+//                                   Adds the statless flag for stateless
+//                                   event subscription.
+//
+//-----------------------------------------------------------------------------
+
+int AttributeProxy::subscribe_event (EventType event, int event_queue_size, 
+                                    const vector<string> &filters, bool stateless)
+{
+	if (ApiUtil::instance()->get_event_consumer() == NULL)
+	{
+		ApiUtil::instance()->create_event_consumer();
+	}
+	
+	return ApiUtil::instance()->get_event_consumer()->subscribe_event(dev_proxy, 
+	                           attr_name, event, event_queue_size, filters, stateless);
+}
+
+
 
 } // End of Tango namespace
