@@ -2,6 +2,27 @@
 // dbapi.h -	include file for TANGO database api
 //
 // 
+// Copyright (C) :      2004,2005,2006,2007,2008,2009
+//						European Synchrotron Radiation Facility
+//                      BP 220, Grenoble 38043
+//                      FRANCE
+//
+// This file is part of Tango.
+//
+// Tango is free software: you can redistribute it and/or modify
+// it under the terms of the GNU Lesser General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+// 
+// Tango is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU Lesser General Public License for more details.
+// 
+// You should have received a copy of the GNU Lesser General Public License
+// along with Tango.  If not, see <http://www.gnu.org/licenses/>.
+
+
 #ifndef _DBAPI_H
 #define _DBAPI_H
 
@@ -33,6 +54,7 @@ class DbDatumExt;
 class FileDatabase;
 class DbServerCache;
 class Util;
+class AccessProxy;
 
 typedef vector<DbDatum> DbData;
 typedef vector<DbDevInfo> DbDevInfos;
@@ -58,20 +80,37 @@ public:
 class Database : public Tango::Connection
 {
 private :	
-	virtual string get_corba_name();
+	virtual string get_corba_name(bool);
 	virtual string build_corba_name() {return string("nada");}
-	inline string dev_name() { return string("Db device"); }
+	virtual int get_lock_ctr() {return 0;}
+	virtual void set_lock_ctr(int lo) {}
 	
-	bool			db_multi_svc;
-	vector<string>	multi_db_port;
-	vector<string>	multi_db_host;
-	DatabaseExt		*ext;
-	FileDatabase 	*filedb; 
-	string 			file_name;
-	int				serv_version;
+	inline string dev_name() 
+	{if (db_device_name.empty() == true)
+		{CORBA::String_var n = device->name();db_device_name = n;}
+	return db_device_name; }
+	
+	bool				db_multi_svc;
+	vector<string>		multi_db_port;
+	vector<string>		multi_db_host;
+	DatabaseExt			*ext;
+	FileDatabase 		*filedb; 
+	string 				file_name;
+	int					serv_version;
+	
+	AccessProxy			*access_proxy;
+	bool				access_checked;
+	DevErrorList		access_except_errors;
+	
+	map<string,string>	dev_class_cache;
+	string				db_device_name;
+	
+	bool				access_service_defined;
 	
 	DbDatum         make_string_array(string, CORBA::Any_var &);
 	vector<DbHistory> make_history_array(bool, CORBA::Any_var &);
+	
+	void check_access();
 
 public :
 	Database(CORBA::ORB *orb=NULL);
@@ -83,9 +122,16 @@ public :
 	void build_connection ();
 	~Database();
 	inline Device_var &get_dbase() { return device;}
-	void check_tango_host(char *);
+	void check_tango_host(const char *);
+	AccessControlType check_access_control(string &);
+	bool is_control_access_checked() {return access_checked;}
+	void set_access_checked(bool val) {access_checked = val;}
 	
 	void set_tango_utils(Tango::Util *ptr) {ext->db_tg=ptr;}
+	
+	DevErrorList &get_access_except_errors() {return access_except_errors;}
+	void clear_access_except_errors() {access_except_errors.length(0);}
+	bool is_command_allowed(string &,string &);
 
 #ifdef _TG_WINDOWS_
 	Database(CORBA::ORB *orb,string &,string &);
@@ -153,7 +199,8 @@ public :
 //
 
 	void get_property(string, DbData &,DbServerCache *dsc);
-	void get_property(string st, DbData &db) {get_property(st,db,NULL);}
+	void get_property(string st, DbData &db) {get_property(st,db,NULL);}	
+	void get_property_forced(string, DbData &,DbServerCache *dsc = NULL);  
 	void put_property(string, DbData &);  
 	void delete_property(string, DbData &);
 	vector<DbHistory> get_property_history(string &,string &);
@@ -166,6 +213,7 @@ public :
 	void delete_device_property(string, DbData &);
 	vector<DbHistory> get_device_property_history(string &,string &);
 	DbDatum get_device_property_list(string &,string &);
+	void get_device_property_list(string &,const string &,vector<string> &,DbServerCache *dsc = NULL);
 	
 	void get_device_attribute_property(string, DbData &, DbServerCache *dsc);
 	void get_device_attribute_property(string st, DbData &db) {get_device_attribute_property(st,db,NULL);}
@@ -331,6 +379,9 @@ public :
 	void get_attribute_property(DbData&);
 	void put_attribute_property(DbData&);
 	void delete_attribute_property(DbData&);
+	AccessControlType check_access_control();
+	void clear_access_except_errors();
+	void get_property_list(const string &,vector<string> &);
 };
 
 //
@@ -434,7 +485,11 @@ public :
 //
 	DbDatum();
 	DbDatum (string);
+	DbDatum (const char *);
 	~DbDatum();
+	DbDatum(const DbDatum &);
+	DbDatum &operator=(const DbDatum &);
+
 	int size() {return value_string.size();}
 	bool is_empty();
 	
@@ -647,6 +702,7 @@ public:
 	const DevVarStringArray *get_class_att_property(DevVarStringArray *);
 	const DevVarStringArray *get_dev_att_property(DevVarStringArray *);
 	const DevVarStringArray *get_obj_property(DevVarStringArray *);
+	const DevVarStringArray *get_device_property_list(DevVarStringArray *);
 	
 	const EltIdx &get_imp_dat() {return imp_adm;}
 	const EltIdx &get_imp_notifd_event() {return imp_notifd_event;}
@@ -666,6 +722,7 @@ private:
 	int find_class(DevString );
 	int find_dev_att(DevString,int &,int &);
 	int find_obj(DevString obj_name,int &);
+	void get_obj_prop_list(DevVarStringArray *,PropEltIdx &);
 	
 	CORBA::Any_var			received;
 	const DevVarStringArray *data_list;
@@ -687,6 +744,7 @@ private:
 	DevVarStringArray		ret_obj_prop;
 	DevVarStringArray		ret_dev_list;
 	DevVarStringArray		ret_obj_att_prop;
+	DevVarStringArray		ret_prop_list;
 };
 
 //
