@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: dbapi_class.cpp,v 3.3 2008/02/29 13:21:23 taurel Exp $\n$Name:  $";
+static const char *RcsId = "$Id: dbapi_class.cpp,v 3.7 2009/03/13 09:32:27 taurel Exp $\n$Name:  $";
 //
 // dbapi_class.cpp - C++ source code file for TANGO dbapi class DbClass
 //
@@ -6,9 +6,25 @@ static const char *RcsId = "$Id: dbapi_class.cpp,v 3.3 2008/02/29 13:21:23 taure
 //
 // original 	- October 2000
 //
-// last changed	- 17/10/2000 
+// Copyright (C) :      2000,2001,2002,2003,2004,2005,2006,2007,2008,2009
+//						European Synchrotron Radiation Facility
+//                      BP 220, Grenoble 38043
+//                      FRANCE
 //
-// version 	- 1.0
+// This file is part of Tango.
+//
+// Tango is free software: you can redistribute it and/or modify
+// it under the terms of the GNU Lesser General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+// 
+// Tango is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU Lesser General Public License for more details.
+// 
+// You should have received a copy of the GNU Lesser General Public License
+// along with Tango.  If not, see <http://www.gnu.org/licenses/>.
 //
 
 #if HAVE_CONFIG_H
@@ -138,7 +154,7 @@ void DbClass::put_property(DbData &db_data)
 			(au->get_db_vect())[db_ind]->put_class_property(name, db_data);
 		}
 	}
-	catch (Tango::DevFailed &e)
+	catch (Tango::DevFailed &)
 	{
 		if (forget_except == false)
 			throw;
