@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: dbapi_datum.cpp,v 3.5 2007/11/08 12:00:20 taurel Exp $\n$Name:  $";
+static const char *RcsId = "$Id: dbapi_datum.cpp,v 3.10 2009/04/20 13:25:50 taurel Exp $\n$Name:  $";
 //
 // dbdatum.cpp 	- C++ source code file for TANGO dbapi class DbDatum
 //
@@ -6,11 +6,43 @@ static const char *RcsId = "$Id: dbapi_datum.cpp,v 3.5 2007/11/08 12:00:20 taure
 //
 // original 	- October 2000
 //
-// last changed	- 17/10/2000 
+// Copyright (C) :      2000,2001,2002,2003,2004,2005,2006,2007,2008,2009
+//						European Synchrotron Radiation Facility
+//                      BP 220, Grenoble 38043
+//                      FRANCE
 //
-// version 	- 1.0
+// This file is part of Tango.
+//
+// Tango is free software: you can redistribute it and/or modify
+// it under the terms of the GNU Lesser General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+// 
+// Tango is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU Lesser General Public License for more details.
+// 
+// You should have received a copy of the GNU Lesser General Public License
+// along with Tango.  If not, see <http://www.gnu.org/licenses/>.
 //
 // $Log: dbapi_datum.cpp,v $
+// Revision 3.10  2009/04/20 13:25:50  taurel
+// - Fix bug in case of default constructed DeviceProxy and alias() method
+// - Add some ctors from "const char *" to make programmer's life easier
+//
+// Revision 3.9  2009/03/13 09:32:27  taurel
+// - Small changes to fix Windows VC8 warnings in Warning level 3
+//
+// Revision 3.8  2009/01/21 12:45:15  taurel
+// - Change CopyRights for 2009
+//
+// Revision 3.7  2008/10/06 15:02:16  taurel
+// - Changed the licensing info from GPL to LGPL
+//
+// Revision 3.6  2008/10/02 16:09:25  taurel
+// - Add some licensing information in each files...
+//
 // Revision 3.5  2007/11/08 12:00:20  taurel
 // - Add miscellaneous getter methods in the DbServerCache class
 // - The DbDatum::Size() method is now inline
@@ -182,20 +214,26 @@ namespace Tango
 //
 //-----------------------------------------------------------------------------
 
-DbDatum::DbDatum(string p_name)
+DbDatum::DbDatum(string p_name):ext(NULL)
 {
 	name = p_name;
 	value_size = 0;
 	value_string.resize(0);
 }
 
+DbDatum::DbDatum(const char *p_name):ext(NULL),name(p_name)
+{
+	value_size = 0;
+	value_string.resize(0);
+}
+
 //-----------------------------------------------------------------------------
 //
 // DbDatum::DbDatum() - constructor to create DbDatum without arguments
 //
 //-----------------------------------------------------------------------------
 
-DbDatum::DbDatum()
+DbDatum::DbDatum():ext(NULL)
 {
 }
 
@@ -209,6 +247,40 @@ DbDatum::~DbDatum()
 {
 }
 
+//-----------------------------------------------------------------------------
+//
+// DbDatum::DbDatum() - Copy constructor
+//
+//-----------------------------------------------------------------------------
+
+DbDatum::DbDatum(const DbDatum &source) 
+{
+	name = source.name;
+	value_string = source.value_string;
+	value_type = source.value_type;
+	value_size = source.value_size;
+	exceptions_flags = source.exceptions_flags;
+	ext = source.ext;
+}
+
+//-----------------------------------------------------------------------------
+//
+// DbDatum::operator() - assignement operator
+//
+//-----------------------------------------------------------------------------
+
+DbDatum &DbDatum::operator=(const DbDatum &rval)
+{
+	name = rval.name;
+	value_string = rval.value_string;
+	value_type = rval.value_type;
+	value_size = rval.value_size;
+	exceptions_flags = rval.exceptions_flags;
+	ext = rval.ext;
+
+	return *this;
+}
+
 //-----------------------------------------------------------------------------
 //
 // DbDatum::is_empty() - returns true or false if datum is empty
