@@ -2,6 +2,26 @@
 // devsyn.h - include file for TANGO api device asynchronous calls 
 //
 // 
+// Copyright (C) :      2004,2005,2006,2007,2008,2009
+//						European Synchrotron Radiation Facility
+//                      BP 220, Grenoble 38043
+//                      FRANCE
+//
+// This file is part of Tango.
+//
+// Tango is free software: you can redistribute it and/or modify
+// it under the terms of the GNU Lesser General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+// 
+// Tango is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU Lesser General Public License for more details.
+// 
+// You should have received a copy of the GNU Lesser General Public License
+// along with Tango.  If not, see <http://www.gnu.org/licenses/>.
+
 
 #ifndef _DEVASYN_H
 #define _DEVASYN_H
@@ -23,14 +43,24 @@ class DeviceData;
 class DeviceAttribute;
 class NamedDevFailedList;
 class EventData;
+class AttrConfEventData;
+class DataReadyEventData;
+class EventDataList;
+class AttrConfEventDataList;
+class DataReadyEventDataList;
 class EventConsumer;
 class EventConsumerKeepAliveThread;
-
 class CmdDoneEventExt;
 class AttrReadEventExt;
 class AttrWrittenEventExt;
 class CallBackExt;
 
+/********************************************************************************
+ *                                                                              *
+ * 						CmdDoneEvent class										*
+ * 																				*
+ *******************************************************************************/
+ 
 class CmdDoneEvent
 {
 public:
@@ -44,14 +74,20 @@ public:
 	{if (errors.length()==0) err=false;else err=true;}
 	
 	Tango::DeviceProxy 	*device;
-	string			&cmd_name;
-	DeviceData		&argout;
-	bool			err;
+	string				&cmd_name;
+	DeviceData			&argout;
+	bool				err;
 	DevErrorList		&errors;
 	
 	CmdDoneEventExt		*ext;
 };
 
+/********************************************************************************
+ * 																				*
+ * 						AttrReadEvent class										*
+ * 																				*
+ *******************************************************************************/
+ 
 class AttrReadEvent
 {
 public:
@@ -64,15 +100,21 @@ public:
 					    errors(err_in)
 	{if (errors.length()==0) err=false;else err=true;}
 
-	Tango::DeviceProxy 	*device;
-	vector<string>		&attr_names;
+	Tango::DeviceProxy 		*device;
+	vector<string>			&attr_names;
 	vector<DeviceAttribute>	*argout;
-	bool			err;
-	DevErrorList		&errors;
+	bool					err;
+	DevErrorList			&errors;
 	
-	AttrReadEventExt	*ext;
+	AttrReadEventExt		*ext;
 };
 
+/********************************************************************************
+ * 																				*
+ * 						AttrWrittenEvent class									*
+ * 																				*
+ *******************************************************************************/
+ 
 class AttrWrittenEvent
 {
 public:
@@ -85,12 +127,18 @@ public:
 	
 	Tango::DeviceProxy	*device;
 	vector<string>		&attr_names;
-	bool			err;
+	bool				err;
 	NamedDevFailedList	&errors;
 	
 	AttrWrittenEventExt	*ext;
 };
 
+/********************************************************************************
+ * 																				*
+ * 						CallBack class											*
+ * 																				*
+ *******************************************************************************/
+ 
 class CallBack
 {
 
@@ -98,10 +146,14 @@ friend class EventConsumer;
 friend class EventConsumerKeepAliveThread;
 
 public:
+	virtual ~CallBack() {};
+
 	virtual void cmd_ended(CmdDoneEvent *) {};
 	virtual void attr_read(AttrReadEvent *) {};
 	virtual void attr_written(AttrWrittenEvent *) {};
 	virtual void push_event(EventData *) {};
+	virtual void push_event(AttrConfEventData *) {};
+	virtual void push_event(DataReadyEventData *) {};
 	
 	CallBackExt		*ext;
 };
@@ -120,38 +172,31 @@ public:
 class TgRequest
 {
 public:
-        enum ReqType
+	enum ReqType
 	{
-                CMD_INOUT,
-                READ_ATTR,
+		CMD_INOUT,
+		READ_ATTR,
 		WRITE_ATTR_SINGLE,
 		WRITE_ATTR
 	};	
 	
-	TgRequest(CORBA::Request_ptr re,ReqType ty):request(re),
-						    req_type(ty),
-						    cb_ptr(NULL),
-						    arrived(false),
-						    dev(NULL)
+	TgRequest(CORBA::Request_ptr re,ReqType ty):request(re),req_type(ty),cb_ptr(NULL),
+						    					arrived(false),dev(NULL)
 	{};
-	TgRequest(CORBA::Request_ptr re,ReqType ty,CallBack *cb):request(re),
-						    	  req_type(ty),
-						    	  cb_ptr(cb),
-							  arrived(false),
-							  dev(NULL)
+	
+	TgRequest(CORBA::Request_ptr re,ReqType ty,CallBack *cb):request(re),req_type(ty),cb_ptr(cb),
+							 								 arrived(false),dev(NULL)
 	{};
-	TgRequest(Tango::Connection *con,ReqType ty,CallBack *cb):request(NULL),
-						    	  req_type(ty),
-						    	  cb_ptr(cb),
-							  arrived(false),
-							  dev(con)
+	
+	TgRequest(Tango::Connection *con,ReqType ty,CallBack *cb):request(NULL),req_type(ty),cb_ptr(cb),
+							 								  arrived(false),dev(con)
 	{};
 		
 	CORBA::Request_ptr	request;
-	ReqType			req_type;
-	CallBack		*cb_ptr;
-	bool			arrived;
-	Connection		*dev;
+	ReqType				req_type;
+	CallBack			*cb_ptr;
+	bool				arrived;
+	Connection			*dev;
 };
 
 class AsynReq: public omni_mutex
@@ -168,24 +213,29 @@ public:
 	void store_request(CORBA::Request_ptr,CallBack *,Connection *,TgRequest::ReqType);
 
 	void remove_request(long);
+	bool remove_cancelled_request(long);
 	void remove_request(Connection *,CORBA::Request_ptr);
 	
-	long get_request_nb() {omni_mutex_lock(*this);return asyn_poll_req_table.size();}
-	long get_cb_request_nb() {omni_mutex_lock(*this);return cb_req_table.size();}
+	size_t get_request_nb() {omni_mutex_lock(*this);return asyn_poll_req_table.size();}
+	size_t get_cb_request_nb() {omni_mutex_lock(*this);return cb_req_table.size();}
 	size_t get_cb_request_nb_i() {return cb_req_table.size();}
 	
 	void mark_as_arrived(CORBA::Request_ptr req);
 	multimap<Connection *,TgRequest> &get_cb_dev_table() {return cb_dev_table;}
 
+	void mark_as_cancelled(long);
+	void mark_all_polling_as_cancelled();
 	void wait() {cond.wait();}
 	void signal() {omni_mutex_lock(*this);cond.signal();}
 			
 protected:	
 	map<long,TgRequest>			asyn_poll_req_table;
-	UniqIdent 				*ui_ptr;
+	UniqIdent 					*ui_ptr;
 	
 	multimap<Connection *,TgRequest>	cb_dev_table;
 	map<CORBA::Request_ptr,TgRequest>	cb_req_table;
+
+	vector<long>				cancelled_request;
 	
 private:
 	omni_condition				cond;
