@@ -1,3 +1,23 @@
+// Copyright (C) :      2004,2005,2006,2007,2008,2009
+//						European Synchrotron Radiation Facility
+//                      BP 220, Grenoble 38043
+//                      FRANCE
+//
+// This file is part of Tango.
+//
+// Tango is free software: you can redistribute it and/or modify
+// it under the terms of the GNU Lesser General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+// 
+// Tango is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU Lesser General Public License for more details.
+// 
+// You should have received a copy of the GNU Lesser General Public License
+// along with Tango.  If not, see <http://www.gnu.org/licenses/>.
+
 #include <iostream>
 #include <tango.h>
 
@@ -1337,7 +1357,11 @@ CORBA::Any*   FileDatabase :: DbGetDeviceAttributeProperty(CORBA::Any& send)
 						char num_val_str[256];
 						data_out->length(index + 1 + 1 + (*dev_it)->attribute_properties[j]->properties[l]->value.size());
 						(*data_out)[index] = CORBA::string_dup((*dev_it)->attribute_properties[j]->properties[l]->name.c_str());index++;
+#ifdef TANGO_LONG64
+						sprintf(num_val_str, "%lu", (*dev_it)->attribute_properties[j]->properties[l]->value.size());
+#else
 						sprintf(num_val_str, "%d", (*dev_it)->attribute_properties[j]->properties[l]->value.size());
+#endif
 						(*data_out)[index] = CORBA::string_dup(num_val_str); index++;
 						
 						for(unsigned int ii = 0; ii < (*dev_it)->attribute_properties[j]->properties[l]->value.size(); ii++)
@@ -1556,7 +1580,11 @@ CORBA::Any*   FileDatabase :: DbGetClassProperty(CORBA::Any& send)
 						seq_length = seq_length + 2 + num_val;
 						(*data_out).length(seq_length);
 						(*data_out)[index] = CORBA::string_dup((*data_in)[j]); index++;
+#ifdef TANGO_LONG64
+						sprintf(num_vals_str,"%lu",m_server.classes[i]->properties[m]->value.size());
+#else
 						sprintf(num_vals_str,"%d",m_server.classes[i]->properties[m]->value.size());
+#endif
 						(*data_out)[index] = CORBA::string_dup(num_vals_str); index++;
 						for (unsigned int n = 0; n < num_val; n++)
 						{
@@ -1763,7 +1791,11 @@ CORBA::Any*   FileDatabase :: DbGetClassAttributeProperty(CORBA::Any& send)
 					char num_val_str[256];
 					data_out->length(index + 1 + 1 + classe_trovata.attribute_properties[j]->properties[l]->value.size());
 					(*data_out)[index] = CORBA::string_dup(classe_trovata.attribute_properties[j]->properties[l]->name.c_str());index++;
+#ifdef TANGO_LONG64
+					sprintf(num_val_str, "%lu", classe_trovata.attribute_properties[j]->properties[l]->value.size());
+#else
 					sprintf(num_val_str, "%d", classe_trovata.attribute_properties[j]->properties[l]->value.size());
+#endif
 					(*data_out)[index] = CORBA::string_dup(num_val_str); index++;
 				 //(*data_out)[index] = CORBA::string_dup(classe_trovata.attribute_properties[j]->properties[l]->name.c_str());index++;
 				 //string temp_value("");
@@ -1963,9 +1995,17 @@ CORBA::Any*  FileDatabase :: DbInfo(CORBA::Any&){
 	(*data_out)[1] = CORBA::string_dup("");
 	(*data_out)[2] = CORBA::string_dup("Running since ----");
 	(*data_out)[3] = CORBA::string_dup("");
+#ifdef TANGO_LONG64
+	sprintf(temp_str,"Devices defined = %lu", m_server.devices.size());
+#else
 	sprintf(temp_str,"Devices defined = %d", m_server.devices.size());
+#endif
 	(*data_out)[4] = CORBA::string_dup(temp_str);
+#ifdef TANGO_LONG64
+	sprintf(temp_str,"Devices exported = %lu", m_server.devices.size());
+#else
 	sprintf(temp_str,"Devices exported = %d", m_server.devices.size());
+#endif
 	(*data_out)[5] = CORBA::string_dup(temp_str);
 	(*data_out)[6] = CORBA::string_dup("Device servers defined = 1");
 	(*data_out)[7] = CORBA::string_dup("Device servers exported = 1");
