@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: proxy_asyn.cpp,v 3.12 2008/03/25 15:47:25 taurel Exp $\n$Name:  $";
+static const char *RcsId = "$Id: proxy_asyn.cpp,v 3.25 2009/03/27 13:05:10 taurel Exp $\n$Name:  $";
 
 //
 // cpp 	- C++ source code file for TANGO api Connection and DeviceProxy
@@ -8,12 +8,80 @@ static const char *RcsId = "$Id: proxy_asyn.cpp,v 3.12 2008/03/25 15:47:25 taure
 //
 // original 	- August 2002
 //
+// Copyright (C) :      2002,2003,2004,2005,2006,2007,2008,2009
+//						European Synchrotron Radiation Facility
+//                      BP 220, Grenoble 38043
+//                      FRANCE
+//
+// This file is part of Tango.
+//
+// Tango is free software: you can redistribute it and/or modify
+// it under the terms of the GNU Lesser General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+// 
+// Tango is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU Lesser General Public License for more details.
+// 
+// You should have received a copy of the GNU Lesser General Public License
+// along with Tango.  If not, see <http://www.gnu.org/licenses/>.
+//
+//
 // $Log: proxy_asyn.cpp,v $
+// Revision 3.25  2009/03/27 13:05:10  taurel
+// - Fix bug due to new Attribute format data member in AttributeValue4
+// structure
+//
+// Revision 3.24  2009/03/27 12:19:33  taurel
+// - Fix bug in read_attribute for IDL 4
+//
+// Revision 3.23  2009/03/13 09:32:27  taurel
+// - Small changes to fix Windows VC8 warnings in Warning level 3
+//
+// Revision 3.22  2009/02/19 08:48:52  taurel
+// - Add cancel asynchronous request calls to the DeviceProxy class
+//
+// Revision 3.21  2009/01/21 12:45:15  taurel
+// - Change CopyRights for 2009
+//
+// Revision 3.20  2008/12/17 09:54:44  taurel
+// - First implementation of attributes sent on the wire using IDL Union
+// instead of IDL Any
+//
+// Revision 3.19  2008/10/13 15:04:37  taurel
+// - Fix bug in command_inout_reply(). SourceForge bug nb 2157328
+//
+// Revision 3.18  2008/10/06 15:02:17  taurel
+// - Changed the licensing info from GPL to LGPL
+//
+// Revision 3.17  2008/10/02 16:09:25  taurel
+// - Add some licensing information in each files...
+//
+// Revision 3.16  2008/09/23 14:38:29  taurel
+// - Commit after the end of DevEncoded data type implementation
+// - The new test suite is also now running fine
+//
+// Revision 3.15  2008/09/01 14:25:47  taurel
+// - Fix some bugs in locking implementation
+// - The Tango::Callback class dtor is now defined as virtual as it should be
+//
+// Revision 3.14  2008/06/14 11:28:07  taurel
+// - DevEncoded attribute data type implementation work going on
+//
+// Revision 3.13  2008/05/20 12:42:30  taurel
+// - Commit after merge with release 7 branch
+//
 // Revision 3.12  2008/03/25 15:47:25  taurel
 // - Fix some warnings for gcc 4.2 (Thank's to F Picca from Soleil)
+// Revision 3.10.2.2  2008/05/20 06:14:19  taurel
+// - Last commit before merge with trunk
 //
 // Revision 3.11  2008/03/11 14:36:45  taurel
 // - Apply patches from Frederic Picca about compilation with gcc 4.2
+// Revision 3.10.2.1  2008/02/07 15:56:58  taurel
+// - First implementation of the Controlled Access done
 //
 // Revision 3.10  2007/04/20 14:38:33  taurel
 // - Ported to Windows 64 bits x64 architecture
@@ -156,7 +224,50 @@ namespace Tango
 
 long Connection::command_inout_asynch(const char *command, DeviceData &data_in, bool faf)
 {
+//
+// Throw exception if caller not allowed to write_attribute
+//
+ 
+	if (access == ACCESS_READ)
+	{
+		int db_num;
+		ApiUtil *au = ApiUtil::instance();
+		
+		if (get_from_env_var() == true)
+			db_num = au->get_db_ind();
+		else
+			db_num = au->get_db_ind(get_db_host(),get_db_port_num());
 
+		vector<Database *> & v_d = au->get_db_vect();
+		Database *db = v_d[db_num];
+
+//
+// If the command is not allowed, throw exception
+// Also throw exception if it was not possible to get the list
+// of allowed commands from the control access service
+//
+	
+		string d_name = dev_name();
+		string cmd(command);
+		
+		if (db->is_command_allowed(d_name,cmd) == false)
+		{
+			DevErrorList &e = db->get_access_except_errors();
+			if (e.length() != 0)
+			{
+				DevFailed df(e);
+				throw df;
+			}
+			
+			TangoSys_OMemStream desc;
+			desc << "Command_inout_asynch on device " << dev_name() << " for command ";
+			desc << command << " is not authorized" << ends;
+	
+			NotAllowedExcept::throw_exception((const char *)"API_ReadOnlyMode",desc.str(),
+									  	  (const char *)"Connection::command_inout_asynch()");
+		}
+	}
+		
 //
 // Reconnect to device in case it is needed
 //
@@ -180,14 +291,29 @@ long Connection::command_inout_asynch(const char *command, DeviceData &data_in,
 //
 
 	CORBA::Request_ptr request;
-	if (version == 2)
+	
+	if (version >= 4)
+		request = ext->device_4->_request("command_inout_4");
+	else if (version >=2)
 		request = device_2->_request("command_inout_2");
 	else	
 		request = device->_request("command_inout");
+	
 	request->add_in_arg() <<= command;
 	request->add_in_arg() <<= data_in.any.in();
-	if (version == 2)
+	
+	if (version >= 4)
+	{
+		ClntIdent ci;
+		ApiUtil *au = ApiUtil::instance();
+		ci.cpp_clnt(au->get_client_pid());
+		
 		request->add_in_arg() <<= source;
+		request->add_in_arg() <<= ci;
+	}
+	else if (version >= 2)
+		request->add_in_arg() <<= source;
+	
 	request->set_return_type(CORBA::_tc_any);
 	request->exceptions()->add(Tango::_tc_DevFailed);
 
@@ -380,7 +506,7 @@ DeviceData Connection::command_inout_reply(long id)
 
 			TangoSys_OMemStream desc;
 			desc << "Failed to execute command_inout_asynch on device " << dev_name();
-			desc << ", command " << cmd << ends;
+			desc << ", command " << tmp << ends;
 			CORBA::string_free(tmp);
 		
 			ApiUtil::instance()->get_pasyn_table()->remove_request(id);
@@ -426,7 +552,6 @@ DeviceData Connection::command_inout_reply(long id)
 				{
 					DeviceProxy *dev_ptr = static_cast<DeviceProxy *>(this);
 					dev_ptr->ping();
-					CORBA::string_free(tmp);
 
 //
 // If we arrive here, this means we have to do a re-try with a synchronous
@@ -439,18 +564,19 @@ DeviceData Connection::command_inout_reply(long id)
 // Remove request from request global table.
 //
 
+					CORBA::string_free(tmp);
 					ApiUtil::instance()->get_pasyn_table()->remove_request(id);
 					pasyn_ctr--;
 				
 					return dd_out;
 					
 				}
-				catch (Tango::DevFailed &e) {}
+				catch (Tango::DevFailed &) {}
 			}
 
 			TangoSys_OMemStream desc;
 			desc << "Failed to execute command_inout_asynch on device " << dev_name();
-			desc << ", command " << cmd << ends;
+			desc << ", command " << tmp << ends;
 			CORBA::string_free(tmp);
 		
 			ApiUtil::instance()->get_pasyn_table()->remove_request(id);
@@ -643,7 +769,7 @@ DeviceData Connection::command_inout_reply(long id,long call_timeout)
 
 			TangoSys_OMemStream desc;
 			desc << "Failed to execute command_inout_asynch on device " << dev_name();
-			desc << ", command " << cmd << ends;
+			desc << ", command " << tmp << ends;
 			CORBA::string_free(tmp);
 			
 			ApiUtil::instance()->get_pasyn_table()->remove_request(id);
@@ -677,7 +803,7 @@ DeviceData Connection::command_inout_reply(long id,long call_timeout)
 
 			TangoSys_OMemStream desc;
 			desc << "Failed to execute command_inout_asynch on device " << dev_name();
-			desc << ", command " << cmd << ends;
+			desc << ", command " << tmp << ends;
 			CORBA::string_free(tmp);
 			
 			ApiUtil::instance()->get_pasyn_table()->remove_request(id);
@@ -746,7 +872,19 @@ long DeviceProxy::read_attributes_asynch(vector<string> &attr_names)
 		names[i] = attr_names[i].c_str();
 		
 	CORBA::Request_ptr request;
-	if (version == 3)
+	if (version == 4)
+	{
+		ClntIdent ci;
+		ApiUtil *au = ApiUtil::instance();
+		ci.cpp_clnt(au->get_client_pid());
+		
+		request = Connection::ext->device_4->_request("read_attributes_4");
+		request->add_in_arg() <<= names;
+		request->add_in_arg() <<= source;
+		request->add_in_arg() <<= ci;
+		request->set_return_type(Tango::_tc_AttributeValueList_4);		
+	}
+	else if (version == 3)
 	{
 		request = Connection::ext->device_3->_request("read_attributes_3");
 		request->add_in_arg() <<= names;
@@ -874,6 +1012,7 @@ vector<DeviceAttribute> *DeviceProxy::read_attributes_reply(long id)
 
 		const Tango::AttributeValueList *received;
 		const Tango::AttributeValueList_3 *received_3;
+		const Tango::AttributeValueList_4 *received_4;
 		CORBA::Any &dii_any = req.request->return_value();
 		unsigned long nb_received;
 
@@ -882,22 +1021,26 @@ vector<DeviceAttribute> *DeviceProxy::read_attributes_reply(long id)
 			dii_any >>= received;
 			nb_received = received->length();
 		}
-		else
+		else if (version == 3)
 		{
 			dii_any >>= received_3;
 			nb_received = received_3->length();
 		}
+		else
+		{
+			dii_any >>= received_4;
+			nb_received = received_4->length();
+		}
 		dev_attr->resize(nb_received);
 				
 		for (unsigned long i=0; i < nb_received; i++)
 		{
 			if (version >= 3)
-			{		
-				ApiUtil::attr_to_device(NULL,
-							&((*received_3)[i]),
-							version,&((*dev_attr)[i]));
-
-
+			{	
+				if (version == 3)
+					ApiUtil::attr_to_device(NULL,&((*received_3)[i]),version,&((*dev_attr)[i]));
+				else
+					ApiUtil::attr_to_device(&((*received_4)[i]),version,&((*dev_attr)[i]));
 				
 //
 // Add an error in the error stack in case there is one
@@ -927,8 +1070,7 @@ vector<DeviceAttribute> *DeviceProxy::read_attributes_reply(long id)
 			else
 			{
 				ApiUtil::attr_to_device(&((*received)[i]),
-							NULL,
-							version,&((*dev_attr)[i]));
+							NULL,version,&((*dev_attr)[i]));
 			}	
 		}
 				
@@ -1032,19 +1174,23 @@ DeviceAttribute *DeviceProxy::read_attribute_reply(long id)
 
 		const Tango::AttributeValueList *received;
 		const Tango::AttributeValueList_3 *received_3;
+		const Tango::AttributeValueList_4 *received_4;
 		
 		CORBA::Any &dii_any = req.request->return_value();
 		
 		if (version < 3)
 			dii_any >>= received;
-		else
+		else if (version == 3)
 			dii_any >>= received_3;
+		else
+			dii_any >>= received_4;
 
 		if (version >= 3)
 		{
-			ApiUtil::attr_to_device(NULL,
-						&((*received_3)[0]),
-						version,dev_attr);				
+			if (version == 3)
+				ApiUtil::attr_to_device(NULL,&((*received_3)[0]),version,dev_attr);
+			else
+				ApiUtil::attr_to_device(&((*received_4)[0]),version,dev_attr);
 				
 //
 // Add an error in the error stack in case there is one
@@ -1074,8 +1220,7 @@ DeviceAttribute *DeviceProxy::read_attribute_reply(long id)
 		else
 		{
 			ApiUtil::attr_to_device(&((*received)[0]),
-						NULL,
-						version,dev_attr);
+						NULL,version,dev_attr);
 		}
 				
 //
@@ -1213,6 +1358,7 @@ vector<DeviceAttribute> *DeviceProxy::read_attributes_reply(long id,long call_ti
 
 	const Tango::AttributeValueList *received;
 	const Tango::AttributeValueList_3 *received_3;
+	const Tango::AttributeValueList_4 *received_4;
 	unsigned long nb_received;
 	
 	CORBA::Any &dii_any = req.request->return_value();
@@ -1221,11 +1367,16 @@ vector<DeviceAttribute> *DeviceProxy::read_attributes_reply(long id,long call_ti
 		dii_any >>= received;
 		nb_received = received->length();
 	}
-	else
+	else if (version == 3)
 	{
 		dii_any >>= received_3;
 		nb_received = received_3->length();
 	}
+	else
+	{
+		dii_any >>= received_4;
+		nb_received = received_4->length();
+	}
 
 	dev_attr->resize(nb_received);
 							
@@ -1233,10 +1384,10 @@ vector<DeviceAttribute> *DeviceProxy::read_attributes_reply(long id,long call_ti
 	{
 		if (version >= 3)
 		{	
-			ApiUtil::attr_to_device(NULL,
-						&((*received_3)[i]),
-						version,&((*dev_attr)[i]));
-
+			if (version == 3)
+				ApiUtil::attr_to_device(NULL,&((*received_3)[i]),version,&((*dev_attr)[i]));
+			else
+				ApiUtil::attr_to_device(&((*received_4)[i]),version,&((*dev_attr)[i]));
 
 			
 //
@@ -1266,9 +1417,7 @@ vector<DeviceAttribute> *DeviceProxy::read_attributes_reply(long id,long call_ti
 		}
 		else
 		{
-			ApiUtil::attr_to_device(&((*received)[i]),
-						NULL,
-						version,&((*dev_attr)[i]));
+			ApiUtil::attr_to_device(&((*received)[i]),NULL,version,&((*dev_attr)[i]));
 		}
 	}
 				
@@ -1406,17 +1555,22 @@ DeviceAttribute *DeviceProxy::read_attribute_reply(long id,long call_timeout)
 
 	const Tango::AttributeValueList *received;
 	const Tango::AttributeValueList_3 *received_3;
+	const Tango::AttributeValueList_4 *received_4;
+	
 	CORBA::Any &dii_any = req.request->return_value();
 	if (version < 3)
 		dii_any >>= received;
-	else
+	else if (version == 3)
 		dii_any >>= received_3;
+	else
+		dii_any >>= received_4;
 		
 	if (version >= 3)
-	{			
-		ApiUtil::attr_to_device(NULL,
-					&((*received_3)[0]),
-					version,dev_attr);
+	{	
+		if (version == 3)
+			ApiUtil::attr_to_device(NULL,&((*received_3)[0]),version,dev_attr);
+		else
+			ApiUtil::attr_to_device(&((*received_4)[0]),version,dev_attr);
 				
 //
 // Add an error in the error stack in case there is one
@@ -1445,9 +1599,7 @@ DeviceAttribute *DeviceProxy::read_attribute_reply(long id,long call_timeout)
 	}
 	else
 	{
-		ApiUtil::attr_to_device(&((*received)[0]),
-					NULL,
-					version,dev_attr);
+		ApiUtil::attr_to_device(&((*received)[0]),NULL,version,dev_attr);
 	}
 				
 //
@@ -1603,7 +1755,7 @@ void DeviceProxy::read_attr_except(CORBA::Request_ptr req,long id,read_attr_type
 				ping();
 				return;
 			}
-			catch (Tango::DevFailed &e) {}
+			catch (Tango::DevFailed &) {}
 		}
 
 		TangoSys_OMemStream desc;
@@ -1651,7 +1803,19 @@ void DeviceProxy::read_attr_except(CORBA::Request_ptr req,long id,read_attr_type
 
 long DeviceProxy::write_attributes_asynch(vector<DeviceAttribute> &attr_list)
 {
-
+//
+// Throw exception if caller not allowed to write_attribute
+//
+ 
+	if (access == ACCESS_READ)
+	{
+		TangoSys_OMemStream desc;
+		desc << "Writing attribute(s) on device " << dev_name() << " is not authorized" << ends;
+	
+		NotAllowedExcept::throw_exception((const char *)"API_ReadOnlyMode",desc.str(),
+									  	  (const char *)"DeviceProxy::write_attributes_asynch()");
+	}
+		
 //
 // Reconnect to device in case it is needed
 //
@@ -1674,90 +1838,46 @@ long DeviceProxy::write_attributes_asynch(vector<DeviceAttribute> &attr_list)
 //
 
 	Tango::AttributeValueList att;
+	Tango::AttributeValueList_4 att_4;
+
 	long nb_attr = attr_list.size();
-	att.length(nb_attr);
+	if (version >= 4)
+		att_4.length(nb_attr);
+	else
+		att.length(nb_attr);
+
 	for (int i = 0;i < nb_attr;i++)
 	{
-		att[i].name = attr_list[i].name.c_str();
-		att[i].quality = attr_list[i].quality;
-		att[i].time = attr_list[i].time;
-		att[i].dim_x = attr_list[i].dim_x;
-		att[i].dim_y = attr_list[i].dim_y;
-		
-		if (attr_list[i].LongSeq.operator->() != NULL)
-		{
-			att[i].value <<= attr_list[i].LongSeq.in();	
-			continue;
-		}
-		if (attr_list[i].ShortSeq.operator->() != NULL)
-		{	
-			att[i].value <<= attr_list[i].ShortSeq.in();
-			continue;
-		}	
-		if (attr_list[i].DoubleSeq.operator->() != NULL)
-		{	
-			att[i].value <<= attr_list[i].DoubleSeq.in();
-			continue;
-		}
-		if (attr_list[i].StringSeq.operator->() != NULL)
-		{	
-			att[i].value  <<= attr_list[i].StringSeq.in();
-			continue;
-		}
-		if (attr_list[i].FloatSeq.operator->() != NULL)
-		{
-			att[i].value <<= attr_list[i].FloatSeq.in();	
-			continue;
-		}
-		if (attr_list[i].BooleanSeq.operator->() != NULL)
-		{	
-			att[i].value <<= attr_list[i].BooleanSeq.in();
-			continue;
-		}	
-		if (attr_list[i].UShortSeq.operator->() != NULL)
-		{	
-			att[i].value <<= attr_list[i].UShortSeq.in();
-			continue;
-		}
-		if (attr_list[i].UCharSeq.operator->() != NULL)
-		{	
-			att[i].value  <<= attr_list[i].UCharSeq.in();
-			continue;
-		}
-		if (attr_list[i].ext->Long64Seq.operator->() != NULL)
-		{	
-			att[i].value  <<= attr_list[i].ext->Long64Seq.in();
-			continue;
-		}
-		if (attr_list[i].ext->ULongSeq.operator->() != NULL)
-		{	
-			att[i].value  <<= attr_list[i].ext->ULongSeq.in();
-			continue;
-		}
-		if (attr_list[i].ext->ULong64Seq.operator->() != NULL)
-		{	
-			att[i].value  <<= attr_list[i].ext->ULong64Seq.in();
-			continue;
-		}
-		if (attr_list[i].ext->StateSeq.operator->() != NULL)
-		{	
-			att[i].value  <<= attr_list[i].ext->StateSeq.in();
-			continue;
-		}
+		if (version >= 4)
+			ApiUtil::device_to_attr(attr_list[i],att_4[i]);
+		else
+			ApiUtil::device_to_attr(attr_list[i],att[i],device_name);
 	}
 		
 	CORBA::Request_ptr request;
-	if (version < 3)
+	if (version >= 4)
 	{
-		request = device->_request("write_attributes");
-		request->exceptions()->add(Tango::_tc_DevFailed);
+		ClntIdent ci;
+		ApiUtil *au = ApiUtil::instance();
+		ci.cpp_clnt(au->get_client_pid());
+		
+		request = ext->device_4->_request("write_attributes_4");
+		request->add_in_arg() <<= att_4;
+		request->add_in_arg() <<= ci;
+		request->exceptions()->add(Tango::_tc_MultiDevFailed);
+	}
+	else if (version == 3)
+	{
+		request = device->_request("write_attributes_3");
+		request->add_in_arg() <<= att;
+		request->exceptions()->add(Tango::_tc_MultiDevFailed);
 	}
 	else
 	{
-		request = device->_request("write_attributes_3");
-		request->exceptions()->add(Tango::_tc_MultiDevFailed);
+		request = device->_request("write_attributes");
+		request->add_in_arg() <<= att;
 	}
-	request->add_in_arg() <<= att;
+	request->exceptions()->add(Tango::_tc_DevFailed);
 
 //
 // Send the request 
@@ -1774,7 +1894,19 @@ long DeviceProxy::write_attributes_asynch(vector<DeviceAttribute> &attr_list)
 
 long DeviceProxy::write_attribute_asynch(DeviceAttribute &attr)
 {
-
+//
+// Throw exception if caller not allowed to write_attribute
+//
+ 
+	if (access == ACCESS_READ)
+	{
+		TangoSys_OMemStream desc;
+		desc << "Writing attribute(s) on device " << dev_name() << " is not authorized" << ends;
+	
+		NotAllowedExcept::throw_exception((const char *)"API_ReadOnlyMode",desc.str(),
+									  	  (const char *)"DeviceProxy::write_attribute_asynch()");
+	}
+		
 //
 // Reconnect to device in case it is needed
 //
@@ -1797,75 +1929,44 @@ long DeviceProxy::write_attribute_asynch(DeviceAttribute &attr)
 //
 
 	Tango::AttributeValueList att;
-	att.length(1);
-	
-	att[0].name = attr.name.c_str();
-	att[0].quality = attr.quality;
-	att[0].time = attr.time;
-	att[0].dim_x = attr.dim_x;
-	att[0].dim_y = attr.dim_y;
-	
-	if (attr.LongSeq.operator->() != NULL)
-	{
-		att[0].value <<= attr.LongSeq.in();	
-	}
-	else if (attr.ShortSeq.operator->() != NULL)
-	{	
-		att[0].value <<= attr.ShortSeq.in();
-	}	
-	else if (attr.DoubleSeq.operator->() != NULL)
-	{	
-		att[0].value <<= attr.DoubleSeq.in();
-	}
-	else if (attr.StringSeq.operator->() != NULL)
-	{	
-		att[0].value  <<= attr.StringSeq.in();
-	}
-	else if (attr.FloatSeq.operator->() != NULL)
-	{	
-		att[0].value  <<= attr.FloatSeq.in();
-	}
-	else if (attr.BooleanSeq.operator->() != NULL)
-	{	
-		att[0].value  <<= attr.BooleanSeq.in();
-	}
-	else if (attr.UShortSeq.operator->() != NULL)
-	{	
-		att[0].value  <<= attr.UShortSeq.in();
-	}
-	else if (attr.UCharSeq.operator->() != NULL)
-	{	
-		att[0].value  <<= attr.UCharSeq.in();
-	}
-	else if (attr.ext->Long64Seq.operator->() != NULL)
-	{	
-		att[0].value  <<= attr.ext->Long64Seq.in();
-	}
-	else if (attr.ext->ULongSeq.operator->() != NULL)
-	{	
-		att[0].value  <<= attr.ext->ULongSeq.in();
-	}
-	else if (attr.ext->ULong64Seq.operator->() != NULL)
-	{	
-		att[0].value  <<= attr.ext->ULong64Seq.in();
-	}
-	else if (attr.ext->StateSeq.operator->() != NULL)
-	{	
-		att[0].value  <<= attr.ext->StateSeq.in();
-	}
+	Tango::AttributeValueList_4 att_4;
 
-	CORBA::Request_ptr request;
-	if (version < 3)
-	{		
-		request = device->_request("write_attributes");
-		request->exceptions()->add(Tango::_tc_DevFailed);
+	if (version < 4)
+	{
+		att.length(1);
+		ApiUtil::device_to_attr(attr,att[0],device_name);
 	}
 	else
+	{
+		att_4.length(1);
+		ApiUtil::device_to_attr(attr,att_4[0]);
+	}
+	
+	CORBA::Request_ptr request;
+	
+	if (version >= 4)
+	{
+		ClntIdent ci;
+		ApiUtil *au = ApiUtil::instance();
+		ci.cpp_clnt(au->get_client_pid());
+		
+		request = ext->device_4->_request("write_attributes_4");
+		request->add_in_arg() <<= att_4;
+		request->add_in_arg() <<= ci;
+		request->exceptions()->add(Tango::_tc_MultiDevFailed);
+	}
+	else if (version == 3)
 	{
 		request = device->_request("write_attributes_3");
+		request->add_in_arg() <<= att;
 		request->exceptions()->add(Tango::_tc_MultiDevFailed);
 	}	
-	request->add_in_arg() <<= att;
+	else
+	{		
+		request = device->_request("write_attributes");
+		request->add_in_arg() <<= att;
+	}
+	request->exceptions()->add(Tango::_tc_DevFailed);
 
 //
 // Send the request 
@@ -2041,7 +2142,6 @@ void DeviceProxy::write_attributes_reply(long id)
 	}
 	else
 	{
-	
 //
 // Check if the reply is an exception
 //
@@ -2100,21 +2200,37 @@ void DeviceProxy::write_attr_except(CORBA::Request_ptr req,long id,TgRequest::Re
 		{		
 			CORBA::NVList_ptr req_arg = req->arguments();
 			const Tango::AttributeValueList *att;
+			const Tango::AttributeValueList_4 *att_4;
+			unsigned int nb_att = 0;
 			CORBA::NamedValue_ptr nv = req_arg->item(0);
-			*(nv->value()) >>= att;
+			
+			if (version < 4)
+			{
+				if ((*(nv->value()) >>= att) == true)
+					nb_att = att->length();
+			}
+			else
+			{
+				if ((*(nv->value()) >>= att_4) == true)
+					nb_att = att_4->length();
+			}
 						
 			char *cb_excep_mess = Tango::Except::print_CORBA_SystemException(tra);
 	
 			TangoSys_OMemStream desc;
 			desc << "Timeout (" << timeout << " mS) exceeded on device " << device_name;
-			desc << "\nAttribute(s): ";
-			for (unsigned int i = 0;i < att->length();i++)
+			if (nb_att != 0)
 			{
-				desc << (*att)[i].name;
-				if (i != (att->length() - 1))
-					desc << ", ";
+				desc << "\nAttribute(s): ";
+				for (unsigned int i = 0;i < nb_att;i++)
+				{
+					(version < 4) ? desc << (*att)[i].name : desc << (*att_4)[i].name;
+				
+					if (i != (nb_att - 1))
+						desc << ", ";
+				}
+				desc << ends;
 			}
-			desc << ends;
 			
 			ApiUtil::instance()->get_pasyn_table()->remove_request(id);
 			pasyn_ctr--;	
@@ -2135,9 +2251,9 @@ void DeviceProxy::write_attr_except(CORBA::Request_ptr req,long id,TgRequest::Re
 // server has sent a DevFailed exception
 //
 	
-		const Tango::DevFailed *serv_ex;
+		const Tango::DevFailed *serv_ex = NULL;
 		Tango::DevFailed ex;
-		const Tango::MultiDevFailed *multi_serv_ex;
+		const Tango::MultiDevFailed *multi_serv_ex = NULL;
 		Tango::MultiDevFailed m_ex;
 		
 		if (version < 3)
@@ -2147,23 +2263,47 @@ void DeviceProxy::write_attr_except(CORBA::Request_ptr req,long id,TgRequest::Re
 		}
 		else
 		{
-			unk_ex->exception() >>= multi_serv_ex;
-			m_ex = *multi_serv_ex;
+			if ((unk_ex->exception() >>= multi_serv_ex) == true)
+				m_ex = *multi_serv_ex;
+			else
+			{
+				unk_ex->exception() >>= serv_ex;
+				ex = *serv_ex;
+			}
 		}
 		
 		CORBA::NVList_ptr req_arg = req->arguments();
 		const Tango::AttributeValueList *att;
+		const Tango::AttributeValueList_4 *att_4;
+		unsigned int nb_att = 0;
 		CORBA::NamedValue_ptr nv = req_arg->item(0);
-		*(nv->value()) >>= att;	
+		
+		if (version < 4)
+		{
+			if ((*(nv->value()) >>= att) == true)
+				nb_att = att->length();
+		}
+		else
+		{
+			if ((*(nv->value()) >>= att_4) == true)
+				nb_att = att_4->length();
+		}
 		
 		TangoSys_OMemStream desc;
 		desc << "Failed to execute write_attributes_asynch on device " << device_name;
-		desc << "\nAttribute(s): ";
-		for (unsigned int i = 0;i < att->length();i++)
+		if (nb_att != 0)
 		{
-			desc << (*att)[i].name;
-			if (i != (att->length() - 1))
-				desc << ", ";
+			desc << "\nAttribute(s): ";
+			for (unsigned int i = 0;i < nb_att;i++)
+			{
+				if (version < 4)
+					desc << (*att)[i].name;
+				else
+					desc << (*att_4)[i].name;
+			
+				if (i != (nb_att - 1))
+					desc << ", ";
+			}
 		}
 		desc << ends;
 		
@@ -2178,6 +2318,13 @@ void DeviceProxy::write_attr_except(CORBA::Request_ptr req,long id,TgRequest::Re
 		}
 		else
 		{
+			if (serv_ex != NULL)
+			{
+				Except::re_throw_exception(ex,(const char*)"API_AttributeFailed",
+							   desc.str(),
+							   (const char*)"DeviceProxy::write_attributes_reply()");				
+			}
+			
 			if (req_type == TgRequest::WRITE_ATTR)
 				throw Tango::NamedDevFailedList(m_ex,
 					       			device_name,
@@ -2191,12 +2338,11 @@ void DeviceProxy::write_attr_except(CORBA::Request_ptr req,long id,TgRequest::Re
 //
 
 				Tango::DevFailed ex(m_ex.errors[0].err_list);
-                		Except::re_throw_exception(ex,(const char*)"API_AttributeFailed",
+				Except::re_throw_exception(ex,(const char*)"API_AttributeFailed",
                         				   desc.str(), (const char*)"DeviceProxy::write_attributes_reply()");
 			
 			}
 		}				
-	
 	}
 
 	CORBA::SystemException *sys_ex;
@@ -2211,8 +2357,19 @@ void DeviceProxy::write_attr_except(CORBA::Request_ptr req,long id,TgRequest::Re
 
 		CORBA::NVList_ptr req_arg = req->arguments();
 		const Tango::AttributeValueList *att;
+		const Tango::AttributeValueList *att_4;
+		unsigned int nb_att = 0;
 		CORBA::NamedValue_ptr nv = req_arg->item(0);
-		*(nv->value()) >>= att;
+		if (version < 4)
+		{
+			if ((*(nv->value()) >>= att) == true)
+				nb_att = att->length();
+		}
+		else
+		{
+			if ((*(nv->value()) >>= att_4) == true)
+				nb_att = att_4->length();
+		}
 
 		char *cb_excep_mess = Tango::Except::print_CORBA_SystemException(sys_ex);
 		
@@ -2231,17 +2388,24 @@ void DeviceProxy::write_attr_except(CORBA::Request_ptr req,long id,TgRequest::Re
 				ping();
 				return;
 			}
-			catch (Tango::DevFailed &e) {}
+			catch (Tango::DevFailed &) {}
 		}
 
 		TangoSys_OMemStream desc;
 		desc << "Failed to execute write_attributes_asynch on device " << device_name;
-		desc << "\nAttribute(s): ";
-		for (unsigned int i = 0;i < att->length();i++)
+		if (nb_att != 0)
 		{
-			desc << (*att)[i].name;
-			if (i != (att->length() - 1))
-				desc << ", ";
+			desc << "\nAttribute(s): ";
+			for (unsigned int i = 0;i < nb_att;i++)
+			{
+				if (version < 4)
+					desc << (*att)[i].name;
+				else
+					desc << (*att_4)[i].name;
+			
+				if (i != (nb_att - 1))
+					desc << ", ";
+			}
 		}
 		desc << ends;
 					
@@ -2421,7 +2585,7 @@ void DeviceProxy::redo_synch_write_call(TgRequest &req)
 
 //-----------------------------------------------------------------------------
 //
-// method : 		DeviceProxy::redo_synch_cmd()
+// method : 		Connection::redo_synch_cmd()
 // 
 // description : 	Redo a command_inout() call but synchronously
 //					This is needed for re-connection with asynchronous call
@@ -2469,5 +2633,40 @@ DeviceData Connection::redo_synch_cmd(TgRequest &req)
 	return command_inout(cmd_name,dd);
 }
 
-		
+//-----------------------------------------------------------------------------
+//
+// method : 		Connection::cancel_asynch_request()
+// 
+// description : 	Cancel a running asynchronous request
+//					This is a client side call. Obviously, the call cannot be aborted
+//					while it is running in the device.
+//
+// argin(s) :		id : The asynchronous call identifier
+//
+//-----------------------------------------------------------------------------
+
+void Connection::cancel_asynch_request(long id)
+{
+	ApiUtil::instance()->get_pasyn_table()->mark_as_cancelled(id);
+	pasyn_ctr--;
+}
+
+//-----------------------------------------------------------------------------
+//
+// method : 		Connection::cancel_all_polling_asynch_request()
+// 
+// description : 	Cancel all running asynchronous request
+//					This is a client side call. Obviously, the calls cannot be aborted
+//					while it is running in the device.
+//
+// argin(s) :		id : The asynchronous call identifier
+//
+//-----------------------------------------------------------------------------
+
+void Connection::cancel_all_polling_asynch_request()
+{
+	ApiUtil::instance()->get_pasyn_table()->mark_all_polling_as_cancelled();
+	pasyn_ctr = 0;
+}
+
 } // End of tango namespace
