@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: proxy_asyn_cb.cpp,v 3.8 2007/04/20 14:38:33 taurel Exp $\n$Name:  $";
+static const char *RcsId = "$Id: proxy_asyn_cb.cpp,v 3.18 2009/03/27 13:05:10 taurel Exp $\n$Name:  $";
 
 //
 // cpp 	- C++ source code file for TANGO api class and asynchronous callback
@@ -8,7 +8,67 @@ static const char *RcsId = "$Id: proxy_asyn_cb.cpp,v 3.8 2007/04/20 14:38:33 tau
 //
 // original 	- January 2003
 //
+// Copyright (C) :      2003,2004,2005,2006,2007,2008,2009
+//						European Synchrotron Radiation Facility
+//                      BP 220, Grenoble 38043
+//                      FRANCE
+//
+// This file is part of Tango.
+//
+// Tango is free software: you can redistribute it and/or modify
+// it under the terms of the GNU Lesser General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+// 
+// Tango is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU Lesser General Public License for more details.
+// 
+// You should have received a copy of the GNU Lesser General Public License
+// along with Tango.  If not, see <http://www.gnu.org/licenses/>.
+//
+//
 // $Log: proxy_asyn_cb.cpp,v $
+// Revision 3.18  2009/03/27 13:05:10  taurel
+// - Fix bug due to new Attribute format data member in AttributeValue4
+// structure
+//
+// Revision 3.17  2009/03/27 12:19:37  taurel
+// - Fix bug in read_attribute for IDL 4
+//
+// Revision 3.16  2009/03/13 09:32:27  taurel
+// - Small changes to fix Windows VC8 warnings in Warning level 3
+//
+// Revision 3.15  2009/01/21 12:45:15  taurel
+// - Change CopyRights for 2009
+//
+// Revision 3.14  2008/12/17 09:54:44  taurel
+// - First implementation of attributes sent on the wire using IDL Union
+// instead of IDL Any
+//
+// Revision 3.13  2008/10/06 15:02:17  taurel
+// - Changed the licensing info from GPL to LGPL
+//
+// Revision 3.12  2008/10/02 16:09:25  taurel
+// - Add some licensing information in each files...
+//
+// Revision 3.11  2008/09/23 14:38:29  taurel
+// - Commit after the end of DevEncoded data type implementation
+// - The new test suite is also now running fine
+//
+// Revision 3.10  2008/06/14 11:28:07  taurel
+// - DevEncoded attribute data type implementation work going on
+//
+// Revision 3.9  2008/05/20 12:42:30  taurel
+// - Commit after merge with release 7 branch
+//
+// Revision 3.8.2.2  2008/05/20 06:14:21  taurel
+// - Last commit before merge with trunk
+//
+// Revision 3.8.2.1  2008/02/07 15:56:58  taurel
+// - First implementation of the Controlled Access done
+//
 // Revision 3.8  2007/04/20 14:38:33  taurel
 // - Ported to Windows 64 bits x64 architecture
 //
@@ -163,15 +223,29 @@ void Connection::command_inout_asynch(const char *command, DeviceData &data_in,
 
 	CORBA::ORB::RequestSeq req_seq;
 	req_seq.length(1);
-	
-	if (version == 2)
+
+	if (version >= 4)
+		req_seq[0] = ext->device_4->_request("command_inout_4");
+	else if (version >= 2)
 		req_seq[0] = device_2->_request("command_inout_2");
 	else	
-		req_seq[0] = device->_request("command_inout");		
+		req_seq[0] = device->_request("command_inout");
+	
 	req_seq[0]->add_in_arg() <<= command;
 	req_seq[0]->add_in_arg() <<= data_in.any.in();
-	if (version == 2)
+
+	if (version >= 4)
+	{
+		ClntIdent ci;
+		ApiUtil *au = ApiUtil::instance();
+		ci.cpp_clnt(au->get_client_pid());
+		
 		req_seq[0]->add_in_arg() <<= source;
+		req_seq[0]->add_in_arg() <<= ci;
+	}
+	else if (version >= 2)
+		req_seq[0]->add_in_arg() <<= source;
+	
 	req_seq[0]->set_return_type(CORBA::_tc_any);
 	req_seq[0]->exceptions()->add(Tango::_tc_DevFailed);
 
@@ -548,6 +622,7 @@ void Connection::Cb_ReadAttr_Request(CORBA::Request_ptr req,Tango::CallBack *cb_
 
 		const Tango::AttributeValueList *received;
 		const Tango::AttributeValueList_3 *received_3;
+		const Tango::AttributeValueList_4 *received_4;
 		unsigned long nb_received;
 		
 		CORBA::Any &dii_any = req->return_value();
@@ -557,20 +632,26 @@ void Connection::Cb_ReadAttr_Request(CORBA::Request_ptr req,Tango::CallBack *cb_
 			dii_any >>= received;
 			nb_received = received->length();
 		}
-		else
+		else if (version == 3)
 		{
 			dii_any >>= received_3;
 			nb_received = received_3->length();
 		}
+		else
+		{
+			dii_any >>= received_4;
+			nb_received = received_4->length();
+		}
 		dev_attr->resize(nb_received);
 				
 		for (unsigned long i=0; i < nb_received; i++)
 		{
 			if (version >= 3)
 			{		
-				ApiUtil::attr_to_device(NULL,
-							&((*received_3)[i]),
-							version,&((*dev_attr)[i]));
+				if (version == 3)
+					ApiUtil::attr_to_device(NULL,&((*received_3)[i]),version,&((*dev_attr)[i]));
+				else
+					ApiUtil::attr_to_device(&((*received_4)[i]),version,&((*dev_attr)[i]));
 						
 				
 //
@@ -600,9 +681,7 @@ void Connection::Cb_ReadAttr_Request(CORBA::Request_ptr req,Tango::CallBack *cb_
 			}
 			else
 			{
-				ApiUtil::attr_to_device(&((*received)[i]),
-							NULL,
-							version,&((*dev_attr)[i]));
+				ApiUtil::attr_to_device(&((*received)[i]),NULL,version,&((*dev_attr)[i]));
 			}				
 		}
 	}
@@ -767,14 +846,23 @@ void Connection::Cb_ReadAttr_Request(CORBA::Request_ptr req,Tango::CallBack *cb_
 
 void Connection::Cb_WriteAttr_Request(CORBA::Request_ptr req,Tango::CallBack *cb_ptr)
 {
-	Tango::DevErrorList errors;
 	Tango::NamedDevFailedList err_3;
 
 	CORBA::NVList_ptr req_arg = req->arguments();
 	const Tango::AttributeValueList *att;
+	const Tango::AttributeValueList_4 *att_4;
 	CORBA::NamedValue_ptr nv = req_arg->item(0);
-	*(nv->value()) >>= att;
-	long nb_attr = att->length();
+	long nb_attr = 0;
+	if (version >= 4)
+	{
+		if ((*(nv->value()) >>= att_4) == true)
+			nb_attr = att_4->length();
+	}
+	else
+	{
+		if ((*(nv->value()) >>= att) == true)
+			nb_attr = att->length();
+	}
 					
 //
 // Check if the reply is an exception
@@ -807,12 +895,16 @@ void Connection::Cb_WriteAttr_Request(CORBA::Request_ptr req,Tango::CallBack *cb
 	
 				TangoSys_OMemStream desc;
 				desc << "Timeout (" << timeout << " mS) exceeded on device " << dev_name();
-				desc << "\nAttribute(s): ";
-				for (int i = 0;i < nb_attr;i++)
+				if (nb_attr != 0)
 				{
-					desc << (*att)[i].name.in();
-					if (i != (nb_attr - 1))
-						desc << ", ";
+					desc << "\nAttribute(s): ";
+					for (int i = 0;i < nb_attr;i++)
+					{
+						(version < 4) ? desc << (*att)[i].name : desc << (*att_4)[i].name;
+					
+						if (i != (nb_attr - 1))
+							desc << ", ";
+					}
 				}
 				desc << ends;
 
@@ -843,9 +935,9 @@ void Connection::Cb_WriteAttr_Request(CORBA::Request_ptr req,Tango::CallBack *cb
 // server has sent a DevFailed exception
 //
 	
-			const Tango::DevFailed *serv_ex;
+			const Tango::DevFailed *serv_ex = NULL;
 			Tango::DevFailed ex;
-			const Tango::MultiDevFailed *multi_serv_ex;
+			const Tango::MultiDevFailed *multi_serv_ex = NULL;
 			Tango::MultiDevFailed m_ex;
 			TangoSys_OMemStream desc;
 
@@ -855,7 +947,8 @@ void Connection::Cb_WriteAttr_Request(CORBA::Request_ptr req,Tango::CallBack *cb
 				ex = *serv_ex;
 				
 				desc << "Failed to execute write_attributes_asynch on device " << dev_name();
-				desc << "\nAttribute(s): ";
+				if (nb_attr != 0)
+					desc << "\nAttribute(s): ";
 				for (int i = 0;i < nb_attr;i++)
 				{
 					desc << (*att)[i].name.in();
@@ -884,27 +977,50 @@ void Connection::Cb_WriteAttr_Request(CORBA::Request_ptr req,Tango::CallBack *cb
 			}
 			else
 			{
-				unk_ex->exception() >>= multi_serv_ex;
-				m_ex = *multi_serv_ex;
-				err_3 = Tango::NamedDevFailedList(m_ex,
+				if ((unk_ex->exception() >>= multi_serv_ex) == true)
+				{
+					m_ex = *multi_serv_ex;
+					err_3 = Tango::NamedDevFailedList(m_ex,
 					       dev_name(),
 					       (const char *)"Connection::Cb_WriteAttr_Request()",
 					       (const char *)"API_AttributeFailed");
-			}			
-	
-			long nb_err = errors.length();
-			errors.length(nb_err + 1);
-			errors[nb_err].severity = Tango::ERR;
+				}
+				else
+				{
+					unk_ex->exception() >>= serv_ex;
+					ex = *serv_ex;
+					
+					desc << "Failed to execute write_attributes_asynch on device " << dev_name();
+					if (nb_attr != 0)
+						desc << "\nAttribute(s): ";
+					for (int i = 0;i < nb_attr;i++)
+					{
+						(version < 4) ? desc << (*att)[i].name : desc << (*att_4)[i].name;
+						
+						if (i != (nb_attr - 1))
+							desc << ", ";
+					}
+					desc << ends;
+
+					err_3.errors.length(1);
+					err_3.errors[0].severity = Tango::ERR;
 #if ((defined _TG_WINDOWS_) || (defined __SUNPRO_CC) || (defined GCC_STD))
-			string st = desc.str();
-			errors[nb_err].desc = CORBA::string_dup(st.c_str());
+					string st = desc.str();
+					err_3.errors[0].desc = CORBA::string_dup(st.c_str());
 #else
-			char *ptr = desc.str();
-			errors[nb_err].desc = CORBA::string_dup(ptr);
-			delete[] ptr;
+					char *ptr = desc.str();
+					err_3.errors[0].desc = CORBA::string_dup(ptr);
+					delete[] ptr;
 #endif
-			errors[nb_err].origin = CORBA::string_dup("Connection::Cb_WriteAttr_Request()");
-			errors[nb_err].reason = CORBA::string_dup("API_AttributeFailed");	
+					err_3.errors[0].origin = CORBA::string_dup("Connection::Cb_WriteAttr_Request()");
+					err_3.errors[0].reason = CORBA::string_dup("API_AttributeFailed");
+
+					err_3.err_list.resize(1);
+					err_3.err_list[0].err_stack = serv_ex->errors;
+					err_3.err_list[0].name = (*att)[0].name.in();				
+					err_3.err_list[0].idx_in_call = 0;				
+				}
+			}			
 		}
 		else if (((sys_ex = CORBA::SystemException::_downcast(ex_ptr)) != NULL) &&
 			 (to_except == false))
@@ -920,10 +1036,12 @@ void Connection::Cb_WriteAttr_Request(CORBA::Request_ptr req,Tango::CallBack *cb
 
 			TangoSys_OMemStream desc;
 			desc << "Failed to execute write_attributes_asynch on device " << dev_name();
-			desc << "\nAttribute(s): ";
+			if (nb_attr != 0)
+				desc << "\nAttribute(s): ";
 			for (int i = 0;i < nb_attr;i++)
 			{
-				desc << (*att)[i].name.in();
+				(version < 4) ? desc << (*att)[i].name : desc << (*att_4)[i].name;
+
 				if (i != (nb_attr - 1))
 					desc << ", ";
 			}
@@ -953,8 +1071,16 @@ void Connection::Cb_WriteAttr_Request(CORBA::Request_ptr req,Tango::CallBack *cb
 //
 		
 	vector<string> att_name;
-	for (int i = 0;i < nb_attr;i++)
-		att_name.push_back((*att)[i].name.in());
+	if (version >= 4)
+	{
+		for (int i = 0;i < nb_attr;i++)
+			att_name.push_back((*att_4)[i].name.in());
+	}
+	else
+	{
+		for (int i = 0;i < nb_attr;i++)
+			att_name.push_back((*att)[i].name.in());
+	}		
 
 	DeviceProxy *local_dev = static_cast<DeviceProxy *>(this);							
 	AttrWrittenEvent *cb_data = new AttrWrittenEvent(local_dev,att_name,err_3);
@@ -1194,7 +1320,19 @@ void DeviceProxy::read_attributes_asynch(vector<string> &attr_names,CallBack &cb
 	for (int i = 0;i < nb_names;i++)
 		names[i] = attr_names[i].c_str();
 
-	if (version == 3)
+	if (version == 4)
+	{
+		ClntIdent ci;
+		ApiUtil *au = ApiUtil::instance();
+		ci.cpp_clnt(au->get_client_pid());
+		
+		req_seq[0] = Connection::ext->device_4->_request("read_attributes_4");
+		req_seq[0]->add_in_arg() <<= names;
+		req_seq[0]->add_in_arg() <<= source;
+		req_seq[0]->add_in_arg() <<= ci;
+		req_seq[0]->set_return_type(Tango::_tc_AttributeValueList_4);
+	}
+	else if (version == 3)
 	{
 		req_seq[0] = Connection::ext->device_3->_request("read_attributes_3");
 		req_seq[0]->add_in_arg() <<= names;
@@ -1279,70 +1417,45 @@ void DeviceProxy::write_attributes_asynch(vector<DeviceAttribute> &attr_list,
 	
 
 	Tango::AttributeValueList att;
+	Tango::AttributeValueList_4 att_4;
+
 	long nb_attr = attr_list.size();
-	att.length(nb_attr);
+	if (version < 4)
+		att.length(nb_attr);
+	else
+		att_4.length(nb_attr);
 
 	for (int i = 0;i < nb_attr;i++)
 	{
-		att[i].name = attr_list[i].name.c_str();
-		att[i].quality = attr_list[i].quality;
-		att[i].time = attr_list[i].time;
-		att[i].dim_x = attr_list[i].dim_x;
-		att[i].dim_y = attr_list[i].dim_y;		
-
-		if (attr_list[i].LongSeq.operator->() != NULL)
-		{
-			att[i].value <<= attr_list[i].LongSeq.in();
-			continue;
-		}
-		if (attr_list[i].ShortSeq.operator->() != NULL)
-		{	
-			att[i].value <<= attr_list[i].ShortSeq.in();
-			continue;
-		}	
-		if (attr_list[i].DoubleSeq.operator->() != NULL)
-		{	
-			att[i].value <<= attr_list[i].DoubleSeq.in();
-			continue;
-		}
-		if (attr_list[i].StringSeq.operator->() != NULL)
-		{	
-			att[i].value  <<= attr_list[i].StringSeq.in();
-			continue;
-		}
-		if (attr_list[i].FloatSeq.operator->() != NULL)
-		{
-			att[i].value <<= attr_list[i].FloatSeq.in();
-			continue;
-		}
-		if (attr_list[i].BooleanSeq.operator->() != NULL)
-		{	
-			att[i].value <<= attr_list[i].BooleanSeq.in();
-			continue;
-		}	
-		if (attr_list[i].UShortSeq.operator->() != NULL)
-		{	
-			att[i].value <<= attr_list[i].UShortSeq.in();
-			continue;
-		}
-		if (attr_list[i].UCharSeq.operator->() != NULL)
-		{	
-			att[i].value  <<= attr_list[i].UCharSeq.in();
-			continue;
-		}
+		if (version < 4)
+			ApiUtil::device_to_attr(attr_list[i],att[i],device_name);
+		else
+			ApiUtil::device_to_attr(attr_list[i],att_4[i]);
 	}
 
-	if (version < 3)
-	{			
-		req_seq[0] = device->_request("write_attributes");
-		req_seq[0]->exceptions()->add(Tango::_tc_DevFailed);
+	if (version >= 4)
+	{
+		ClntIdent ci;
+		ApiUtil *au = ApiUtil::instance();
+		ci.cpp_clnt(au->get_client_pid());
+		
+		req_seq[0] = ext->device_4->_request("write_attributes_4");
+		req_seq[0]->add_in_arg() <<= att_4;
+		req_seq[0]->add_in_arg() <<= ci;
+		req_seq[0]->exceptions()->add(Tango::_tc_MultiDevFailed);
+	}
+	else if (version == 3)
+	{		
+		req_seq[0] = device->_request("write_attributes_3");
+		req_seq[0]->add_in_arg() <<= att;
+		req_seq[0]->exceptions()->add(Tango::_tc_MultiDevFailed);
 	}
 	else
 	{
-		req_seq[0] = device->_request("write_attributes_3");
-		req_seq[0]->exceptions()->add(Tango::_tc_MultiDevFailed);
+		req_seq[0] = device->_request("write_attributes");
+		req_seq[0]->add_in_arg() <<= att;
 	}
-	req_seq[0]->add_in_arg() <<= att;
+	req_seq[0]->exceptions()->add(Tango::_tc_DevFailed);
 	
 //
 // Send the request
@@ -1408,58 +1521,42 @@ void DeviceProxy::write_attribute_asynch(DeviceAttribute &attr,CallBack &cb)
 	
 
 	Tango::AttributeValueList att;
-	att.length(1);
+	Tango::AttributeValueList_4 att_4;
 
-	att[0].name = attr.name.c_str();
-	att[0].quality = attr.quality;
-	att[0].time = attr.time;
-	att[0].dim_x = attr.dim_x;
-	att[0].dim_y = attr.dim_y;		
-
-	if (attr.LongSeq.operator->() != NULL)
+	if (version < 4)
 	{
-		att[0].value <<= attr.LongSeq.in();
+		att.length(1);
+		ApiUtil::device_to_attr(attr,att[0],device_name);
 	}
-	else if (attr.ShortSeq.operator->() != NULL)
+	else
 	{	
-		att[0].value <<= attr.ShortSeq.in();
-	}	
-	else if (attr.DoubleSeq.operator->() != NULL)
-	{	
-		att[0].value <<= attr.DoubleSeq.in();
-	}
-	else if (attr.StringSeq.operator->() != NULL)
-	{	
-		att[0].value  <<= attr.StringSeq.in();
-	}
-	else if (attr.FloatSeq.operator->() != NULL)
-	{	
-		att[0].value  <<= attr.FloatSeq.in();
-	}
-	else if (attr.BooleanSeq.operator->() != NULL)
-	{	
-		att[0].value  <<= attr.BooleanSeq.in();
-	}
-	else if (attr.UShortSeq.operator->() != NULL)
-	{	
-		att[0].value  <<= attr.UShortSeq.in();
-	}
-	else if (attr.UCharSeq.operator->() != NULL)
-	{	
-		att[0].value  <<= attr.UCharSeq.in();
+		att_4.length(1);
+		ApiUtil::device_to_attr(attr,att_4[0]);
 	}
 
-	if (version < 3)
-	{			
-		req_seq[0] = device->_request("write_attributes");
-		req_seq[0]->exceptions()->add(Tango::_tc_DevFailed);
+	if (version >= 4)
+	{
+		ClntIdent ci;
+		ApiUtil *au = ApiUtil::instance();
+		ci.cpp_clnt(au->get_client_pid());
+		
+		req_seq[0] = ext->device_4->_request("write_attributes_4");
+		req_seq[0]->add_in_arg() <<= att_4;
+		req_seq[0]->add_in_arg() <<= ci;
+		req_seq[0]->exceptions()->add(Tango::_tc_MultiDevFailed);
+	}
+	else if (version == 3)
+	{
+		req_seq[0] = device->_request("write_attributes_3");
+		req_seq[0]->add_in_arg() <<= att;
+		req_seq[0]->exceptions()->add(Tango::_tc_MultiDevFailed);
 	}
 	else
 	{
-		req_seq[0] = device->_request("write_attributes_3");
-		req_seq[0]->exceptions()->add(Tango::_tc_MultiDevFailed);
+		req_seq[0] = device->_request("write_attributes");
+		req_seq[0]->add_in_arg() <<= att;
 	}
-	req_seq[0]->add_in_arg() <<= att;
+	req_seq[0]->exceptions()->add(Tango::_tc_DevFailed);
 	
 //
 // Send the request
