@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: attrdesc.cpp,v 3.9 2007/04/16 14:56:36 taurel Exp $\n$Name:  $";
+static const char *RcsId = "$Id: attrdesc.cpp,v 3.14 2009/01/21 12:49:04 taurel Exp $\n$Name:  $";
 
 //+============================================================================
 //
@@ -16,9 +16,48 @@ static const char *RcsId = "$Id: attrdesc.cpp,v 3.9 2007/04/16 14:56:36 taurel E
 //
 // author(s) :          A.Gotz + E.Taurel
 //
-// $Revision: 3.9 $
+// Copyright (C) :      2004,2005,2006,2007,2008,2009
+//						European Synchrotron Radiation Facility
+//                      BP 220, Grenoble 38043
+//                      FRANCE
+//
+// This file is part of Tango.
+//
+// Tango is free software: you can redistribute it and/or modify
+// it under the terms of the GNU Lesser General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+// 
+// Tango is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU Lesser General Public License for more details.
+// 
+// You should have received a copy of the GNU Lesser General Public License
+// along with Tango.  If not, see <http://www.gnu.org/licenses/>.
+//
+// $Revision: 3.14 $
 //
 // $Log: attrdesc.cpp,v $
+// Revision 3.14  2009/01/21 12:49:04  taurel
+// - Change CopyRights for 2009
+//
+// Revision 3.13  2008/10/06 15:00:36  taurel
+// - Changed the licensing info from GPL to LGPL
+//
+// Revision 3.12  2008/10/03 06:51:36  taurel
+// - Add some licensing info in each files
+//
+// Revision 3.11  2008/06/14 11:29:15  taurel
+// - DevEncoded attribute data type implementation work going on
+//
+// Revision 3.10  2008/05/20 12:44:09  taurel
+// - Commit after merge with release 7 branch
+//
+// Revision 3.9.2.1  2008/05/20 06:17:44  taurel
+// - Last commit before merge with trunk
+// (start the implementation of the new DevEncoded data type)
+//
 // Revision 3.9  2007/04/16 14:56:36  taurel
 // - Added 3 new attributes data types (DevULong, DevULong64 and DevState)
 // - Ported to omniORB4.1
@@ -169,11 +208,6 @@ static const char *RcsId = "$Id: attrdesc.cpp,v 3.9 2007/04/16 14:56:36 taurel E
 // Revision 1.1.1.1  2001/02/27 08:46:20  taurel
 // Imported sources
 //
-//
-// copyleft :           European Synchrotron Radiation Facility
-//                      BP 220, Grenoble 38043
-//                      FRANCE
-//
 //-============================================================================
 
 #if HAVE_CONFIG_H
@@ -320,6 +354,8 @@ void Attr::check_type()
 		unsuported = false;
 	else if (type == Tango::DEV_STATE)
 		unsuported = false;
+	else if (type == Tango::DEV_ENCODED)
+		unsuported = false;
 
 	if (unsuported == true)
 	{
@@ -403,7 +439,7 @@ void Attr::set_memorized()
 				      (const char *)"Attr::set_memorized");
 	}
 
-	if (type == DEV_STATE)
+	if ((type == DEV_STATE) || (type == DEV_ENCODED))
 	{
 		cout3 << "Attr::set_memorized() throwing exception" << endl;
 		TangoSys_OMemStream o;
@@ -456,6 +492,17 @@ SpectrumAttr::SpectrumAttr(const char *att_name,long att_type,long x)
 				      o.str(),
 				      (const char *)"SpectrumAttr::SpectrumAttr");
 	}
+	
+	if (type == DEV_ENCODED)
+	{
+		cout3 << "SpectrumAttr::SpectrumAttr throwing exception" << endl;
+		TangoSys_OMemStream o;
+		
+		o << "Attribute: " << name << ": ";
+		o << "DevEncode data type allowed only for SCALAR attribute" << ends;
+		Except::throw_exception((const char *)"API_AttrWrongDefined",o.str(),
+							(const char *)"SpectrumAttr::SpectrumAttr");
+	}
 	max_x = x;	
 }
 
@@ -474,6 +521,17 @@ SpectrumAttr::SpectrumAttr(const char *att_name,long att_type,Tango::AttrWriteTy
 				      o.str(),
 				      (const char *)"SpectrumAttr::SpectrumAttr");
 	}
+	
+	if (type == DEV_ENCODED)
+	{
+		cout3 << "SpectrumAttr::SpectrumAttr throwing exception" << endl;
+		TangoSys_OMemStream o;
+		
+		o << "Attribute: " << name << ": ";
+		o << "DevEncode data type allowed only for SCALAR attribute" << ends;
+		Except::throw_exception((const char *)"API_AttrWrongDefined",o.str(),
+							(const char *)"SpectrumAttr::SpectrumAttr");
+	}
 	max_x = x;	
 }
 
@@ -492,6 +550,17 @@ SpectrumAttr::SpectrumAttr(const char *att_name,long att_type,long x,DispLevel l
 				      o.str(),
 				      (const char *)"SpectrumAttr::SpectrumAttr");
 	}
+	
+	if (type == DEV_ENCODED)
+	{
+		cout3 << "SpectrumAttr::SpectrumAttr throwing exception" << endl;
+		TangoSys_OMemStream o;
+		
+		o << "Attribute: " << name << ": ";
+		o << "DevEncode data type allowed only for SCALAR attribute" << ends;
+		Except::throw_exception((const char *)"API_AttrWrongDefined",o.str(),
+							(const char *)"SpectrumAttr::SpectrumAttr");
+	}
 	max_x = x;	
 }
 
@@ -510,6 +579,17 @@ SpectrumAttr::SpectrumAttr(const char *att_name,long att_type,Tango::AttrWriteTy
 				      o.str(),
 				      (const char *)"SpectrumAttr::SpectrumAttr");
 	}
+
+	if (type == DEV_ENCODED)
+	{
+		cout3 << "SpectrumAttr::SpectrumAttr throwing exception" << endl;
+		TangoSys_OMemStream o;
+		
+		o << "Attribute: " << name << ": ";
+		o << "DevEncode data type allowed only for SCALAR attribute" << ends;
+		Except::throw_exception((const char *)"API_AttrWrongDefined",o.str(),
+							(const char *)"SpectrumAttr::SpectrumAttr");
+	}
 	max_x = x;	
 }
 
