@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: deviceclass.cpp,v 1.26 2008/03/11 14:38:25 taurel Exp $\n$Name:  $";
+static const char *RcsId = "$Id: deviceclass.cpp,v 1.32 2009/01/21 12:49:04 taurel Exp $\n$Name:  $";
 
 //+============================================================================
 //
@@ -13,9 +13,48 @@ static const char *RcsId = "$Id: deviceclass.cpp,v 1.26 2008/03/11 14:38:25 taur
 //
 // author(s) :		E.Taurel
 //
-// $Revision: 1.26 $
+// Copyright (C) :      2004,2005,2006,2007,2008,2009
+//						European Synchrotron Radiation Facility
+//                      BP 220, Grenoble 38043
+//                      FRANCE
+//
+// This file is part of Tango.
+//
+// Tango is free software: you can redistribute it and/or modify
+// it under the terms of the GNU Lesser General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+// 
+// Tango is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU Lesser General Public License for more details.
+// 
+// You should have received a copy of the GNU Lesser General Public License
+// along with Tango.  If not, see <http://www.gnu.org/licenses/>.
+//
+// $Revision: 1.32 $
 //
 // $Log: deviceclass.cpp,v $
+// Revision 1.32  2009/01/21 12:49:04  taurel
+// - Change CopyRights for 2009
+//
+// Revision 1.31  2008/10/06 15:00:36  taurel
+// - Changed the licensing info from GPL to LGPL
+//
+// Revision 1.30  2008/10/03 06:51:36  taurel
+// - Add some licensing info in each files
+//
+// Revision 1.29  2008/10/02 09:09:47  taurel
+// - First implementation of multiple polling thread(s)
+//
+// Revision 1.28  2008/09/04 07:37:05  taurel
+// - Fix bug in memorized attributes
+// - Changes for the new IDL 4
+//
+// Revision 1.27  2008/05/20 12:44:11  taurel
+// - Commit after merge with release 7 branch
+//
 // Revision 1.26  2008/03/11 14:38:25  taurel
 // - Apply patches from Frederic Picca about compilation with gcc 4.2
 //
@@ -25,6 +64,8 @@ static const char *RcsId = "$Id: deviceclass.cpp,v 1.26 2008/03/11 14:38:25 taur
 //
 // Revision 1.24  2007/12/12 10:17:18  taurel
 // - Db calls during DS startup has a separate timeout and some retries
+// Revision 1.23.2.1  2008/02/07 15:58:14  taurel
+// - First implementation of the Controlled Access done
 //
 // Revision 1.23  2007/11/08 12:03:44  taurel
 // - Start implementing user interceptors
@@ -119,11 +160,6 @@ static const char *RcsId = "$Id: deviceclass.cpp,v 1.26 2008/03/11 14:38:25 taur
 // Splitted device.cpp/device.h files in two sets device.cpp/device.h for the DeviceImpl
 // class and deviceclass.cpp/deviceclass.h for the DeviceClass class
 //
-//
-// copyleft :		European Synchrotron Radiation Facility
-//			BP 220, Grenoble 38043
-//			FRANCE
-//
 //-============================================================================
 
 #if HAVE_CONFIG_H
@@ -149,6 +185,10 @@ extern omni_thread::key_t key_py_data;
 namespace Tango
 {
 
+static void lower_cmd_name(string &cmd)
+{
+	transform(cmd.begin(),cmd.end(),cmd.begin(),::tolower);
+}
 
 //+----------------------------------------------------------------------------
 //
@@ -243,6 +283,7 @@ void DeviceClass::get_class_system_resource()
 		db_data.push_back(DbDatum("doc_url"));
 		db_data.push_back(DbDatum("cvs_tag"));
 		db_data.push_back(DbDatum("cvs_location"));
+		db_data.push_back(DbDatum("AllowedAccessCmd"));
 
 		try
 		{
@@ -262,6 +303,16 @@ void DeviceClass::get_class_system_resource()
 			db_data[1] >> ext->cvs_tag;
 		if (db_data[2].is_empty() == false)
 			db_data[2] >> ext->cvs_location;
+		
+//
+// Init allowed commands vector (in lowercase letters)
+//
+			
+		if (db_data[3].is_empty() == false)
+		{
+			db_data[3] >> allowed_cmds;
+			for_each(allowed_cmds.begin(),allowed_cmds.end(),lower_cmd_name);
+		}
 					
 		if (db_data[0].is_empty() == true)
 		{
@@ -289,7 +340,6 @@ void DeviceClass::get_class_system_resource()
 		}
 		else
 			db_data[0] >> doc_url;
-			
 		
 	}
 	else
@@ -313,7 +363,7 @@ void DeviceClass::get_class_system_resource()
 //
 //-----------------------------------------------------------------------------
 
-void DeviceClass::set_memorized_values(bool all,long idx)
+void DeviceClass::set_memorized_values(bool all,long idx,bool from_init)
 {
 	cout4 << "Entering DeviceClass::set_memorized_values() method" << endl;
 	
@@ -328,7 +378,8 @@ void DeviceClass::set_memorized_values(bool all,long idx)
 	double db;
 	Tango::DevVarDoubleArray db_seq(1);
 	db_seq.length(1);
-						
+
+	Tango::DevString tmp_str;
 	Tango::DevVarStringArray str_seq(1);
 	str_seq.length(1);
 
@@ -392,7 +443,7 @@ void DeviceClass::set_memorized_values(bool all,long idx)
 			{
 
 				string &mem_value = att.get_mem_value();
-				if (mem_value != MemNotUsed)
+				if ((mem_value != MemNotUsed) || (from_init == true))
 				{
 					nb_wr++;
 					att_val.length(nb_wr);
@@ -411,96 +462,140 @@ void DeviceClass::set_memorized_values(bool all,long idx)
 //
 
 					TangoSys_MemStream str;
-					str << mem_value << ends;
+					if (from_init == false)
+						str << mem_value << ends;
 					
 					try
 					{
 						switch (att.get_data_type())
 						{
 						case Tango::DEV_SHORT:
-							if (!(str >> sh))
-								throw_mem_value(device_list[i],att);
-							att.set_write_value(sh);
+							if (from_init == false)
+							{
+								if (!(str >> sh))
+									throw_mem_value(device_list[i],att);
+								att.set_write_value(sh);
+							}
+							else
+								att.get_write_value(sh);
 							
 							sh_seq[0] = sh;
 							att_val[nb_wr - 1].value <<= sh_seq;
 							break;
 
 						case Tango::DEV_LONG:
-							if (!(str >> lg))
-								throw_mem_value(device_list[i],att);
-							att.set_write_value(lg);	
+							if (from_init == false)
+							{
+								if (!(str >> lg))
+									throw_mem_value(device_list[i],att);
+								att.set_write_value(lg);
+							}
+							else
+								att.get_write_value(lg);
 								
 							lg_seq[0] = lg;
 							att_val[nb_wr - 1].value <<= lg_seq;
 							break;
 
 						case Tango::DEV_DOUBLE:
-							if (!(str >> db))
-								throw_mem_value(device_list[i],att);
-							att.set_write_value(db);
+							if (from_init == false)
+							{
+								if (!(str >> db))
+									throw_mem_value(device_list[i],att);
+								att.set_write_value(db);
+							}
+							else
+								att.get_write_value(db);
 							
 							db_seq[0] = db;
 							att_val[nb_wr - 1].value <<= db_seq;
 							break;
 
 						case Tango::DEV_STRING:
-							att.set_write_value(mem_value);
+							if (from_init == false)
+							{
+								att.set_write_value(mem_value);
 							
-							str_seq[0] = CORBA::string_dup(mem_value.c_str());
+								str_seq[0] = CORBA::string_dup(mem_value.c_str());
+							}
+							else
+							{
+								att.get_write_value(tmp_str);
+								str_seq[0] = CORBA::string_dup(tmp_str);
+							}
 							att_val[nb_wr - 1].value <<= str_seq;
 							break;
 
 						case Tango::DEV_FLOAT:
-							if (!(str >> fl))
-								throw_mem_value(device_list[i],att);
-							att.set_write_value(fl);	
+							if (from_init == false)
+							{
+								if (!(str >> fl))
+									throw_mem_value(device_list[i],att);
+								att.set_write_value(fl);
+							}
+							else
+								att.get_write_value(fl);
 								
 							fl_seq[0] = fl;
 							att_val[nb_wr - 1].value <<= fl_seq;
 							break;
 
 						case Tango::DEV_BOOLEAN:
+							if (from_init == false)
+							{
 #if ((defined _TG_WINDOWS_) || (defined __SUNPRO_CC) || (defined GCC_STD))
-							if (!(str >> boolalpha >> boo))
-								throw_mem_value(device_list[i],att);
+								if (!(str >> boolalpha >> boo))
+									throw_mem_value(device_list[i],att);
 #else
-							transform(mem_value.begin(),mem_value.end(),mem_value.begin(),::tolower);
-							if (mem_value == "true")
-								boo = true;
-							else if (mem_value == "false")
-								boo = false;
-							else
-								throw_mem_value(device_list[i],att);
+								transform(mem_value.begin(),mem_value.end(),mem_value.begin(),::tolower);
+								if (mem_value == "true")
+									boo = true;
+								else if (mem_value == "false")
+									boo = false;
+								else
+									throw_mem_value(device_list[i],att);
 #endif
-							att.set_write_value(boo);
+								att.set_write_value(boo);
+							}
+							else
+								att.get_write_value(boo);
 							
 							boo_seq[0] = boo;
 							att_val[nb_wr - 1].value <<= boo_seq;
 							break;
 
 						case Tango::DEV_USHORT:
-							if (!(str >> ush))
-								throw_mem_value(device_list[i],att);
-							att.set_write_value(ush);	
+							if (from_init == false)
+							{
+								if (!(str >> ush))
+									throw_mem_value(device_list[i],att);
+								att.set_write_value(ush);
+							}
+							else
+								att.get_write_value(ush);
 								
 							ush_seq[0] = ush;
 							att_val[nb_wr - 1].value <<= ush_seq;
 							break;
 
 						case Tango::DEV_UCHAR:
-							if (!(str >> uch))
-								throw_mem_value(device_list[i],att);
-							att.set_write_value(uch);
+							if (from_init == false)
+							{
+								if (!(str >> uch))
+									throw_mem_value(device_list[i],att);
+								att.set_write_value(uch);
+							}
+							else
+								att.get_write_value(uch);
 								
 							uch_seq[0] = uch;
 							att_val[nb_wr - 1].value <<= uch_seq;
 							break;
 						}
 						
-					// Check the initialisation flag for memorized attributes.
-					// The the flag is false, do not add the element to the att_val
-					// vector. This avoids a call to write the memorozied value to the attribute.
+// Check the initialisation flag for memorized attributes.
+// The the flag is false, do not add the element to the att_val
+// vector. This avoids a call to write the memorozied value to the attribute.
 					
 					if ( att.is_memorized_init() == false )
 						{
@@ -511,9 +606,9 @@ void DeviceClass::set_memorized_values(bool all,long idx)
 						}
 					else
 						{
-						//
-						// Init the AttributeValue structure
-						//
+//
+// Init the AttributeValue structure
+//
 					
 						att_val[nb_wr - 1].name = CORBA::string_dup(att.get_name().c_str());
 						att_val[nb_wr - 1].dim_x = 1;
@@ -537,15 +632,13 @@ void DeviceClass::set_memorized_values(bool all,long idx)
 		}
 			
 																	
-
-
 		if (nb_wr != 0)
 		{
 		
-		//
-		// Write attribute values. Re-throw exception if any but reset memorized flags before the
-		// re-throw.
-		//
+//
+// Write attribute values. Re-throw exception if any but reset memorized flags before the
+// re-throw.
+//
 			
 			try
 			{
@@ -582,9 +675,9 @@ void DeviceClass::set_memorized_values(bool all,long idx)
 				//	       			(const char *)"API_AttributeFailed");
 			}
 						
-			//
-			// Reset memorized flags
-			//
+//
+// Reset memorized flags
+//
 
 			for (unsigned long k = 0;k < att_val.length();k++)
 			{
@@ -736,8 +829,10 @@ void DeviceClass::delete_dev(long idx,Tango::Util *tg,PortableServer::POA_ptr r_
 // ask polling thread to stop polling
 //
 
-	if ((tg->get_polling_thread_object() != NULL) && (device_list[idx]->is_polled() == true))
-		device_list[idx]->stop_polling();
+	if ((tg->get_polling_threads_info().empty() == false) && (device_list[idx]->is_polled() == true))
+	{
+		device_list[idx]->stop_polling(false);
+	}
 	
 //
 // Deactivate the CORBA object
@@ -1362,4 +1457,31 @@ void DeviceClass::device_destroyer(const char *dev_name)
 	return device_destroyer(name_str);
 }
 
+//+----------------------------------------------------------------------------
+//
+// method :		DeviceClass::is_command_allowed
+// 
+// description :	Method to check if a command is allowed even if the device 
+//					is locked by another client. It follows the definition
+//					of the Tango control access system to define what is an
+//					allowed command
+//
+// in : 	cmd : The command name
+//
+//-----------------------------------------------------------------------------
+
+bool DeviceClass::is_command_allowed(const char *cmd)
+{
+	bool ret = true;
+	
+	string tmp_cmd(cmd);
+	transform(tmp_cmd.begin(),tmp_cmd.end(),tmp_cmd.begin(),::tolower);
+
+	vector<string>::iterator pos = find(allowed_cmds.begin(),allowed_cmds.end(),tmp_cmd);
+	if (pos == allowed_cmds.end())
+		ret = false;
+	
+	return ret;
+}
+
 } // End of Tango namespace
