@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: dserver.cpp,v 3.29 2008/01/25 15:44:51 taurel Exp $\n$Name:  $";
+static const char *RcsId = "$Id: dserver.cpp,v 3.39 2009/03/23 15:21:40 taurel Exp $\n$Name:  $";
 
 //+=============================================================================
 //
@@ -14,9 +14,61 @@ static const char *RcsId = "$Id: dserver.cpp,v 3.29 2008/01/25 15:44:51 taurel E
 //
 // author(s) :          A.Gotz + E.Taurel
 //
-// $Revision: 3.29 $
+// Copyright (C) :      2004,2005,2006,2007,2008,2009
+//						European Synchrotron Radiation Facility
+//                      BP 220, Grenoble 38043
+//                      FRANCE
+//
+// This file is part of Tango.
+//
+// Tango is free software: you can redistribute it and/or modify
+// it under the terms of the GNU Lesser General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+// 
+// Tango is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU Lesser General Public License for more details.
+// 
+// You should have received a copy of the GNU Lesser General Public License
+// along with Tango.  If not, see <http://www.gnu.org/licenses/>.
+//
+// $Revision: 3.39 $
 //
 // $Log: dserver.cpp,v $
+// Revision 3.39  2009/03/23 15:21:40  taurel
+// - Forgot to remove some commented code when debugging  on Windows Vista
+//
+// Revision 3.38  2009/03/13 09:33:29  taurel
+// - Small changes to fix Windows VC8 warnings in Warning level 3
+//
+// Revision 3.37  2009/02/03 15:15:08  jensmeyer
+// Added a QuerySubDevice command to read the list of opened sub device
+// connections.
+//
+// Revision 3.36  2009/01/21 12:49:04  taurel
+// - Change CopyRights for 2009
+//
+// Revision 3.35  2008/11/18 09:28:56  taurel
+// - Ported to gcc 4.3
+// - Removed some cout messages
+//
+// Revision 3.34  2008/10/06 15:01:09  taurel
+// - Changed the licensing info from GPL to LGPL
+//
+// Revision 3.33  2008/10/03 06:52:31  taurel
+// - Add some licensing info in each files
+//
+// Revision 3.32  2008/10/02 12:24:43  taurel
+// - The user now has the possibility to install its own event loop
+//
+// Revision 3.31  2008/10/02 09:09:47  taurel
+// - First implementation of multiple polling thread(s)
+//
+// Revision 3.30  2008/05/20 12:44:11  taurel
+// - Commit after merge with release 7 branch
+//
 // Revision 3.29  2008/01/25 15:44:51  taurel
 // - Some changes in the Db cache
 // - A lighter system to shutdown DS in case of dynamic attribute
@@ -29,9 +81,19 @@ static const char *RcsId = "$Id: dserver.cpp,v 3.29 2008/01/25 15:44:51 taurel E
 // is also used as an event client.
 // Added a EventConsumer::disconnect_from_notifd method as for
 // the EventSupplier.
+// Revision 3.25.2.4  2008/01/03 16:05:52  taurel
+// - Some changes in locking feature implementation
 //
 // Revision 3.26  2007/12/19 15:42:43  taurel
 // - Add some cleanup when quitting applis or device server (notifd objects)
+// Revision 3.25.2.3  2007/12/20 14:29:01  taurel
+// - Some more work on locking
+//
+// Revision 3.25.2.2  2007/12/19 15:54:47  taurel
+// - Still some work going on for the locking feature
+//
+// Revision 3.25.2.1  2007/11/22 12:33:10  taurel
+// - First part of the device locking implementation
 //
 // Revision 3.25  2007/10/26 11:31:52  taurel
 // - Simply remove some log message sin source code
@@ -122,12 +184,6 @@ static const char *RcsId = "$Id: dserver.cpp,v 3.29 2008/01/25 15:44:51 taurel E
 // Revision 3.9.2.3  2004/09/27 09:09:07  taurel
 // - Changes to allow reading state and/or status as attributes
 //
-//
-//
-// copyleft :           European Synchrotron Radiation Facility
-//                      BP 220, Grenoble 38043
-//                      FRANCE
-//
 //-=============================================================================
 
 #if HAVE_CONFIG_H
@@ -171,7 +227,7 @@ namespace Tango
 
 DServer::DServer(DeviceClass *cl_ptr,const char *n,const char *d,Tango::DevState s,
 		 const char *st)
-:Device_3Impl(cl_ptr,n,d,s,st)
+:Device_4Impl(cl_ptr,n,d,s,st)
 {
 	process_name = Tango::Util::instance()->get_ds_exec_name();
 	instance_name = Tango::Util::instance()->get_ds_inst_name();
@@ -182,6 +238,9 @@ DServer::DServer(DeviceClass *cl_ptr,const char *n,const char *d,Tango::DevState
 	
 	last_heartbeat = time(NULL);
 	heartbeat_started = false;
+	
+	polling_th_pool_size = DEFAULT_POLLING_THREADS_POOL_SIZE;
+	optimize_pool_usage = true;
 
 	from_constructor = true;	
 	init_device();
@@ -198,7 +257,6 @@ bool less_than (Command *a,Command *b)
 
 void DServer::init_device()
 {
-
 //
 // In case of database 
 //
@@ -207,6 +265,12 @@ void DServer::init_device()
 	if (tg->_FileDb)
 		tg->reset_filedatabase();
 	
+//
+// Get device properties
+//
+	
+	get_dev_prop(tg);
+	
 //	
 // Destroy already registered classes
 //
@@ -508,7 +572,6 @@ void DServer::init_device()
 		}
 		throw;
 	}
-
 }
 
 //+----------------------------------------------------------------------------
@@ -579,7 +642,7 @@ void DServer::delete_devices()
 				
 //
 // Clear vectors used to memorize info used to clean db
-// in case of devices with dyn attr removed durind device
+// in case of devices with dyn attr removed during device
 // destruction
 //
 
@@ -591,7 +654,7 @@ void DServer::delete_devices()
 //
 // Delete device
 //
-					
+
 					class_list[i]->delete_dev(0,tg,r_poa);
 					
 //
@@ -609,8 +672,9 @@ void DServer::delete_devices()
 				}
 				devs.clear();
 				CORBA::release(r_poa);
-				
+
 				delete class_list[i];
+
 				class_list.pop_back();
 			}
 			else
@@ -621,7 +685,6 @@ void DServer::delete_devices()
 		}
 		class_list.clear();
 	}
-
 }
 
 //+----------------------------------------------------------------------------
@@ -727,9 +790,37 @@ Tango::DevVarStringArray *DServer::query_device()
 	for (int k = 0;k < nb_dev;k++)
 		(*ret)[k] = CORBA::string_dup(vs[k].c_str());
 		
-	return(ret);	
+	return(ret);
 }
 
+
+//+----------------------------------------------------------------------------
+//
+// method :         DServer::query_sub_device()
+// 
+// description :    command to read all the sub devices used by a device
+//                  server process
+////
+// out :            The sub device name list in a sequence of strings
+//
+//-----------------------------------------------------------------------------
+
+Tango::DevVarStringArray *DServer::query_sub_device()
+{
+	NoSyncModelTangoMonitor mon(this);
+	
+	cout4 << "In query_sub_device command" << endl;
+	
+	Tango::DevVarStringArray *ret;
+	
+	Tango::Util *tg = Tango::Util::instance();
+	ret = tg->get_sub_dev_diag().get_sub_devices();
+	
+	return(ret);
+}
+
+
+
 //+----------------------------------------------------------------------------
 //
 // method : 		DServer::restart()
@@ -803,6 +894,45 @@ void DServer::restart(string &d_name)
 	
 	DeviceImpl *dev_to_del = *ite;
 	
+//
+// If the device is locked and if the client is not the lock owner,
+// refuse to do the job
+//
+	
+	if (dev_to_del->is_device_locked() == true)
+	{
+		if (dev_to_del->valid_lock() == true)
+		{
+			client_addr *cl = get_client_ident();
+			if (cl->client_ident == true)
+			{
+				if (*cl != *(dev_to_del->get_locker()))
+				{
+					TangoSys_OMemStream o;
+					o << "Device " << d_name << " is locked by another client.";
+					o << " You can't restart it"<< ends;
+					Except::throw_exception((const char *)"API_DeviceLocked",o.str(),
+        									(const char *)"DServer::restart");
+				}
+			}
+			else
+			{
+				TangoSys_OMemStream o;
+				o << "Device " << d_name << " is locked by another client.";
+				o << " You can't restart it."<< ends;
+				Except::throw_exception((const char *)"API_DeviceLocked",o.str(),
+    									(const char *)"DServer::restart");
+			}
+		}
+	}
+	
+//
+// clean the sub-device list for this device
+//
+	Tango::Util *tg = Tango::Util::instance();
+	tg->get_sub_dev_diag().remove_sub_devices (dev_to_del->get_name());
+	tg->get_sub_dev_diag().set_associated_device(dev_to_del->get_name());
+
 //
 // Remove ourself from device list
 //
@@ -834,20 +964,39 @@ void DServer::restart(string &d_name)
 	
 	dev_to_del->get_device_attr()->get_event_param(eve);
 
+//
+// Also get device locker parameters if device locked
+//
+	
+	client_addr *cl_addr = NULL;
+	client_addr *old_cl_addr;
+	time_t l_date;
+	DevLong l_ctr,l_valid;
+	
+	if (dev_to_del->is_device_locked() == true)
+	{
+		cl_addr = dev_to_del->get_locker();
+		old_cl_addr = dev_to_del->get_old_locker();
+		l_date = dev_to_del->get_locking_date();
+		l_ctr = dev_to_del->get_locking_ctr();
+		l_valid = dev_to_del->get_lock_validity();
+		dev_to_del->clean_locker_ptrs();
+	}
+
 //
 // If the device was polled, stop polling
 //
 
 	if (dev_pol.empty() == false)
 	{
-		dev_to_del->stop_polling();
+		dev_to_del->stop_polling(false);
 	}
 		
 //
 // Delete the device (deactivate it and remove it)
 //
 	
-	Tango::Util *tg = Tango::Util::instance();
+	//Tango::Util *tg = Tango::Util::instance();
 	PortableServer::POA_ptr r_poa = tg->get_poa();
 	if (dev_to_del->get_exported_flag() == true)
 		r_poa->deactivate_object(dev_to_del->get_obj_id().in());
@@ -917,48 +1066,57 @@ void DServer::restart(string &d_name)
 	delete send;
 	
 //
-// Re-set event parameters (if needed)
+// Find the new device
 //
 
-	if (eve.size() != 0)
+	DeviceImpl *new_dev = NULL;
+
+	vector<Tango::DeviceImpl *> &d_list = dev_cl->get_device_list();
+	for (i = 0;i < d_list.size();i++)
 	{
-		vector<Tango::DeviceImpl *> &d_list = dev_cl->get_device_list();
-		DeviceImpl *new_dev = NULL;
-		for (i = 0;i < d_list.size();i++)
+		if (d_list[i]->get_name() == lower_d_name)
 		{
-			if (d_list[i]->get_name() == lower_d_name)
-			{
-				new_dev = d_list[i];
-				break;
-			}
-		}
-		if (i == d_list.size())
-		{
-			cout3 << "Not able to restart event rapidly" << endl;
-			TangoSys_OMemStream o;
-			o << "Not able to restart event rapidly" << ends;
-			Except::throw_exception((const char *)"API_DeviceNotFound",
-				        	o.str(),
-				        	(const char *)"Dserver::restart()");			
-		}
-		
-		for (i = 0;i < eve.size();i++)
-		{
-			Tango::MultiAttribute *m_attr = new_dev->get_device_attr();
-			Tango::Attribute &att = m_attr->get_attr_by_ind(eve[i].attr_id);
-			if (eve[i].change == true)
-				att.set_change_event_sub();
-			if (eve[i].periodic == true)
-				att.set_periodic_event_sub();
-			if (eve[i].quality == true)
-				att.set_quality_event_sub();
-			if (eve[i].archive == true)
-				att.set_archive_event_sub();
-			if (eve[i].user == true)
-				att.set_user_event_sub();
+			new_dev = d_list[i];
+			break;
 		}
 	}
+	if (i == d_list.size())
+	{
+		cout3 << "Not able to find the new device" << endl;
+		TangoSys_OMemStream o;
+		o << "Not able to find the new device" << ends;
+		Except::throw_exception((const char *)"API_DeviceNotFound",
+			        	o.str(),
+			        	(const char *)"Dserver::restart()");			
+	}
+	
+//
+// Re-set event parameters (if needed)
+//
+	
+	for (i = 0;i < eve.size();i++)
+	{
+		Tango::MultiAttribute *m_attr = new_dev->get_device_attr();
+		Tango::Attribute &att = m_attr->get_attr_by_ind(eve[i].attr_id);
+		if (eve[i].change == true)
+			att.set_change_event_sub();
+		if (eve[i].periodic == true)
+			att.set_periodic_event_sub();
+		if (eve[i].quality == true)
+			att.set_quality_event_sub();
+		if (eve[i].archive == true)
+			att.set_archive_event_sub();
+		if (eve[i].user == true)
+			att.set_user_event_sub();
+	}
 
+//
+// Re-lock device if necessary
+//
+	
+	if (cl_addr != NULL)
+		new_dev->set_locking_param(cl_addr,old_cl_addr,l_date,l_ctr,l_valid);
+		
 }
 
 //+----------------------------------------------------------------------------
@@ -996,6 +1154,12 @@ void ServRestartThread::run(void *ptr)
 
 	DServer *dev = (DServer *)ptr;
 	
+//
+// clean the sub-device list for the server
+//
+	Tango::Util *tg = Tango::Util::instance();
+	tg->get_sub_dev_diag().remove_sub_devices();
+
 //
 // Change the POA manager to discarding state. This is necessary to discard all
 // request arriving while the server restart.
@@ -1044,7 +1208,6 @@ void ServRestartThread::run(void *ptr)
 
 	{
 		AutoPyLock PyLo;
-		
 		dev->init_device();
 	}
 	
@@ -1052,7 +1215,7 @@ void ServRestartThread::run(void *ptr)
 // Restart polling (if any)
 //
 
-	Tango::Util *tg = Tango::Util::instance();
+	//Tango::Util *tg = Tango::Util::instance();
 	tg->polling_configure();
 
 //
@@ -1117,7 +1280,7 @@ Tango::DevVarStringArray *DServer::query_class_prop(string &class_name)
 	try
 	{
 		ret = new Tango::DevVarStringArray(nb_prop);
-                ret->length(nb_prop);
+		ret->length(nb_prop);
 	
 		for (int i = 0;i < nb_prop;i++)
 		{
@@ -1240,92 +1403,12 @@ void *KillThread::run_undetached(void *ptr)
 	omni_thread::self()->set_value(key_py_data,new PyData());
 
 //
-// Mark the server as shutting down
+// Shutdown the server
 //
 
 	Tango::Util *tg = Tango::Util::instance();
-	tg->set_svr_shutting_down(true);
-
-//
-//  Send kill command to the polling thread
-// 
-
-	TangoMonitor &mon = tg->get_poll_monitor();
-	PollThCmd &shared_cmd = tg->get_poll_shared_cmd();
-
-	{	
-		omni_mutex_lock sync(mon);
-
-		shared_cmd.cmd_pending = true;
-		shared_cmd.cmd_code = POLL_EXIT;
-
-		mon.signal();
-
-	}
-	tg->get_polling_thread_object()->join(0);
-	tg->clr_poll_th_ptr();
-					
-//
-// Unregister server device(s) from database
-//
-
-	try
-	{
-		tg->unregister_server();
-	}
-	catch(...) {}
-
-//
-// Delete devices (except the admin one)
-//
-
-
-	{
-		AutoPyLock PyLo;
-		tg->get_dserver_device()->delete_devices();
-	}
-
-//
-// Write file database for server started with the -file option
-//
-
-	if (tg->_FileDb == true)
-	{
-		Database *db_ptr = tg->get_database();
-		delete db_ptr;
-	}
-
-// 	Stop the KeepAliveThread and the EventConsumer Thread when 
-//  	they have been started to receive events.
-
-	EventConsumer *ec = ApiUtil::instance()->get_event_consumer();
-	if (ec != NULL)
-		ec->disconnect_from_notifd();
-
-//
-// If the server has used event, disconnect from the notifd
-// structured push consumer
-//
-
-	EventSupplier *ev = tg->get_event_supplier();
-	if (ev != NULL)
-		ev->disconnect_from_notifd();
-						
-//
-// Shutdown the ORB
-//
-
-	CORBA::ORB_ptr orb = Tango::Util::instance()->get_orb();
-	orb->shutdown(true);
-
-//
-// Cleanup logging stuffs
-//
-
-#ifdef TANGO_HAS_LOG4TANGO
-  	Logging::cleanup();
-#endif
-
+	tg->shutdown_server();
+	
 	return NULL;
 }
 
@@ -1428,6 +1511,61 @@ void DServer::create_cpp_class(const char *cl_name,const char *par_name)
 	add_class(dc);
 }
 
+//+----------------------------------------------------------------------------
+//
+// method : 		DServer::get_dev_prop()
+// 
+// description : 	Retrieve device properties
+//
+// argin: tg : Tango Util object ptr
+//
+//-----------------------------------------------------------------------------
+
+void DServer::get_dev_prop(Tango::Util *tg)
+{
+
+//
+// Try to retrieve device properties (Polling threads pool conf.)
+//
+
+	if (tg->_UseDb == true)
+	{
+		DbData db_data;
+		
+		db_data.push_back(DbDatum("polling_threads_pool_size"));
+		db_data.push_back(DbDatum("polling_threads_pool_conf"));
+
+		try
+		{		
+			db_dev->get_property(db_data);
+		}
+		catch (Tango::DevFailed &)
+		{
+			TangoSys_OMemStream o;
+			o << "Database error while trying to retrieve device properties for device " << device_name.c_str() << ends;
+
+			Except::throw_exception((const char *)"API_DatabaseAccess",
+					o.str(),
+					(const char *)"DServer::get_dev_prop");
+		}
+
+//
+// If the prop is not defined in db and if the user has defines it in the Util class,
+// takes the user definition
+//
+		
+		if (db_data[0].is_empty() == false)
+			db_data[0] >> polling_th_pool_size;
+		else
+		{
+			int p_size = tg->get_polling_threads_pool_size();
+			if (p_size != -1)
+				polling_th_pool_size = p_size;
+		}
+		if (db_data[1].is_empty() == false)
+			db_data[1] >> polling_th_pool_conf;
+	}
+}
 
 } // End of Tango namespace
 
