@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: dserverclass.cpp,v 3.14 2008/03/20 07:39:24 taurel Exp $\n$Name:  $";
+static const char *RcsId = "$Id: dserverclass.cpp,v 3.22 2009/02/03 15:15:08 jensmeyer Exp $\n$Name:  $";
 
 //+=============================================================================
 //
@@ -14,14 +14,70 @@ static const char *RcsId = "$Id: dserverclass.cpp,v 3.14 2008/03/20 07:39:24 tau
 //
 // author(s) :          A.Gotz + E.Taurel
 //
-// $Revision: 3.14 $
+// Copyright (C) :      2004,2005,2006,2007,2008,2009
+//						European Synchrotron Radiation Facility
+//                      BP 220, Grenoble 38043
+//                      FRANCE
+//
+// This file is part of Tango.
+//
+// Tango is free software: you can redistribute it and/or modify
+// it under the terms of the GNU Lesser General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+// 
+// Tango is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU Lesser General Public License for more details.
+// 
+// You should have received a copy of the GNU Lesser General Public License
+// along with Tango.  If not, see <http://www.gnu.org/licenses/>.
+//
+// $Revision: 3.22 $
 //
 // $Log: dserverclass.cpp,v $
+// Revision 3.22  2009/02/03 15:15:08  jensmeyer
+// Added a QuerySubDevice command to read the list of opened sub device
+// connections.
+//
+// Revision 3.21  2009/01/29 16:23:49  taurel
+// - Commit after merge with branch Release_6_1_1-bugfixes
+//
+// Revision 3.20  2009/01/21 12:49:04  taurel
+// - Change CopyRights for 2009
+//
+// Revision 3.19  2008/12/17 09:50:59  taurel
+// - First implementation of attributes sent on the wire using IDL Union
+// instead of IDL Any
+//
+// Revision 3.18  2008/10/06 15:01:09  taurel
+// - Changed the licensing info from GPL to LGPL
+//
+// Revision 3.17  2008/10/03 06:52:31  taurel
+// - Add some licensing info in each files
+//
+// Revision 3.16  2008/09/04 13:19:31  taurel
+// - Add push_event() feature for attribute of the DevEncoded data type
+//
+// Revision 3.15  2008/05/20 12:44:11  taurel
+// - Commit after merge with release 7 branch
+// Revision 3.14.2.1  2009/01/29 15:17:01  taurel
+// - Add some bug fixes for server used with the -f option (database as
+// file)
+//
 // Revision 3.14  2008/03/20 07:39:24  taurel
 // - Last commit before Release 6.1 ?
+// Revision 3.12.2.3  2008/01/03 16:05:52  taurel
+// - Some changes in locking feature implementation
 //
 // Revision 3.13  2007/12/12 10:17:18  taurel
 // - Db calls during DS startup has a separate timeout and some retries
+// Revision 3.12.2.2  2007/12/19 15:54:47  taurel
+// - Still some work going on for the locking feature
+//
+// Revision 3.12.2.1  2007/11/22 12:33:10  taurel
+// - First part of the device locking implementation
 //
 // Revision 3.12  2007/04/20 14:40:33  taurel
 // - Ported to Windows 64 bits x64 architecture
@@ -214,11 +270,6 @@ static const char *RcsId = "$Id: dserverclass.cpp,v 3.14 2008/03/20 07:39:24 tau
 // Revision 1.1.1.1  2000/02/04 10:58:27  taurel
 // Imported sources
 //
-//
-// copyleft :           European Synchrotron Radiation Facility
-//                      BP 220, Grenoble 38043
-//                      FRANCE
-//
 //-=============================================================================
 
 #if HAVE_CONFIG_H
@@ -462,6 +513,67 @@ CORBA::Any *DevQueryDeviceCmd::execute(DeviceImpl *device,const CORBA::Any &in_a
 	return(out_any);
 }
 
+//+----------------------------------------------------------------------------
+//
+// method : 		DevQuerySubDeviceCmd::DevQuerySubDeviceCmd()
+// 
+// description : 	constructor for the DevQuerySubDevice command of the 
+//					DServer.
+//
+//-----------------------------------------------------------------------------
+
+DevQuerySubDeviceCmd::DevQuerySubDeviceCmd(const char *name,
+			       	     Tango::CmdArgType in,
+			       	     Tango::CmdArgType out,
+				     const char *out_desc):Command(name,in,out)
+{
+	set_out_type_desc(out_desc);
+}
+
+//+----------------------------------------------------------------------------
+//
+// method : 		DevQuerySubDeviceCmd::execute(string &s)
+// 
+// description : 	method to trigger the execution of the "QuerySubDevice" 
+//					command
+//
+//-----------------------------------------------------------------------------
+
+CORBA::Any *DevQuerySubDeviceCmd::execute(DeviceImpl *device,const CORBA::Any &in_any)
+{	
+
+	cout4 << "DevQuerySubDeviceCmd::execute(): arrived" << endl;
+
+//	
+// call DServer method which implements this command
+//
+
+	Tango::DevVarStringArray *ret = ((DServer *)device)->query_sub_device();
+
+//
+// return data to the caller
+//
+	CORBA::Any *out_any;
+	try
+	{	
+		out_any = new CORBA::Any();
+	}
+	catch (bad_alloc)
+	{
+		cout3 << "Bad allocation while in DevQuerySubDeviceCmd::execute()" << endl;
+		delete ret;
+		Except::throw_exception((const char *)"API_MemoryAllocation",
+				      (const char *)"Can't allocate memory in server",
+				      (const char *)"DevQuerySubDeviceCmd::execute");
+	}
+	(*out_any) <<= ret;
+	
+	cout4 << "Leaving DevQuerySubDeviceCmd::execute()" << endl;		
+	return(out_any);
+}
+
+
+
 
 //+----------------------------------------------------------------------------
 //
@@ -1055,6 +1167,248 @@ CORBA::Any *QueryEventChannelIORCmd::execute(DeviceImpl *device,const CORBA::Any
 }
 
 
+//+----------------------------------------------------------------------------
+//
+// method : 		LockDeviceCmd::LockDeviceCmd
+// 
+// description : 	constructor for the LockDevice command of the DServer.
+//
+//-----------------------------------------------------------------------------
+
+
+LockDeviceCmd::LockDeviceCmd(const char *name,
+			     	     	   Tango::CmdArgType in,
+			     	     	   Tango::CmdArgType out,
+			     	     	   const char *in_desc):Command(name,in,out)
+{
+	set_in_type_desc(in_desc);
+}
+
+
+//+----------------------------------------------------------------------------
+//
+// method : 		LockDeviceCmd::execute()
+// 
+// description : 	method to trigger the execution of the "LockDevice" command
+//
+//-----------------------------------------------------------------------------
+
+CORBA::Any *LockDeviceCmd::execute(DeviceImpl *device,const CORBA::Any &in_any)
+{	
+
+	cout4 << "LockDeviceCmd::execute(): arrived" << endl;
+
+//
+// Extract the input argument
+//
+
+	const Tango::DevVarLongStringArray *in_data;
+	extract(in_any,in_data);
+			
+//	
+// call DServer method which implements this command
+//
+	
+	((DServer *)device)->lock_device(in_data);
+
+//
+// return data to the caller
+//
+
+	CORBA::Any *ret = return_empty_any("LockDeviceCmd");
+	return ret;
+}
+
+
+//+----------------------------------------------------------------------------
+//
+// method : 		ReLockDevicesCmd::ReLockDeviceCmd
+// 
+// description : 	constructor for the ReLockDevices command of the DServer.
+//
+//-----------------------------------------------------------------------------
+
+
+ReLockDevicesCmd::ReLockDevicesCmd(const char *name,
+			     	     	   Tango::CmdArgType in,
+			     	     	   Tango::CmdArgType out,
+					   const char *in_desc):Command(name,in,out)
+{
+	set_in_type_desc(in_desc);
+}
+
+
+//+----------------------------------------------------------------------------
+//
+// method : 		ReLockDevicesCmd::execute()
+// 
+// description : 	method to trigger the execution of the "ReLockDevices" command
+//
+//-----------------------------------------------------------------------------
+
+CORBA::Any *ReLockDevicesCmd::execute(DeviceImpl *device,const CORBA::Any &in_any)
+{	
+
+	cout4 << "ReLockDevicesCmd::execute(): arrived" << endl;
+
+//
+// Extract the input argument
+//
+
+	const Tango::DevVarStringArray *in_data;
+	extract(in_any,in_data);
+			
+//	
+// call DServer method which implements this command
+//
+
+	((DServer *)device)->re_lock_devices(in_data);
+
+//
+// return data to the caller
+//
+
+	CORBA::Any *ret = return_empty_any("ReLockDevicesCmd");
+	return ret;
+	
+}
+
+//+----------------------------------------------------------------------------
+//
+// method : 		UnLockDeviceCmd::UnLockDeviceCmd
+// 
+// description : 	constructor for the UnLockDevice command of the DServer.
+//
+//-----------------------------------------------------------------------------
+
+
+UnLockDeviceCmd::UnLockDeviceCmd(const char *name,
+			     	     	   Tango::CmdArgType in,
+			     	     	   Tango::CmdArgType out,
+					   		   const char *in_desc,
+					   		   const char *out_desc):Command(name,in,out)
+{
+	set_in_type_desc(in_desc);
+	set_out_type_desc(out_desc);
+}
+
+
+//+----------------------------------------------------------------------------
+//
+// method : 		UnLockDeviceCmd::execute()
+// 
+// description : 	method to trigger the execution of the "UnLockDevice" command
+//
+//-----------------------------------------------------------------------------
+
+CORBA::Any *UnLockDeviceCmd::execute(DeviceImpl *device,const CORBA::Any &in_any)
+{	
+
+	cout4 << "UnLockDeviceCmd::execute(): arrived" << endl;
+
+//
+// Extract the input string
+//
+
+	const Tango::DevVarLongStringArray *in_data;
+	extract(in_any,in_data);
+			
+//	
+// call DServer method which implements this command
+//
+
+	Tango::DevLong ret = ((DServer *)device)->un_lock_device(in_data);
+
+//
+// return data to the caller
+//
+
+	CORBA::Any *out_any;
+	try
+	{	
+		out_any = new CORBA::Any();
+	}
+	catch (bad_alloc)
+	{
+		cout3 << "Bad allocation while in UnLockDeviceCmd::execute()" << endl;
+		Except::throw_exception((const char *)"API_MemoryAllocation",
+				      (const char *)"Can't allocate memory in server",
+				      (const char *)"UnLockDeviceCmd::execute");
+	}
+	(*out_any) <<= ret;
+	
+	cout4 << "Leaving UnLockDeviceCmd::execute()" << endl;		
+	return(out_any);
+}
+
+//+----------------------------------------------------------------------------
+//
+// method : 		DevLockStatusCmd::DevLockStatusCmd
+// 
+// description : 	constructor for the DevLockStatus command of the DServer.
+//
+//-----------------------------------------------------------------------------
+
+
+DevLockStatusCmd::DevLockStatusCmd(const char *name,
+			     	     	   Tango::CmdArgType in,
+			     	     	   Tango::CmdArgType out,
+					   		   const char *in_desc,
+					   		   const char *out_desc):Command(name,in,out)
+{
+	set_in_type_desc(in_desc);
+	set_out_type_desc(out_desc);
+}
+
+
+//+----------------------------------------------------------------------------
+//
+// method : 		DevLockStatusCmd::execute()
+// 
+// description : 	method to trigger the execution of the "DevLockStatus" command
+//
+//-----------------------------------------------------------------------------
+
+CORBA::Any *DevLockStatusCmd::execute(DeviceImpl *device,const CORBA::Any &in_any)
+{	
+
+	cout4 << "DevLockStatusCmd::execute(): arrived" << endl;
+
+//
+// Extract the input string
+//
+
+	Tango::ConstDevString in_data;
+	extract(in_any,in_data);
+			
+//	
+// call DServer method which implements this command
+//
+
+	Tango::DevVarLongStringArray *ret = ((DServer *)device)->dev_lock_status(in_data);
+
+//
+// return to the caller
+//
+
+	CORBA::Any *out_any;
+	try
+	{	
+		out_any = new CORBA::Any();
+	}
+	catch (bad_alloc)
+	{
+		cout3 << "Bad allocation while in DevLockStatusCmd::execute()" << endl;
+		Except::throw_exception((const char *)"API_MemoryAllocation",
+				      (const char *)"Can't allocate memory in server",
+				      (const char *)"DevLockStatusCmd::execute");
+	}
+	(*out_any) <<= ret;
+	
+	cout4 << "Leaving DevLockStatusCmd::execute()" << endl;		
+	return(out_any);
+}
+
 
 
 
@@ -1202,6 +1556,10 @@ void DServerClass::command_factory()
 						     Tango::DEV_VOID,
 						     Tango::DEVVAR_STRINGARRAY,
 						     "Device server device(s) list"));
+	command_list.push_back(new DevQuerySubDeviceCmd("QuerySubDevice",
+						     Tango::DEV_VOID,
+						     Tango::DEVVAR_STRINGARRAY,
+						     "Device server sub device(s) list"));
 	command_list.push_back(new DevKillCmd("Kill",
 					      Tango::DEV_VOID,
 					      Tango::DEV_VOID));
@@ -1320,6 +1678,33 @@ void DServerClass::command_factory()
 							Tango::DEVVAR_STRINGARRAY,
 							"Class name",
 							"Device property list (name - description and default value)"));
+	
+//
+// Locking device commands
+//
+
+	command_list.push_back(new LockDeviceCmd("LockDevice",
+							Tango::DEVVAR_LONGSTRINGARRAY,
+							Tango::DEV_VOID,
+							"Str[0] = Device name. Lg[0] = Lock validity"));
+							
+	command_list.push_back(new UnLockDeviceCmd("UnLockDevice",
+							Tango::DEVVAR_LONGSTRINGARRAY,
+							Tango::DEV_LONG,
+							"Str[x] = Device name(s). Lg[0] = Force flag",
+							"Device global lock counter"));
+	
+	command_list.push_back(new ReLockDevicesCmd("ReLockDevices",
+							Tango::DEVVAR_STRINGARRAY,
+							Tango::DEV_VOID,
+							"Device(s) name"));
+							
+	command_list.push_back(new DevLockStatusCmd("DevLockStatus",
+							Tango::DEV_STRING,
+							Tango::DEVVAR_LONGSTRINGARRAY,
+							"Device name",
+							"Device locking status"));	
+
 	if (Util::_FileDb == true)
 	{
 		command_list.push_back(new QueryEventChannelIORCmd("QueryEventChannelIOR",
@@ -1378,7 +1763,7 @@ void DServerClass::device_factory(const Tango::DevVarStringArray *devlist_ptr)
 
 		tg->set_svr_starting(false);
 		Database *db = tg->get_database();
-		if (db != NULL)
+		if ((db != NULL) && (Util::_FileDb == false))
 			db->set_timeout_millis(CLNT_TIMEOUT);
 			
 	}
