@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: dserverpoll.cpp,v 3.20 2008/01/08 14:38:18 taurel Exp $\n$Name:  $";
+static const char *RcsId = "$Id: dserverpoll.cpp,v 3.30 2009/04/01 06:31:25 taurel Exp $\n$Name:  $";
 
 //+=============================================================================
 //
@@ -14,9 +14,60 @@ static const char *RcsId = "$Id: dserverpoll.cpp,v 3.20 2008/01/08 14:38:18 taur
 //
 // author(s) :          E.Taurel
 //
-// $Revision: 3.20 $
+// Copyright (C) :      2004,2005,2006,2007,2008,2009
+//						European Synchrotron Radiation Facility
+//                      BP 220, Grenoble 38043
+//                      FRANCE
+//
+// This file is part of Tango.
+//
+// Tango is free software: you can redistribute it and/or modify
+// it under the terms of the GNU Lesser General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+// 
+// Tango is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU Lesser General Public License for more details.
+// 
+// You should have received a copy of the GNU Lesser General Public License
+// along with Tango.  If not, see <http://www.gnu.org/licenses/>.
+//
+// $Revision: 3.30 $
 //
 // $Log: dserverpoll.cpp,v $
+// Revision 3.30  2009/04/01 06:31:25  taurel
+// - Only some changes in printed messages when using -v5
+//
+// Revision 3.29  2009/03/30 15:03:44  taurel
+// - Fix last bugs before Tango 7 ??
+//
+// Revision 3.28  2009/03/13 09:33:29  taurel
+// - Small changes to fix Windows VC8 warnings in Warning level 3
+//
+// Revision 3.27  2009/02/27 13:26:46  taurel
+// - Small changes for Solaris
+//
+// Revision 3.26  2009/01/21 12:49:04  taurel
+// - Change CopyRights for 2009
+//
+// Revision 3.25  2009/01/15 13:57:20  taurel
+// - Fix bugs found by Jens
+//
+// Revision 3.24  2008/12/17 09:50:59  taurel
+// - First implementation of attributes sent on the wire using IDL Union
+// instead of IDL Any
+//
+// Revision 3.23  2008/10/06 15:01:09  taurel
+// - Changed the licensing info from GPL to LGPL
+//
+// Revision 3.22  2008/10/03 06:52:31  taurel
+// - Add some licensing info in each files
+//
+// Revision 3.21  2008/10/02 09:09:47  taurel
+// - First implementation of multiple polling thread(s)
+//
 // Revision 3.20  2008/01/08 14:38:18  taurel
 // - strcasecmp() is not named like this on Windows !!
 //
@@ -258,11 +309,6 @@ static const char *RcsId = "$Id: dserverpoll.cpp,v 3.20 2008/01/08 14:38:18 taur
 // Revision 2.0  2002/04/09 14:45:10  taurel
 // See Tango WEB pages for list of changes
 //
-//
-// copyleft :           European Synchrotron Radiation Facility
-//                      BP 220, Grenoble 38043
-//                      FRANCE
-//
 //-=============================================================================
 
 #if HAVE_CONFIG_H
@@ -908,6 +954,25 @@ void DServer::add_obj_polling(const Tango::DevVarLongStringArray *argin,
 	poll_list.push_back(new PollObj(dev,type,obj_name,upd,depth));
 	dev->get_poll_monitor().rel_monitor();
 
+//
+// Find out which thread is in charge of the device.
+// If none exists already, create one
+//
+
+	PollingThreadInfo *th_info;
+	int thread_created = -2;
+	
+	int poll_th_id = tg->get_polling_thread_id_by_name((argin->svalue)[0]);
+	if (poll_th_id == 0)
+	{
+		cout4 << "POLLING: Creating a thread to poll device " << (argin->svalue)[0] << endl;
+		thread_created = tg->create_poll_thread((argin->svalue)[0],false);
+		poll_th_id = tg->get_polling_thread_id_by_name((argin->svalue)[0]);
+	}
+
+	cout4 << "POLLING: Thread in charge of device " << (argin->svalue)[0] << " is thread " << poll_th_id << endl;
+	th_info = tg->get_polling_thread_info_by_id(poll_th_id);
+	
 //
 // Send command to the polling thread but wait in case of previous cmd 
 // still not executed
@@ -915,8 +980,8 @@ void DServer::add_obj_polling(const Tango::DevVarLongStringArray *argin,
 
 	cout4 << "Sending cmd to polling thread" << endl;
 	int interupted;
-	TangoMonitor &mon = tg->get_poll_monitor();
-	PollThCmd &shared_cmd = tg->get_poll_shared_cmd();
+	TangoMonitor &mon = th_info->poll_mon;
+	PollThCmd &shared_cmd = th_info->shared_data;
 
 	{	
 		omni_mutex_lock sync(mon);
@@ -941,7 +1006,7 @@ void DServer::add_obj_polling(const Tango::DevVarLongStringArray *argin,
 
 		omni_thread *th = omni_thread::self();
 		int th_id = th->id();
-		if (th_id != tg->get_polling_thread_id())
+		if (th_id != poll_th_id)
 		{
 			while (shared_cmd.cmd_pending == true)
 			{
@@ -951,6 +1016,18 @@ void DServer::add_obj_polling(const Tango::DevVarLongStringArray *argin,
 					cout4 << "TIME OUT" << endl;
 					delete poll_list.back();
 					poll_list.pop_back();
+
+//
+// If the thread has been created by this request, try to kill it
+//
+					
+					if (thread_created == -1)
+					{
+						shared_cmd.cmd_pending = true;
+						shared_cmd.cmd_code = POLL_ADD_OBJ;
+						
+						mon.signal();
+					}
 					Except::throw_exception((const char *)"API_CommandTimedOut",
 					        		(const char *)"Polling thread blocked !!!",
 					        		(const char *)"DServer::add_obj_polling");
@@ -959,6 +1036,7 @@ void DServer::add_obj_polling(const Tango::DevVarLongStringArray *argin,
 		}
 	}
 	cout4 << "Thread cmd normally executed" << endl;
+	th_info->nb_polled_objects++;
 		
 //
 // Update polling parameters in database (if wanted and possible)
@@ -1048,9 +1126,38 @@ void DServer::add_obj_polling(const Tango::DevVarLongStringArray *argin,
 		DbData send_data;
 		send_data.push_back(db_info);	
 		dev->get_db_device()->put_property(send_data);
-
-		cout4 << "Polling properties updated" << endl;
 	}
+		
+//
+// Also update the polling threads pool conf if one thread has been created by this call
+//
+
+	if (thread_created != -2)
+	{
+		string dev_name((argin->svalue)[0]);
+		transform(dev_name.begin(),dev_name.end(),dev_name.begin(),::tolower);
+		cout4 << "thread_created = " << thread_created << endl;
+		if (thread_created == -1)
+		{
+			tg->get_poll_pool_conf().push_back(dev_name);
+		}
+		else if (thread_created >= 0)
+		{
+			string &conf_entry = (tg->get_poll_pool_conf())[thread_created];
+			conf_entry = conf_entry + ',' + dev_name;
+		}
+
+		if ((with_db_upd == true) && (Tango::Util::_UseDb == true))
+		{
+			DbData send_data;
+			send_data.push_back(DbDatum("polling_threads_pool_conf"));			
+			send_data[0] << tg->get_poll_pool_conf();
+
+			tg->get_dserver_device()->get_db_device()->put_property(send_data);
+		}
+	}
+
+	cout4 << "Polling properties updated" << endl;
 	
 //
 // Mark the device as polled
@@ -1198,9 +1305,27 @@ void DServer::upd_obj_polling_period(const Tango::DevVarLongStringArray *argin,
 		TangoSys_OMemStream o;
 		o << (argin->lvalue)[0] << " is below the min authorized period (" << MIN_POLL_PERIOD << " mS)" << ends;
 		Except::throw_exception((const char *)"API_NotSupported",o.str(),
-					(const char *)"DServer::add_obj_polling");
+					(const char *)"DServer::upd_obj_polling");
 	}
-				
+
+//
+// Find out which thread is in charge of the device.
+// If none exists already, create one
+//
+
+	PollingThreadInfo *th_info;
+		
+	int poll_th_id = tg->get_polling_thread_id_by_name((argin->svalue)[0]);
+	if (poll_th_id == 0)
+	{
+		TangoSys_OMemStream o;
+		o << "Can't find a polling thread for device " << (argin->svalue)[0] << ends;
+		Except::throw_exception((const char *)"API_NotSupported",o.str(),
+					(const char *)"DServer::upd_obj_polling");
+	}
+
+	th_info = tg->get_polling_thread_info_by_id(poll_th_id);
+
 //
 // Update polling period
 //
@@ -1211,12 +1336,11 @@ void DServer::upd_obj_polling_period(const Tango::DevVarLongStringArray *argin,
 // Send command to the polling thread
 //
 
-	TangoMonitor &mon = tg->get_poll_monitor();
-	PollThCmd &shared_cmd = tg->get_poll_shared_cmd();
+	TangoMonitor &mon = th_info->poll_mon;
+	PollThCmd &shared_cmd = th_info->shared_data;
 
-	omni_thread *th = omni_thread::self();
-	int th_id = th->id();
-	if (th_id != tg->get_polling_thread_id())		
+	int th_id = omni_thread::self()->id();
+	if (th_id != poll_th_id)		
 	{	
 		omni_mutex_lock sync(mon);
 		if (shared_cmd.cmd_pending == true)
@@ -1229,7 +1353,12 @@ void DServer::upd_obj_polling_period(const Tango::DevVarLongStringArray *argin,
 		shared_cmd.name = obj_name;
 		shared_cmd.type = type;
 		shared_cmd.new_upd = (argin->lvalue)[0];
-	
+
+#ifdef __SUNPRO_CC
+		distance(dev->get_poll_obj_list().begin(),ite,shared_cmd.index);
+#else
+		shared_cmd.index = distance(dev->get_poll_obj_list().begin(),ite);
+#endif
 		mon.signal();
 	}
 	else
@@ -1240,13 +1369,17 @@ void DServer::upd_obj_polling_period(const Tango::DevVarLongStringArray *argin,
 		shared_cmd.name = obj_name;
 		shared_cmd.type = type;
 		shared_cmd.new_upd = (argin->lvalue)[0];
+#ifdef __SUNPRO_CC
+		distance(dev->get_poll_obj_list().begin(),ite,shared_cmd.index);
+#else
+		shared_cmd.index = distance(dev->get_poll_obj_list().begin(),ite);
+#endif
 		
-		PollThread *poll_th = tg->get_polling_thread_object();
+		PollThread *poll_th = th_info->poll_th;
 		poll_th->set_local_cmd(shared_cmd);
 		poll_th->execute_cmd();
 	}
 	
-	
 //
 // Update database property --> Update polling period if this object is already
 // defined in the polling property. Add object name and update period if the 
@@ -1257,7 +1390,7 @@ void DServer::upd_obj_polling_period(const Tango::DevVarLongStringArray *argin,
 	{
 		TangoSys_MemStream s;
 		string upd_str;	
-		s << (argin->lvalue)[0];
+		s << (argin->lvalue)[0] << ends;
 		s >> upd_str;
 	
 		DbDatum db_info("polled_attr");		
@@ -1404,11 +1537,29 @@ void DServer::rem_obj_polling(const Tango::DevVarStringArray *argin,
 	vector<PollObj *>::iterator ite = dev->get_polled_obj_by_type_name(type,obj_name);
 	long tmp_upd = (*ite)->get_upd();
 
+//
+// Find out which thread is in charge of the device.
+//
+
+	PollingThreadInfo *th_info;
+			
+	int poll_th_id = tg->get_polling_thread_id_by_name((*argin)[0]);
+	if (poll_th_id == 0)
+	{
+		TangoSys_OMemStream o;
+		o << "Can't find a polling thread for device " << (*argin)[0] << ends;
+		Except::throw_exception((const char *)"API_NotSupported",o.str(),
+						(const char *)"DServer::rem_obj_polling");
+	}
+
+	cout4 << "Thread in charge of device " << (*argin)[0] << " is thread " << poll_th_id << endl;
+	th_info = tg->get_polling_thread_info_by_id(poll_th_id);
+	
 //
 // Test whether the polling thread is still running!
 //
 
-	if ( tg->get_polling_thread_object() != NULL )
+	if ( th_info->poll_th != NULL )
 	{
 //
 // Send command to the polling thread
@@ -1416,8 +1567,8 @@ void DServer::rem_obj_polling(const Tango::DevVarStringArray *argin,
 
 		cout4 << "Sending cmd to polling thread" << endl;
 		int interupted;
-		TangoMonitor &mon = tg->get_poll_monitor();
-		PollThCmd &shared_cmd = tg->get_poll_shared_cmd();
+		TangoMonitor &mon = th_info->poll_mon;
+		PollThCmd &shared_cmd = th_info->shared_data;
 		{	
 			omni_mutex_lock sync(mon);
 			if (shared_cmd.cmd_pending == true)
@@ -1442,9 +1593,8 @@ void DServer::rem_obj_polling(const Tango::DevVarStringArray *argin,
 // requested by the polling thread itself
 //
 
-			omni_thread *th = omni_thread::self();
-			int th_id = th->id();
-			if (th_id != tg->get_polling_thread_id())
+			int th_id = omni_thread::self()->id();
+			if (th_id != poll_th_id)
 			{
 				while (shared_cmd.cmd_pending == true)
 				{
@@ -1577,6 +1727,87 @@ void DServer::rem_obj_polling(const Tango::DevVarStringArray *argin,
 			cout4 << "Database polling properties updated" << endl;
 		}
 	}
+
+//
+// If the device is not polled any more, update the pool conf first locally.
+// Also update the map<device name,thread id>
+// If this device was the only one for a polling thread, kill the thread
+// Then in Db if possible
+//
+
+	bool kill_thread = false;
+	if (poll_list.empty() == true)
+	{
+		int ind;
+		string dev_name((*argin)[0]);
+		transform(dev_name.begin(),dev_name.end(),dev_name.begin(),::tolower);
+		
+		if ((ind = tg->get_dev_entry_in_pool_conf(dev_name)) ==  -1)
+		{
+			TangoSys_OMemStream o;
+			o << "Can't find entry for device " << (*argin)[0] << " in polling threads pool configuration !"<< ends;
+			Except::throw_exception((const char *)"API_NotSupported",o.str(),
+							(const char *)"DServer::rem_obj_polling");				
+		}
+
+		vector<string> &pool_conf = tg->get_poll_pool_conf();
+		string &conf_entry = pool_conf[ind];
+		string::size_type pos;
+		if ((pos = conf_entry.find(',')) != string::npos)
+		{
+			pos = conf_entry.find(dev_name);
+			if ((pos + dev_name.size()) != conf_entry.size())
+				conf_entry.erase(pos,dev_name.size() + 1);
+			else
+				conf_entry.erase(pos - 1);
+		}
+		else
+		{
+			vector<string>::iterator iter = pool_conf.begin() + ind;
+			pool_conf.erase(iter);
+			kill_thread = true;
+		}
+
+		tg->remove_dev_from_polling_map(dev_name);
+		
+//
+// Kill the thread if needed and join
+//
+
+		if (kill_thread == true)
+		{
+			TangoMonitor &mon = th_info->poll_mon;
+			PollThCmd &shared_cmd = th_info->shared_data;
+	
+			{
+				omni_mutex_lock sync(mon);
+
+				shared_cmd.cmd_pending = true;
+				shared_cmd.cmd_code = POLL_EXIT;
+
+				mon.signal();
+			}
+
+			void *dummy_ptr;
+			cout4 << "POLLING: Joining with one polling thread" << endl;
+			th_info->poll_th->join(&dummy_ptr);
+		
+			tg->remove_polling_thread_info_by_id(poll_th_id);
+		}
+
+//
+// Update db
+//
+
+		if ((with_db_upd == true) && (Tango::Util::_UseDb == true))
+		{
+			DbData send_data;
+			send_data.push_back(DbDatum("polling_threads_pool_conf"));
+			send_data[0] << tg->get_poll_pool_conf();
+			
+			tg->get_dserver_device()->get_db_device()->put_property(send_data);
+		}
+	}
 }
 
 //+----------------------------------------------------------------------------
@@ -1598,31 +1829,38 @@ void DServer::stop_polling()
 //
 
 	int interupted;
-	Tango::Util *tg = Tango::Util::instance();	
-	TangoMonitor &mon = tg->get_poll_monitor();
-	PollThCmd &shared_cmd = tg->get_poll_shared_cmd();
+	Tango::Util *tg = Tango::Util::instance();
+	
+	vector<PollingThreadInfo *> &th_info = tg->get_polling_threads_info();
+	vector<PollingThreadInfo *>:: iterator iter;
 
-	{	
-		omni_mutex_lock sync(mon);
-		if (shared_cmd.cmd_pending == true)
-		{
-			mon.wait();
-		}
-		shared_cmd.cmd_pending = true;
-		shared_cmd.cmd_code = POLL_STOP;
+	for (iter = th_info.begin();iter != th_info.end();++iter)
+	{
+		TangoMonitor &mon = (*iter)->poll_mon;
+		PollThCmd &shared_cmd = (*iter)->shared_data;
 
-		mon.signal();
-		
-		while (shared_cmd.cmd_pending == true)
-		{
-			interupted = mon.wait(DEFAULT_TIMEOUT);
-
-			if ((shared_cmd.cmd_pending == true) && (interupted == false))
+		{	
+			omni_mutex_lock sync(mon);
+			if (shared_cmd.cmd_pending == true)
 			{
-				cout4 << "TIME OUT" << endl;
-				Except::throw_exception((const char *)"API_CommandTimedOut",
-					        	(const char *)"Polling thread blocked !!!",
-					        	(const char *)"DServer::stop_polling");
+				mon.wait();
+			}
+			shared_cmd.cmd_pending = true;
+			shared_cmd.cmd_code = POLL_STOP;
+	
+			mon.signal();
+			
+			while (shared_cmd.cmd_pending == true)
+			{
+				interupted = mon.wait(DEFAULT_TIMEOUT);
+	
+				if ((shared_cmd.cmd_pending == true) && (interupted == false))
+				{
+					cout4 << "TIME OUT" << endl;
+					Except::throw_exception((const char *)"API_CommandTimedOut",
+						        	(const char *)"Polling thread blocked !!!",
+						        	(const char *)"DServer::stop_polling");
+				}
 			}
 		}
 	}
@@ -1652,35 +1890,42 @@ void DServer::start_polling()
 	cout4 << "In start_polling method" << endl;
 	
 //
-// Send command to the polling thread and wait for its execution
+// Send command to the polling thread(s) and wait for its execution
 //
 
 	int interupted;
-	Tango::Util *tg = Tango::Util::instance();	
-	TangoMonitor &mon = tg->get_poll_monitor();
-	PollThCmd &shared_cmd = tg->get_poll_shared_cmd();
-
-	{	
-		omni_mutex_lock sync(mon);
-		if (shared_cmd.cmd_pending == true)
-		{
-			mon.wait();
-		}
-		shared_cmd.cmd_pending = true;
-		shared_cmd.cmd_code = POLL_START;
+	Tango::Util *tg = Tango::Util::instance();
 	
-		mon.signal();
-		
-		while (shared_cmd.cmd_pending == true)
-		{
-			interupted = mon.wait(DEFAULT_TIMEOUT);
+	vector<PollingThreadInfo *> &th_info = tg->get_polling_threads_info();
+	vector<PollingThreadInfo *>:: iterator iter;
 
-			if ((shared_cmd.cmd_pending == true) && (interupted == false))
+	for (iter = th_info.begin();iter != th_info.end();++iter)
+	{
+		TangoMonitor &mon = (*iter)->poll_mon;
+		PollThCmd &shared_cmd = (*iter)->shared_data;
+
+		{	
+			omni_mutex_lock sync(mon);
+			if (shared_cmd.cmd_pending == true)
 			{
-				cout4 << "TIME OUT" << endl;
-				Except::throw_exception((const char *)"API_CommandTimedOut",
-					        	(const char *)"Polling thread blocked !!!",
-					        	(const char *)"DServer::start_polling");
+				mon.wait();
+			}
+			shared_cmd.cmd_pending = true;
+			shared_cmd.cmd_code = POLL_START;
+		
+			mon.signal();
+			
+			while (shared_cmd.cmd_pending == true)
+			{
+				interupted = mon.wait(DEFAULT_TIMEOUT);
+	
+				if ((shared_cmd.cmd_pending == true) && (interupted == false))
+				{
+					cout4 << "TIME OUT" << endl;
+					Except::throw_exception((const char *)"API_CommandTimedOut",
+						        	(const char *)"Polling thread blocked !!!",
+						        	(const char *)"DServer::start_polling");
+				}
 			}
 		}
 	}
@@ -1699,7 +1944,7 @@ void DServer::start_polling()
 //
 // method : 		DServer::add_event_heartbeat()
 // 
-// description : 	command to ask the polling thread to send
+// description : 	command to ask the heartbeat thread to send
 //			the event heartbeat every 10 seconds
 //
 //-----------------------------------------------------------------------------
@@ -1711,15 +1956,16 @@ void DServer::add_event_heartbeat()
 	cout4 << "In add_event_heartbeat method" << endl;
 
 //
-// Send command to the polling thread but wait in case of previous cmd 
+// Send command to the heartbeat thread but wait in case of previous cmd 
 // still not executed
 //
 
 	cout4 << "Sending cmd to polling thread" << endl;
 	int interupted;
-	Tango::Util *tg = Tango::Util::instance();	
-	TangoMonitor &mon = tg->get_poll_monitor();
-	PollThCmd &shared_cmd = tg->get_poll_shared_cmd();
+	Tango::Util *tg = Tango::Util::instance();
+	
+	TangoMonitor &mon = tg->get_heartbeat_monitor();
+	PollThCmd &shared_cmd = tg->get_heartbeat_shared_cmd();
 
 	{	
 		omni_mutex_lock sync(mon);
@@ -1739,9 +1985,8 @@ void DServer::add_event_heartbeat()
 // requested by the polling thread itself
 //
 
-		omni_thread *th = omni_thread::self();
-		int th_id = th->id();
-		if (th_id != tg->get_polling_thread_id())
+		int th_id = omni_thread::self()->id();
+		if (th_id != tg->get_heartbeat_thread_id())
 		{
 			while (shared_cmd.cmd_pending == true)
 			{
@@ -1764,7 +2009,7 @@ void DServer::add_event_heartbeat()
 //
 // method : 		DServer::rem_event_heartbeat()
 // 
-// description : 	command to ask the polling thread to stop sending
+// description : 	command to ask the heartbeat thread to stop sending
 //			the event heartbeat 
 //
 //-----------------------------------------------------------------------------
@@ -1776,15 +2021,15 @@ void DServer::rem_event_heartbeat()
 	cout4 << "In rem_event_heartbeat method" << endl;
 
 //
-// Send command to the polling thread but wait in case of previous cmd 
+// Send command to the heartbeat thread but wait in case of previous cmd 
 // still not executed
 //
 
 	cout4 << "Sending cmd to polling thread" << endl;
 	int interupted;
 	Tango::Util *tg = Tango::Util::instance();	
-	TangoMonitor &mon = tg->get_poll_monitor();
-	PollThCmd &shared_cmd = tg->get_poll_shared_cmd();
+	TangoMonitor &mon = tg->get_heartbeat_monitor();
+	PollThCmd &shared_cmd = tg->get_heartbeat_shared_cmd();
 
 	{	
 		omni_mutex_lock sync(mon);
@@ -1804,9 +2049,8 @@ void DServer::rem_event_heartbeat()
 // requested by the polling thread itself
 //
 
-		omni_thread *th = omni_thread::self();
-		int th_id = th->id();
-		if (th_id != tg->get_polling_thread_id())
+		int th_id = omni_thread::self()->id();
+		if (th_id != tg->get_heartbeat_thread_id())
 		{
 			while (shared_cmd.cmd_pending == true)
 			{
