@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: dserversignal.cpp,v 3.12 2008/04/04 14:25:02 jensmeyer Exp $\n$Name:  $";
+static const char *RcsId = "$Id: dserversignal.cpp,v 3.15 2009/01/21 12:49:04 taurel Exp $\n$Name:  $";
 
 //+=============================================================================
 //
@@ -14,9 +14,38 @@ static const char *RcsId = "$Id: dserversignal.cpp,v 3.12 2008/04/04 14:25:02 je
 //
 // author(s) :          A.Gotz + E.Taurel
 //
-// $Revision: 3.12 $
+// Copyright (C) :      2004,2005,2006,2007,2008,2009
+//						European Synchrotron Radiation Facility
+//                      BP 220, Grenoble 38043
+//                      FRANCE
+//
+// This file is part of Tango.
+//
+// Tango is free software: you can redistribute it and/or modify
+// it under the terms of the GNU Lesser General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+// 
+// Tango is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU Lesser General Public License for more details.
+// 
+// You should have received a copy of the GNU Lesser General Public License
+// along with Tango.  If not, see <http://www.gnu.org/licenses/>.
+//
+// $Revision: 3.15 $
 //
 // $Log: dserversignal.cpp,v $
+// Revision 3.15  2009/01/21 12:49:04  taurel
+// - Change CopyRights for 2009
+//
+// Revision 3.14  2008/10/06 15:01:09  taurel
+// - Changed the licensing info from GPL to LGPL
+//
+// Revision 3.13  2008/10/03 06:52:31  taurel
+// - Add some licensing info in each files
+//
 // Revision 3.12  2008/04/04 14:25:02  jensmeyer
 // Added the necessary signal definitions for FreeBSD __freebsd__
 //
@@ -207,11 +236,6 @@ static const char *RcsId = "$Id: dserversignal.cpp,v 3.12 2008/04/04 14:25:02 je
 // Revision 1.1.1.1  2000/02/04 10:58:28  taurel
 // Imported sources
 //
-//
-// copyleft :           European Synchrotron Radiation Facility
-//                      BP 220, Grenoble 38043
-//                      FRANCE
-//
 //-=============================================================================
 
 #if HAVE_CONFIG_H
