@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: eventcmds.cpp,v 1.15 2008/01/18 14:25:30 jensmeyer Exp $";
+static const char *RcsId = "$Id: eventcmds.cpp,v 1.24 2009/04/27 11:16:34 taurel Exp $";
 //+=============================================================================
 //
 // file :        eventcmds.cpp
@@ -8,11 +8,60 @@ static const char *RcsId = "$Id: eventcmds.cpp,v 1.15 2008/01/18 14:25:30 jensme
 //
 // project :     TANGO events
 //
-// $Author: jensmeyer $
+// $Author: taurel $
 //
-// $Revision: 1.15 $
+// Copyright (C) :      2004,2005,2006,2007,2008,2009
+//						European Synchrotron Radiation Facility
+//                      BP 220, Grenoble 38043
+//                      FRANCE
+//
+// This file is part of Tango.
+//
+// Tango is free software: you can redistribute it and/or modify
+// it under the terms of the GNU Lesser General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+// 
+// Tango is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU Lesser General Public License for more details.
+// 
+// You should have received a copy of the GNU Lesser General Public License
+// along with Tango.  If not, see <http://www.gnu.org/licenses/>.
+//
+// $Revision: 1.24 $
 //
 // $Log: eventcmds.cpp,v $
+// Revision 1.24  2009/04/27 11:16:34  taurel
+// - Attribute with DevState data type do not need a relative or absolute
+// change property
+//
+// Revision 1.23  2009/01/29 15:25:41  taurel
+// - First implementation of the Data Ready event
+//
+// Revision 1.22  2009/01/21 12:49:03  taurel
+// - Change CopyRights for 2009
+//
+// Revision 1.21  2009/01/06 09:28:50  taurel
+// - Add compatibility between a server IDL V4 and clients IDL V3
+//
+// Revision 1.20  2008/10/06 15:01:09  taurel
+// - Changed the licensing info from GPL to LGPL
+//
+// Revision 1.19  2008/10/03 06:52:31  taurel
+// - Add some licensing info in each files
+//
+// Revision 1.18  2008/10/02 09:09:47  taurel
+// - First implementation of multiple polling thread(s)
+//
+// Revision 1.17  2008/09/23 14:59:35  taurel
+// - Commit after the end of DevEncoded data type implementation
+// - The new test suite is also now running fine
+//
+// Revision 1.16  2008/07/01 07:38:40  taurel
+// - Some more code for a proper implementation of the DevEncoded data type with the new IDL release 4
+//
 // Revision 1.15  2008/01/18 14:25:30  jensmeyer
 // Added the stateless event subscription in the EventConsumer.
 // The subscribe_event method has an option now to subscribe for
@@ -130,11 +179,6 @@ static const char *RcsId = "$Id: eventcmds.cpp,v 1.15 2008/01/18 14:25:30 jensme
 // Revision 1.1.2.1  2003/04/10 11:07:44  andy_gotz
 // added event commands to DServer admin object
 //
-//
-// copyleft :   European Synchrotron Radiation Facility
-//              BP 220, Grenoble 38043
-//              FRANCE
-//
 //-=============================================================================
 //
 //  		This file was generated by POGO
@@ -165,10 +209,10 @@ namespace Tango
 //
 //-----------------------------------------------------------------------------
 EventSubscriptionChangeCmd::EventSubscriptionChangeCmd(const char *name,
-						       Tango::CmdArgType in,
-				       		       Tango::CmdArgType out,
-						       const char *in_desc,
-				       		       const char *out_desc)
+								Tango::CmdArgType in,
+								Tango::CmdArgType out,
+								const char *in_desc,
+								const char *out_desc)
 :Command(name,in,out,in_desc,out_desc)
 {
 }
@@ -177,9 +221,7 @@ EventSubscriptionChangeCmd::EventSubscriptionChangeCmd(const char *name,
 //	Constructor without in/out parameters description
 //
 
-EventSubscriptionChangeCmd::EventSubscriptionChangeCmd(const char		*name,
-						       Tango::CmdArgType in,
-				       		       Tango::CmdArgType out)
+EventSubscriptionChangeCmd::EventSubscriptionChangeCmd(const char *name,Tango::CmdArgType in,Tango::CmdArgType out)
 :Command(name,in,out)
 {
 }
@@ -260,7 +302,7 @@ CORBA::Any *EventSubscriptionChangeCmd::execute(Tango::DeviceImpl *device,const
 // shuting down sequence, do nothing
 //
 
-	if (tg->get_polling_thread_object() == NULL)
+	if (tg->get_heartbeat_thread_object() == NULL)
 	{
      	TangoSys_OMemStream o;
 		o << "The device server is shutting down! You can no longer subscribe for events" << ends;
@@ -268,9 +310,6 @@ CORBA::Any *EventSubscriptionChangeCmd::execute(Tango::DeviceImpl *device,const
 		Except::throw_exception((const char *)"DServer_Events",
 									    o.str(),
 									   (const char *)"EventSubscriptionChangeCmd::execute");
-
-		//  CORBA::Any *ret = return_empty_any("EventSubscriptionChange");
-		//return ret;
 	}
 		
 //
@@ -302,6 +341,20 @@ CORBA::Any *EventSubscriptionChangeCmd::execute(Tango::DeviceImpl *device,const
 
 	int attr_ind = dev_impl->dev_attr->get_attr_ind_by_name(attr_name.c_str());
 	Attribute &attribute = dev_impl->dev_attr->get_attr_by_ind(attr_ind);
+
+//
+// Check if the request comes from a Tango 6 client (without client identification)
+// If true, the event has to be sent using AttributeValue_3 data structure
+//
+
+	client_addr *cl = device->get_client_ident();
+	int cl_release;
+
+	if (cl->client_ident == true)
+		cl_release = 4;
+	else
+		cl_release = 3;
+
 	
 	if (action == "subscribe")
 	{
@@ -309,16 +362,26 @@ CORBA::Any *EventSubscriptionChangeCmd::execute(Tango::DeviceImpl *device,const
 		{
 			cout4 << "EventSubscriptionChangeCmd::execute(): update user_event subscription\n";
 			attribute.ext->event_user_subscription = time(NULL);
+			if (cl_release == 3)
+				attribute.ext->event_user_client_3 = true;
 		}
 		else if (event == "attr_conf")
 		{
-			cout4 << "EventSubscriptionChangeCmd::execute(): update attr_conf subsription\n";
+			cout4 << "EventSubscriptionChangeCmd::execute(): update attr_conf subscription\n";
 			attribute.ext->event_attr_conf_subscription = time(NULL);
 		}
+		else if (event == "data_ready")
+		{
+			cout4 << "EventSubscriptionChangeCmd::execute(): update data_ready subscription\n";
+			attribute.ext->event_data_ready_subscription = time(NULL);
+		}
 		else
-		{		
-			// If the polling is necessary to send events, check whether the polling is
-			// started for the requested attribute.
+		{	
+			
+//
+// If the polling is necessary to send events, check whether the polling is
+// started for the requested attribute.
+//
 			
 			if (attribute.is_polled() == false )			
 			{
@@ -361,14 +424,16 @@ CORBA::Any *EventSubscriptionChangeCmd::execute(Tango::DeviceImpl *device,const
        		{
 				cout4 << "EventSubscriptionChangeCmd::execute(): update change subscription\n";
 
-				//
-				// Check if the attribute has some of the change properties defined
-				//
+//
+// Check if the attribute has some of the change properties defined
+//
 
 				if (attr_name_lower != "state")
 				{
 					if ((attribute.get_data_type() != Tango::DEV_STRING) &&
-			            (attribute.get_data_type() != Tango::DEV_BOOLEAN))
+			            (attribute.get_data_type() != Tango::DEV_BOOLEAN) &&
+			            (attribute.get_data_type() != Tango::DEV_ENCODED) &&
+						(attribute.get_data_type() != Tango::DEV_STATE))
 					{
 						if ( attribute.is_check_change_criteria() == true )
 						{
@@ -390,6 +455,8 @@ CORBA::Any *EventSubscriptionChangeCmd::execute(Tango::DeviceImpl *device,const
 					}
 				}
        			attribute.ext->event_change_subscription = time(NULL);
+				if (cl_release == 3)
+					attribute.ext->event_change_client_3 = true;
 			}
       		else if (event == "quality")
       		{
@@ -400,17 +467,22 @@ CORBA::Any *EventSubscriptionChangeCmd::execute(Tango::DeviceImpl *device,const
       		{
 				cout4 << "EventSubscriptionChangeCmd::execute(): update periodic subscription\n";
        			attribute.ext->event_periodic_subscription = time(NULL);
+				if (cl_release == 3)
+					attribute.ext->event_periodic_client_3 = true;
       		}                                                  
       		else if (event == "archive")
       		{
-				//
-				// Check if the attribute has some of the archive properties defined
-				//
+      			
+//
+// Check if the attribute has some of the archive properties defined
+//
 
 				if (attr_name_lower != "state")
 				{
 					if ((attribute.get_data_type() != Tango::DEV_STRING) &&
-			                    (attribute.get_data_type() != Tango::DEV_BOOLEAN))
+			            (attribute.get_data_type() != Tango::DEV_BOOLEAN) &&
+			            (attribute.get_data_type() != Tango::DEV_ENCODED) &&
+						(attribute.get_data_type() != Tango::DEV_STATE))
 					{
 						if ( attribute.is_check_archive_criteria() == true )
 						{
@@ -433,22 +505,23 @@ CORBA::Any *EventSubscriptionChangeCmd::execute(Tango::DeviceImpl *device,const
 		
 				cout4 << "EventSubscriptionChangeCmd::execute(): update archive subscription\n";
        			attribute.ext->event_archive_subscription = time(NULL);
+				if (cl_release == 3)
+					attribute.ext->event_archive_client_3 = true;
       		}				 			
 		}		  
 	
-		//
-		// Start polling for attribute in question. I suppose I should
-		// check to see if the attribute is polled already. For the 
-		// moment I will simply ignore the exception. Why not rather 
-		// introduce a is_polled() method in each Attribute ?
-		//
-		// Use the add_obj_polling() admin device method whith no
-		// db update. The polling will poll the attribute next time
-		// the server is started if  there no more event client connected
-		// to the attribute
-		//
+//
+// Start polling for attribute in question. I suppose I should
+// check to see if the attribute is polled already. For the 
+// moment I will simply ignore the exception. Why not rather 
+// introduce a is_polled() method in each Attribute ?
+//
+// Use the add_obj_polling() admin device method whith no
+// db update. The polling will poll the attribute next time
+// the server is started if  there no more event client connected
+// to the attribute
+//
 			
-		//DevVarLongStringArray *send = new DevVarLongStringArray();
 		try
 		{
 			Tango::Util *tg = Tango::Util::instance();
@@ -459,25 +532,10 @@ CORBA::Any *EventSubscriptionChangeCmd::execute(Tango::DeviceImpl *device,const
 				adm_dev->add_event_heartbeat();
 				adm_dev->set_heartbeat_started(true);
 			}
-
-				//send->lvalue.length(1);
-				//send->svalue.length(3);
-				//send->svalue[0] = CORBA::string_dup(dev_name.c_str());
-				//send->svalue[1] = CORBA::string_dup("attribute");
-
-				//send->lvalue[0] = attribute.ext->poll_period;
-				//send->svalue[2] = CORBA::string_dup(attr_name.c_str());
-
-				//cout4 << "EventSubscriptionChangeCmd::execute(): start attribute " << attr_name << " polling with period " << attribute.ext->poll_period << endl;
-				//adm_dev->add_obj_polling(send,false);
 		}
 		catch (...)
 		{
-			//cout << "EventSubscriptionChangeCmd::execute(): failed to start polling !\n";
-			// nada
 		}
-		//delete send;			
-		//}
 	}
 
 	CORBA::Any *ret = return_empty_any("EventSubscriptionChange");
