@@ -11,8 +11,28 @@
 /// 		author(s) : A.Gotz (goetz@esrf.fr)
 ///
 /// 		original : 7 April 2003
+//
+// Copyright (C) :      2003,2004,2005,2006,2007,2008,2009
+//						European Synchrotron Radiation Facility
+//                      BP 220, Grenoble 38043
+//                      FRANCE
+//
+// This file is part of Tango.
+//
+// Tango is free software: you can redistribute it and/or modify
+// it under the terms of the GNU Lesser General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+// 
+// Tango is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU Lesser General Public License for more details.
+// 
+// You should have received a copy of the GNU Lesser General Public License
+// along with Tango.  If not, see <http://www.gnu.org/licenses/>.
 ///
-/// 		$Revision: 1.22 $
+/// 		$Revision: 1.29 $
 ///
 /// 		copyright : European Synchrotron Radiation Facility
 ///                         BP 220, Grenoble 38043
@@ -103,11 +123,10 @@ private :
 	int heartbeat_period;
 	int subscription_timeout;
 	string event_channel_ior;
-	
+
 	void get_attribute_value(AttributeValue attr_value, LastAttrValue &curr_attr_value);
 	void reconnect_notifd();
-	TANGO_IMP_EXP static void connect_to_notifd(NotifService &,
-			CORBA::ORB_var &,string &,Database *,string &,Util *);
+	TANGO_IMP_EXP static void connect_to_notifd(NotifService &,CORBA::ORB_var &,string &,Database *,string &,Util *);
 			
 	// Added a mutex to synchronize the access to 
 	//	detect_and_push_change_event_3	and
@@ -126,7 +145,8 @@ private :
 	omni_mutex		detect_mutex;
 	
 public :
-	void detect_and_push_events_3(DeviceImpl *,long,AttributeValue_3 &,DevFailed *,string &);
+	void push_att_data_ready_event(DeviceImpl *,const string &,long,DevLong);
+	void detect_and_push_events_3(DeviceImpl *,long,AttributeValue_3 *,AttributeValue_4 *,DevFailed *,string &);
 
 
 /////////////////////////////////////////////////////////////////////////////
@@ -248,7 +268,8 @@ public :
 
 
 	bool detect_change_3(Attribute &attr,
-				  AttributeValue_3 &curr_attr_value,
+				  AttributeValue_3 *curr_attr_value,
+				  AttributeValue_4 *curr_attr_value_4,
 				  bool archive,
 				  double &delta_change_rel,
 				  double &delta_change_abs,
@@ -869,7 +890,8 @@ public :
 
 
 	void detect_and_push_change_event_3(DeviceImpl *device_impl,
-						 AttributeValue_3 &attr_value,
+						 AttributeValue_3 *attr_value,
+						 AttributeValue_4 *attr_value_4,
 						 Attribute &attr,
 						 string &attr_name,
 						 DevFailed *except);
@@ -986,6 +1008,7 @@ public :
 //
 // argument : in :	device_impl : The device
 //			attr_value : The attribute value
+//			attr_value_4 : The attribute value for devic eimplementing IDL 4
 //			attr : The attribute object
 //			attr_name : The attribute name
 //			except : The exception thrown during the last
@@ -994,7 +1017,8 @@ public :
 //-----------------------------------------------------------------------------
 
 	void detect_and_push_archive_event_3(DeviceImpl *device_impl,
-						  AttributeValue_3 &attr_value, 
+						  AttributeValue_3 *attr_value,
+						  AttributeValue_4 *attr_value_4, 
 						  Attribute &attr,
 						  string &attr_name,
 						  DevFailed *except);
@@ -1188,13 +1212,6 @@ public :
 //
 //-----------------------------------------------------------------------------
 
-	void detect_and_push_quality_change_event_3(DeviceImpl *device_impl,
-						  AttributeValue_3 &attr_value,
-						  Attribute &attr,
-						  string &attr_name,
-						  DevFailed *except);
-
-
 	template <typename T>
 	void detect_and_push_quality_change_event(DeviceImpl *device_impl,
 						  T &attr_value,
@@ -1274,6 +1291,7 @@ public :
 //
 // argument : in :	device_impl : The device
 //			attr_value : The attribute value
+//			attr_value_4 : The attribute value for device implementing IDL 4
 //			attr : The attribute object
 //			attr_name : The attribute name
 //			except : The exception thrown during the last
@@ -1282,7 +1300,8 @@ public :
 //-----------------------------------------------------------------------------
 
 	void detect_and_push_periodic_event_3(DeviceImpl *device_impl,
-					    AttributeValue_3 &attr_value,
+					    AttributeValue_3 *attr_value,
+					 	AttributeValue_4 *attr_value_4,
 					    Attribute &attr,
 					    string &attr_name,
 					    DevFailed *except);
@@ -1405,7 +1424,8 @@ public :
 		       	vector<double> &filterable_data,
 		       	vector<string> &filterable_names_lg,
 		       	vector<long> &filterable_data_lg,
-		       	AttributeValue_3 &attr_value,
+		       	AttributeValue_3 *attr_value,
+				AttributeValue_4 *attr_value_4,
 		       	string &attr_name,
 		       	DevFailed *except);
 			
@@ -1489,21 +1509,21 @@ public :
 			cout3 << "EventSupplier::push_event() event channel disconnected !\n";
 			fail = true;
 		}
-       		catch(const CORBA::TRANSIENT &)
-       		{
-       			cout3 << "EventSupplier::push_event() caught a CORBA::TRANSIENT ! " << endl;
+    	catch(const CORBA::TRANSIENT &)
+    	{
+    		cout3 << "EventSupplier::push_event() caught a CORBA::TRANSIENT ! " << endl;
 			fail = true;
-       		}
-       		catch(const CORBA::COMM_FAILURE &)
-       		{
-       			cout3 << "EventSupplier::push_event() caught a CORBA::COMM_FAILURE ! " << endl;
+    	}
+    	catch(const CORBA::COMM_FAILURE &)
+    	{
+    		cout3 << "EventSupplier::push_event() caught a CORBA::COMM_FAILURE ! " << endl;
 			fail = true;
 		}
-    		catch(const CORBA::SystemException &)
-    		{
-       			cout3 << "EventSupplier::push_event() caught a CORBA::SystemException ! " << endl;
+		catch(const CORBA::SystemException &)
+		{
+    		cout3 << "EventSupplier::push_event() caught a CORBA::SystemException ! " << endl;
 			fail = true;
-    		}
+		}
 	
 //
 // If it was not possible to communicate with notifd,
