@@ -8,9 +8,26 @@
 //
 // author(s) :	  N.Leclercq - SOLEIL
 //
-// copyleft :	    European Synchrotron Radiation Facility
-//			          BP 220, Grenoble 38043
-//			          FRANCE
+//
+// Copyright (C) :      2004,2005,2006,2007,2008,2009
+//						European Synchrotron Radiation Facility
+//                      BP 220, Grenoble 38043
+//                      FRANCE
+//
+// This file is part of Tango.
+//
+// Tango is free software: you can redistribute it and/or modify
+// it under the terms of the GNU Lesser General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+// 
+// Tango is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU Lesser General Public License for more details.
+// 
+// You should have received a copy of the GNU Lesser General Public License
+// along with Tango.  If not, see <http://www.gnu.org/licenses/>.
 //
 //-=============================================================================
 
@@ -239,6 +256,9 @@ log4tango::LoggerStream& operator<< (log4tango::LoggerStream& ls, const Attribut
 	ls << "Attribute data_format: ";
 	switch (conf.data_format)
 	{
+	  case Tango::FMT_UNKNOWN:
+	  	  break;
+		  
 	  case Tango::SCALAR :
 		  ls << "scalar" << endl;
 		  break;
