@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: multiattribute.cpp,v 3.16 2008/03/11 14:38:25 taurel Exp $\n$Name:  $";
+static const char *RcsId = "$Id: multiattribute.cpp,v 3.20 2009/01/21 12:47:15 taurel Exp $\n$Name:  $";
 
 //+============================================================================
 //
@@ -14,9 +14,41 @@ static const char *RcsId = "$Id: multiattribute.cpp,v 3.16 2008/03/11 14:38:25 t
 //
 // author(s) :          E.Taurel
 //
-// $Revision: 3.16 $
+// Copyright (C) :      2004,2005,2006,2007,2008,2009
+//						European Synchrotron Radiation Facility
+//                      BP 220, Grenoble 38043
+//                      FRANCE
+//
+// This file is part of Tango.
+//
+// Tango is free software: you can redistribute it and/or modify
+// it under the terms of the GNU Lesser General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+// 
+// Tango is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU Lesser General Public License for more details.
+// 
+// You should have received a copy of the GNU Lesser General Public License
+// along with Tango.  If not, see <http://www.gnu.org/licenses/>.
+//
+// $Revision: 3.20 $
 //
 // $Log: multiattribute.cpp,v $
+// Revision 3.20  2009/01/21 12:47:15  taurel
+// - Change CopyRights for 2009
+//
+// Revision 3.19  2008/10/06 15:01:36  taurel
+// - Changed the licensing info from GPL to LGPL
+//
+// Revision 3.18  2008/10/03 06:52:31  taurel
+// - Add some licensing info in each files
+//
+// Revision 3.17  2008/06/10 07:52:15  taurel
+// - Add code for the DevEncoded attribute data type
+//
 // Revision 3.16  2008/03/11 14:38:25  taurel
 // - Apply patches from Frederic Picca about compilation with gcc 4.2
 //
@@ -264,11 +296,6 @@ static const char *RcsId = "$Id: multiattribute.cpp,v 3.16 2008/03/11 14:38:25 t
 // Revision 1.3  2000/04/13 10:40:41  taurel
 // Added attribute support
 //
-//
-// copyleft :           European Synchrotron Radiation Facility
-//                      BP 220, Grenoble 38043
-//                      FRANCE
-//
 //-============================================================================
 
 #if HAVE_CONFIG_H
@@ -1428,6 +1455,13 @@ void MultiAttribute::add_write_value(Attribute &att)
 		state_write_val = assoc_att.get_last_written_state();
 		att.add_write_value(state_write_val);
 		break;
+		
+	case Tango::DEV_ENCODED :
+		{
+			DevEncoded &enc_write_val = assoc_att.get_last_written_encoded();
+			att.add_write_value(enc_write_val);
+			break;
+		}
 	}
 }
 
