@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: pollthread.cpp,v 3.17 2007/05/15 07:46:59 taurel Exp $\n$Name:  $";
+static const char *RcsId = "$Id: pollthread.cpp,v 3.26 2009/02/16 09:30:37 jensmeyer Exp $\n$Name:  $";
 
 //+============================================================================
 //
@@ -13,9 +13,61 @@ static const char *RcsId = "$Id: pollthread.cpp,v 3.17 2007/05/15 07:46:59 taure
 //
 // author(s) :          E.Taurel
 //
-// $Revision: 3.17 $
+// Copyright (C) :      2004,2005,2006,2007,2008,2009
+//						European Synchrotron Radiation Facility
+//                      BP 220, Grenoble 38043
+//                      FRANCE
+//
+// This file is part of Tango.
+//
+// Tango is free software: you can redistribute it and/or modify
+// it under the terms of the GNU Lesser General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+// 
+// Tango is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU Lesser General Public License for more details.
+// 
+// You should have received a copy of the GNU Lesser General Public License
+// along with Tango.  If not, see <http://www.gnu.org/licenses/>.
+//
+// $Revision: 3.26 $
 //
 // $Log: pollthread.cpp,v $
+// Revision 3.26  2009/02/16 09:30:37  jensmeyer
+// Modified print_list() method to handle sub device diagnostics.
+//
+// Revision 3.25  2009/02/03 15:17:11  jensmeyer
+// Added the storage of sub device properties on a regular basis of
+// 30 minutes to the heartbeat thread of a device server.
+//
+// Revision 3.24  2009/01/21 12:47:15  taurel
+// - Change CopyRights for 2009
+//
+// Revision 3.23  2009/01/08 14:58:03  taurel
+// - The read_attribute_4 also transfer the client authentification
+//
+// Revision 3.22  2008/10/06 15:01:36  taurel
+// - Changed the licensing info from GPL to LGPL
+//
+// Revision 3.21  2008/10/03 06:52:31  taurel
+// - Add some licensing info in each files
+//
+// Revision 3.20  2008/10/02 09:09:47  taurel
+// - First implementation of multiple polling thread(s)
+//
+// Revision 3.19  2008/07/01 07:38:40  taurel
+// - Some more code for a proper implementation of the DevEncoded data type with the new IDL release 4
+//
+// Revision 3.18  2008/05/20 12:44:12  taurel
+// - Commit after merge with release 7 branch
+//
+// Revision 3.17.2.1  2008/05/20 06:17:46  taurel
+// - Last commit before merge with trunk
+// (start the implementation of the new DevEncoded data type)
+//
 // Revision 3.17  2007/05/15 07:46:59  taurel
 // - The polling thread is not configured by a separate thread any more.
 // The Add_obj_polling command now support a delta_t to start the first polling
@@ -215,10 +267,6 @@ static const char *RcsId = "$Id: pollthread.cpp,v 3.17 2007/05/15 07:46:59 taure
 //   mutex
 //
 //
-// copyleft :           European Synchrotron Radiation Facility
-//                      BP 220, Grenoble 38043
-//                      FRANCE
-//
 //-============================================================================
 
 #if HAVE_CONFIG_H
@@ -257,12 +305,15 @@ PollObjType PollThread::type_to_del = Tango::POLL_CMD;
 //
 //--------------------------------------------------------------------------
 
-PollThread::PollThread(PollThCmd &cmd,TangoMonitor &m): shared_cmd(cmd),p_mon(m),
+PollThread::PollThread(PollThCmd &cmd,TangoMonitor &m,bool heartbeat): shared_cmd(cmd),p_mon(m),
 					    sleep(1),polling_stop(false),
 					    attr_names(1),tune_ctr(1),
-					    need_two_tuning(false),auto_upd(-1)
+					    need_two_tuning(false),auto_upd(-1),send_heartbeat(heartbeat)
 {
 	attr_names.length(1);
+
+	cci = 0;
+	dummy_cl_id.cpp_clnt(cci);
 	
 #ifdef _TG_WINDOWS_
 	LARGE_INTEGER f;
@@ -291,6 +342,36 @@ void *PollThread::run_undetached(void *ptr)
 	PollCmdType received;
 	bool per_thread_data_created = false;
 
+//
+// If the thread is the event heartbeat thread,
+// use it also for the storage of sub device properties.
+// Declare a work item to check the for new sub devices
+// regularly.
+// 
+	if ( send_heartbeat == true )
+	{
+		WorkItem wo;
+		
+		wo.dev = NULL;
+		wo.poll_list = NULL;
+		wo.type = STORE_SUBDEV;
+		wo.update = 30*60*1000;			// check ervery 30 minutes
+		wo.name = "Sub device property storage";
+		wo.needed_time.tv_sec  = 0;
+		wo.needed_time.tv_usec = 0;
+		
+#ifdef _TG_WINDOWS_
+		_ftime(&now_win);
+		now.tv_sec = (unsigned long)now_win.time;
+		now.tv_usec = (long)now_win.millitm * 1000;
+#else
+		gettimeofday(&now,NULL);
+#endif
+		now.tv_sec = now.tv_sec - DELTA_T;
+		wo.wake_up_date = now; 
+		insert_in_list(wo);
+	}
+
 //
 // The infinite loop
 //
@@ -757,7 +838,7 @@ void PollThread::execute_cmd()
 		break;
 
 //
-// Add the event heartbeat every 10 seconds
+// Add the event heartbeat every 9 seconds
 //
 
 	case Tango::POLL_ADD_HEARTBEAT:
@@ -765,7 +846,7 @@ void PollThread::execute_cmd()
 		wo.dev = NULL;
 		wo.poll_list = NULL;
 		wo.type = EVENT_HEARTBEAT;
-		wo.update = 10000;
+		wo.update = 9000;
 		wo.name = "Event heartbeat";
 		wo.needed_time.tv_sec = 0;
 		wo.needed_time.tv_usec = TIME_HEARTBEAT;
@@ -896,6 +977,9 @@ void PollThread::one_more_poll()
 		case Tango::EVENT_HEARTBEAT:
 			eve_heartbeat(tmp);
 			break;
+		case Tango::STORE_SUBDEV:
+			store_subdev(tmp);
+			break;
 		}
 	}
 
@@ -963,16 +1047,6 @@ void PollThread::one_more_trigg()
 		return;
 	}
 	
-//
-// Inform requesting thread that the work is done
-//
-
-	{
-		omni_mutex_lock sync(p_mon);
-		shared_cmd.trigger = false;
-		p_mon.signal();
-	}
-	
 //
 // Do the job
 //	
@@ -985,7 +1059,16 @@ void PollThread::one_more_trigg()
 		else
 			poll_attr(tmp);
 	}
-			
+
+//
+// Inform requesting thread that the work is done
+//
+
+	{
+		omni_mutex_lock sync(p_mon);
+		shared_cmd.trigger = false;
+		p_mon.signal();
+	}
 }
 
 
@@ -1006,13 +1089,23 @@ void PollThread::print_list()
 	ite = works.begin();
 	for (i = 0;i < nb_elt;i++)
 	{
-		if (ite->type != EVENT_HEARTBEAT)
+		if (ite->type != EVENT_HEARTBEAT )
 		{
-			cout4 << "Dev name = " << ite->dev->get_name()
-          		<< ", obj name = " << ite->name
-          		<< ", next wake_up at " << + ite->wake_up_date.tv_sec
-          		<< "," << setw(6) << setfill('0')
-          		<< ite->wake_up_date.tv_usec << endl;
+			if ( ite->type != STORE_SUBDEV)
+			{
+				cout4 << "Dev name = " << ite->dev->get_name()
+					<< ", obj name = " << ite->name
+					<< ", next wake_up at " << + ite->wake_up_date.tv_sec
+					<< "," << setw(6) << setfill('0')
+					<< ite->wake_up_date.tv_usec << endl;
+			}
+			else
+			{
+				cout4 <<  ite->name
+					<< ", next wake_up at " << + ite->wake_up_date.tv_sec
+					<< "," << setw(6) << setfill('0')
+					<< ite->wake_up_date.tv_usec << endl;
+			}
 		}
 		else
 		{
@@ -1299,7 +1392,8 @@ void PollThread::err_out_of_sync(WorkItem &to_do)
 		if (idl_vers >= 3)
 			event_supplier->detect_and_push_events_3(to_do.dev,
 						       idl_vers,
-						       dummy_att3,
+						       &dummy_att3,
+						       NULL,
 						       &except,
 						       to_do.name);
 		else	
@@ -1467,6 +1561,7 @@ void PollThread::poll_attr(WorkItem &to_do)
 #endif
 	Tango::AttributeValueList *argout = NULL;
 	Tango::AttributeValueList_3 *argout_3 = NULL;
+	Tango::AttributeValueList_4 *argout_4 = NULL;
 	Tango::DevFailed *save_except = NULL;
 	bool attr_failed = false;	
 	vector<PollObj *>::iterator ite;
@@ -1490,7 +1585,9 @@ void PollThread::poll_attr(WorkItem &to_do)
 //
 
 		attr_names[0] = to_do.name.c_str();
-		if (idl_vers >= 3)
+		if (idl_vers >= 4)
+			argout_4 = (static_cast<Device_4Impl *>(to_do.dev))->read_attributes_4(attr_names,Tango::DEV,dummy_cl_id);
+		else if (idl_vers == 3)
 			argout_3 = (static_cast<Device_3Impl *>(to_do.dev))->read_attributes_3(attr_names,Tango::DEV);
 		else		
 			argout = to_do.dev->read_attributes(attr_names);
@@ -1562,11 +1659,23 @@ void PollThread::poll_attr(WorkItem &to_do)
 
 	if (idl_vers >= 3)
 	{
-		if ((attr_failed == false) && ((*argout_3)[0].err_list.length() != 0))
+		if (idl_vers == 4)
 		{
-			attr_failed = true;
-			save_except = new Tango::DevFailed((*argout_3)[0].err_list);
-			delete argout_3;
+			if ((attr_failed == false) && ((*argout_4)[0].err_list.length() != 0))
+			{
+				attr_failed = true;
+				save_except = new Tango::DevFailed((*argout_4)[0].err_list);
+				delete argout_4;
+			}
+		}
+		else
+		{
+			if ((attr_failed == false) && ((*argout_3)[0].err_list.length() != 0))
+			{
+				attr_failed = true;
+				save_except = new Tango::DevFailed((*argout_3)[0].err_list);
+				delete argout_3;
+			}
 		}
 	}
 	
@@ -1585,7 +1694,8 @@ void PollThread::poll_attr(WorkItem &to_do)
 			if (idl_vers >= 3)
 				event_supplier->detect_and_push_events_3(to_do.dev,
 						               idl_vers,
-						       	       dummy_att3,
+						       	       &dummy_att3,
+						       	       NULL,
 						               save_except,
 						               to_do.name);
 			else	
@@ -1597,15 +1707,23 @@ void PollThread::poll_attr(WorkItem &to_do)
 		}
 		else
 		{
-			if (idl_vers >= 3)
+			if (idl_vers >= 4)
 				event_supplier->detect_and_push_events_3(to_do.dev,
-							       	       idl_vers,
-						       	               (*argout_3)[0],
+							       	       	   idl_vers,
+							       	       	   NULL,
+						       	               &((*argout_4)[0]),
 						                       save_except,
 						                       to_do.name);
+			else if (idl_vers == 3)
+				event_supplier->detect_and_push_events_3(to_do.dev,
+							       	       	   idl_vers,
+						       	               &((*argout_3)[0]),
+						       	               NULL,
+						                       save_except,
+						                       to_do.name);				
 			else
 				event_supplier->detect_and_push_events(to_do.dev,
-							       	       idl_vers,
+							       	       	   idl_vers,
 						       	               (*argout)[0],
 						                       save_except,
 						                       to_do.name);
@@ -1615,7 +1733,8 @@ void PollThread::poll_attr(WorkItem &to_do)
 // Heartbeat - check to see if it is time to send a heartbeat event
 //
 
-		event_supplier->push_heartbeat_event();
+		if (send_heartbeat == true)
+			event_supplier->push_heartbeat_event();
 	}
 	
 			
@@ -1630,7 +1749,9 @@ void PollThread::poll_attr(WorkItem &to_do)
 		ite = to_do.dev->get_polled_obj_by_type_name(to_do.type,to_do.name);
 		if (attr_failed == false)
 		{
-			if (idl_vers >= 3)
+			if (idl_vers >= 4)
+				(*ite)->insert_data(argout_4,before_cmd,needed_time);
+			else if (idl_vers == 3)
 				(*ite)->insert_data(argout_3,before_cmd,needed_time);						
 			else
 				(*ite)->insert_data(argout,before_cmd,needed_time);
@@ -1643,7 +1764,9 @@ void PollThread::poll_attr(WorkItem &to_do)
 	{
 		if (attr_failed == false)
 		{
-			if (idl_vers >= 3)
+			if (idl_vers >= 4)
+				delete argout_4;
+			else if (idl_vers == 3)
 				delete argout_3;
 			else
 				delete argout;
@@ -1673,11 +1796,45 @@ void PollThread::eve_heartbeat(WorkItem &to_do)
 
 	EventSupplier *event_supplier;
 	event_supplier = Util::instance()->get_event_supplier();
-	if (event_supplier != NULL)
+	if ((event_supplier != NULL) && (send_heartbeat == true))
 	{
 		event_supplier->push_heartbeat_event();
 	}
 	
 }
 
+
+//+-------------------------------------------------------------------------
+//
+// method : 		PollThread::store_subdev
+// 
+// description : 	Store the sub device properties when
+//                  needed.
+//
+// argument : in :	- to_do : The work item
+//
+//--------------------------------------------------------------------------
+
+void PollThread::store_subdev(WorkItem &to_do)
+{
+	static bool ignore_call = true;
+	
+	cout5 << "----------> Time = " << now.tv_sec << ","
+	      << setw(6) << setfill('0') << now.tv_usec
+	      << " Store sub device property data if needed!" << endl;
+	
+	
+	if ( !ignore_call )
+	{
+		Tango::Util *tg = Tango::Util::instance();
+		tg->get_sub_dev_diag().store_sub_devices();
+	}
+	else
+	{
+		// ignore the first call to avoid storage during
+		// device server start-up.
+		ignore_call = false;
+	}
+}
+
 } // End of Tango namespace
