@@ -8,9 +8,49 @@
 //
 // author(s) :          A.Gotz + E.Taurel
 //
-// $Revision: 3.12 $
+// Copyright (C) :      2004,2005,2006,2007,2008,2009
+//						European Synchrotron Radiation Facility
+//                      BP 220, Grenoble 38043
+//                      FRANCE
+//
+// This file is part of Tango.
+//
+// Tango is free software: you can redistribute it and/or modify
+// it under the terms of the GNU Lesser General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+// 
+// Tango is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU Lesser General Public License for more details.
+// 
+// You should have received a copy of the GNU Lesser General Public License
+// along with Tango.  If not, see <http://www.gnu.org/licenses/>.
+//
+// $Revision: 3.17 $
 //
 // $Log: tango.h,v $
+// Revision 3.17  2009/01/21 12:49:03  taurel
+// - Change CopyRights for 2009
+//
+// Revision 3.16  2008/11/18 09:28:56  taurel
+// - Ported to gcc 4.3
+// - Removed some cout messages
+//
+// Revision 3.15  2008/10/06 15:01:36  taurel
+// - Changed the licensing info from GPL to LGPL
+//
+// Revision 3.14  2008/10/03 06:53:09  taurel
+// - Add some licensing info in each files
+//
+// Revision 3.13  2008/05/20 12:44:14  taurel
+// - Commit after merge with release 7 branch
+//
+// Revision 3.12.2.1  2007/11/16 14:12:35  taurel
+// - Added a new IDL interface (Device_4)
+// - Added a new way to get attribute history from polling buffer (must faster)
+//
 // Revision 3.12  2007/04/20 14:41:34  taurel
 // - Ported to Windows 64 bits x64 architecture
 //
@@ -129,11 +169,6 @@
 // - Change the way how TangoMonitor class is implemented. It's a recursive
 //   mutex
 //
-//
-// copyleft :           European Synchrotron Radiation Facility
-//                      BP 220, Grenoble 38043
-//                      FRANCE
-//
 //=============================================================================
 
 #ifndef _TANGO_H
@@ -174,7 +209,6 @@
 	#endif   // VC8+
 #endif
 
-
 //
 // Insert typeinfo file as the first include file (needed for Hp-UX aCC as the
 // first include file)
@@ -272,16 +306,28 @@
 	#include <auto_tango_monitor.h>
 	#include <device_2.h>
 	#include <device_3.h>
+	#include <device_4.h>
 	#include <command.h>
 	#include <dserver.h>
 #endif
 
 #include <event.h>
 
+//
+// minor is also defined (Linux) in sysmacros.h. We want the compiler to
+// use the SystemException::minor() method !!
+//
+
+#ifdef minor
+# undef minor
+#endif
+
 //
 // Default std namespace
 //
 
 using namespace std;
+
+
 	
 #endif /* TANGO_H */
