@@ -8,9 +8,102 @@
 //
 // author(s) :          A.Gotz + E.Taurel
 //
-// $Revision: 3.24 $
+// Copyright (C) :      2004,2005,2006,2007,2008,2009
+//						European Synchrotron Radiation Facility
+//                      BP 220, Grenoble 38043
+//                      FRANCE
+//
+// This file is part of Tango.
+//
+// Tango is free software: you can redistribute it and/or modify
+// it under the terms of the GNU Lesser General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+// 
+// Tango is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU Lesser General Public License for more details.
+// 
+// You should have received a copy of the GNU Lesser General Public License
+// along with Tango.  If not, see <http://www.gnu.org/licenses/>.
+//
+// $Revision: 3.41 $
 //
 // $Log: tango_const.h,v $
+// Revision 3.41  2009/04/30 12:49:24  taurel
+// - Adapted to Tango 7.0.2
+//
+// Revision 3.40  2009/04/27 11:17:16  taurel
+// - Release 7.0.1
+//
+// Revision 3.39  2009/03/20 11:52:50  taurel
+// - Add tangorc files management (for env. variables)
+//
+// Revision 3.38  2009/02/03 15:17:11  jensmeyer
+// Added the storage of sub device properties on a regular basis of
+// 30 minutes to the heartbeat thread of a device server.
+//
+// Revision 3.37  2009/01/29 15:25:41  taurel
+// - First implementation of the Data Ready event
+//
+// Revision 3.36  2009/01/21 12:49:03  taurel
+// - Change CopyRights for 2009
+//
+// Revision 3.35  2008/10/06 15:01:36  taurel
+// - Changed the licensing info from GPL to LGPL
+//
+// Revision 3.34  2008/10/03 06:53:09  taurel
+// - Add some licensing info in each files
+//
+// Revision 3.33  2008/10/02 09:09:47  taurel
+// - First implementation of multiple polling thread(s)
+//
+// Revision 3.32  2008/05/20 12:44:14  taurel
+// - Commit after merge with release 7 branch
+//
+// Revision 3.31  2008/04/02 16:03:56  taurel
+// - Fix compatibility bug in the Database::get_device_name() method
+// Revision 3.26.2.6  2008/05/20 06:17:46  taurel
+// - Last commit before merge with trunk
+// (start the implementation of the new DevEncoded data type)
+//
+// Revision 3.30  2008/03/20 07:39:24  taurel
+// - Last commit before Release 6.1 ?
+// Revision 3.26.2.5  2008/02/07 15:58:14  taurel
+// - First implementation of the Controlled Access done
+//
+// Revision 3.29  2008/01/15 12:31:37  taurel
+// - Change db connection and re-connection timeout
+// Revision 3.26.2.4  2008/01/03 16:05:52  taurel
+// - Some changes in locking feature implementation
+//
+// Revision 3.26.2.3  2007/12/20 14:29:01  taurel
+// - Some more work on locking
+//
+// Revision 3.26.2.2  2007/12/19 15:54:48  taurel
+// - Still some work going on for the locking feature
+//
+// Revision 3.26.2.1  2007/11/16 14:12:35  taurel
+// - Added a new IDL interface (Device_4)
+// - Added a new way to get attribute history from polling buffer (must faster)
+//
+// Revision 3.28  2008/01/07 07:17:21  taurel
+// - First access control system coding
+//
+// Revision 3.27  2007/12/12 10:17:18  taurel
+// - Db calls during DS startup has a separate timeout and some retries
+//
+// Revision 3.26  2007/11/08 12:03:44  taurel
+// - Start implementing user interceptors
+// - Fix bug in poll thread pproperty management when removing polling object
+// - Set a database timeout to 6 sec
+//
+// Revision 3.25  2007/10/16 08:24:14  taurel
+// - Add management of the TC connection establishment timeout for DB access
+// - Add DB server cache in DS used during DS startup sequence
+// - Comment out the sleep time during DS startup sequence
+//
 // Revision 3.24  2007/06/15 07:29:43  jensmeyer
 // Increase the tango library version number
 //
@@ -236,11 +329,6 @@
 // - Change the way how TangoMonitor class is implemented. It's a recursive
 //   mutex
 //
-//
-// copyleft :           European Synchrotron Radiation Facility
-//                      BP 220, Grenoble 38043
-//                      FRANCE
-//
 //=============================================================================
 
 #ifndef _TANGO_CONST_H
@@ -253,16 +341,16 @@ namespace Tango
 // Some general interest define
 //
 
-#define		TgLibVers				"6.0.0"		// Please, always code this following format "X.Y.Z"
+#define		TgLibVers				"7.0.2"		// Please, always code this following format "X.Y.Z"
 
-#define		DevVersion				3
+#define		DevVersion				4			// IDL version number
 #define		DefaultMaxSeq			20
-#define		DefaultBlackBoxDepth	25
+#define		DefaultBlackBoxDepth	50
 #define 	DefaultPollRingDepth	10
 
 #define		InitialOutput			"Initial Output"
 #define		DSDeviceDomain  		"dserver"
-#define		DefaultDocUrl			"http://www.esrf.fr/tango"
+#define		DefaultDocUrl			"http://www.tango-controls.org"
 #define		EnvVariable				"TANGO_HOST"
 #define		DbObjName				"database"
 #define		DescNotSet				"Uninitialised"
@@ -285,8 +373,7 @@ namespace Tango
 #define		ONE_SECOND				1000000
 #define		DISCARD_THRESHOLD		0.02
 
-#define		DEFAULT_TIMEOUT_SEC		4
-#define		DEFAULT_TIMEOUT			DEFAULT_TIMEOUT_SEC * 1000
+#define		DEFAULT_TIMEOUT			3200
 #define		DEFAULT_POLL_OLD_FACTOR	4
 
 #define		TG_IMP_MINOR_TO			10
@@ -307,6 +394,14 @@ namespace Tango
 #define     DELTA_PERIODIC_LONG			100   // For times > 5000ms only keep a delta of 100ms
 #define		HEARTBEAT					"Event heartbeat"
 
+//
+// Locking feature related defines
+//
+
+#define		DEFAULT_LOCK_VALIDITY	10
+#define		DEVICE_UNLOCKED_REASON	"API_DeviceUnlocked"
+#define		MIN_LOCK_VALIDITY		2
+
 //
 // Client timeout as defined by omniORB4.0.0
 //
@@ -314,12 +409,38 @@ namespace Tango
 #define		CLNT_TIMEOUT_STR		"3000"
 #define		CLNT_TIMEOUT			3000
 
+//
+// Connection and call timeout for database device
+//
+
+#define		DB_CONNECT_TIMEOUT		25000
+#define		DB_RECONNECT_TIMEOUT	20000
+#define		DB_TIMEOUT				13000
+#define		DB_START_PHASE_RETRIES	3
+
 //
 // Time to wait before trying to reconnect after
 // a connevtion failure
 //
 #define 	RECONNECTION_DELAY		1000   //ms. Only try to reconnect every second
 
+//
+// Access Control related defines
+// WARNING: these string are also used within the Db stored procedure
+// introduced in Tango V6.1. If you chang eit here, don't forget to
+// also update the stored procedure
+//
+
+#define		CONTROL_SYSTEM			"CtrlSystem"
+#define		SERVICE_PROP_NAME		"Services"
+#define		ACCESS_SERVICE			"AccessControl"
+
+//
+// Polling threads pool related defines
+//
+
+#define		DEFAULT_POLLING_THREADS_POOL_SIZE	1
+
 //
 // Max transfer size 256 MBytes (in byte). Needed by omniORB
 //
@@ -332,6 +453,13 @@ namespace Tango
 
 #define		MaxServerNameLength		255
 
+//
+// Files used to retrieve env. variables
+//
+
+#define		USER_ENV_VAR_FILE		".tangorc"
+#define		HOST_ENV_VAR_FILE		"/etc/tangorc"
+
 //
 // A short inline function to hide the CORBA::string_dup function
 //
@@ -805,7 +933,8 @@ enum CmdArgType {
 	DEV_ULONG64,
 	DEVVAR_LONG64ARRAY,
 	DEVVAR_ULONG64ARRAY,
-	DEV_INT
+	DEV_INT,
+	DEV_ENCODED
 };
 
 enum MessBoxType {
@@ -816,7 +945,8 @@ enum MessBoxType {
 enum PollObjType {
 	POLL_CMD = 0,
 	POLL_ATTR,
-	EVENT_HEARTBEAT
+	EVENT_HEARTBEAT,
+	STORE_SUBDEV
 };
 
 enum PollCmdCode {
@@ -844,13 +974,20 @@ enum AttReqType {
 	WRITE_REQ
 };
 
+enum LockCmdCode {
+	LOCK_ADD_DEV = 0,
+	LOCK_REM_DEV,
+	LOCK_UNLOCK_ALL_EXIT,
+	LOCK_EXIT
+};
+
 //
 // The polled device structure
 //
 
 typedef struct _PollDevice
 {
-	string		dev_name;
+	string			dev_name;
 	vector<long>	ind_list;
 }PollDevice;
 
@@ -967,13 +1104,18 @@ enum EventType {
 	PERIODIC_EVENT,
 	ARCHIVE_EVENT,
 	USER_EVENT,
-	ATTR_CONF_EVENT
+	ATTR_CONF_EVENT,
+	DATA_READY_EVENT
 };
 
 enum KeepAliveCmdCode {
 	EXIT_TH = 0
 };
 
+enum AccessControlType {
+	ACCESS_READ = 0,
+	ACCESS_WRITE
+};
 
 //
 // The optional attribute properties
@@ -1049,7 +1191,22 @@ typedef struct _OptAttrProp
  * API_EventSupplierNotConstructed
  * API_CantGetClientIdent
  * API_DeviceLocked
+ * API_CannotCheckAccessControl
  
+ * API_DeviceNotLocked
+ * API_DeviceUnlocked
+ * API_DeviceUnlockable
+ * API_WrongLockingStatus
+ * API_CantFindLockingThread
+ * API_WrongHistoryDataBuffer
+ * API_CannotCheckAccessControl 
+ * API_ReadOnlyMode
+ * API_CantStoreDeviceClass
+ * API_CantCreateLockingThread
+ * 
+ * API_PollingThreadNotFound
+ * API_PolledDeviceNotInPoolConf
+ * API_PolledDeviceNotInPoolMap
  */
 
 } // End of Tango namespace
