@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: utils.cpp,v 3.40 2008/03/26 16:51:12 taurel Exp $\n$Name:  $";
+static const char *RcsId = "$Id: utils.cpp,v 3.58 2009/03/18 10:23:12 taurel Exp $\n$Name:  $";
 
 //+=============================================================================
 //
@@ -11,14 +11,101 @@ static const char *RcsId = "$Id: utils.cpp,v 3.40 2008/03/26 16:51:12 taurel Exp
 //
 // author(s) :          A.Gotz + E.Taurel
 //
-// $Revision: 3.40 $
+// Copyright (C) :      2004,2005,2006,2007,2008,2009
+//						European Synchrotron Radiation Facility
+//                      BP 220, Grenoble 38043
+//                      FRANCE
+//
+// This file is part of Tango.
+//
+// Tango is free software: you can redistribute it and/or modify
+// it under the terms of the GNU Lesser General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+// 
+// Tango is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU Lesser General Public License for more details.
+// 
+// You should have received a copy of the GNU Lesser General Public License
+// along with Tango.  If not, see <http://www.gnu.org/licenses/>.
+//
+// $Revision: 3.58 $
 //
 // $Log: utils.cpp,v $
+// Revision 3.58  2009/03/18 10:23:12  taurel
+// - Fix bug for Windows in GUI mode
+//
+// Revision 3.57  2009/03/17 08:10:42  taurel
+// - Remove the endPoint specification before the ORB_init() method call
+// - Still some Windows warnings fixes
+//
+// Revision 3.56  2009/03/04 08:26:55  taurel
+// - Windows DS: Add a console handler for DS to survive a Log-off
+//
+// Revision 3.55  2009/02/26 07:48:21  taurel
+// - The attribute data format is now transferred within the AttributeValue_4 structure
+//
+// Revision 3.54  2009/02/16 12:07:19  taurel
+// - Fix bug in Util::get_device_by_name() for admin device
+//
+// Revision 3.53  2009/02/03 15:18:51  jensmeyer
+// Extract sub device properties from the db_cache when the cache
+// is available.
+//
+// Revision 3.52  2009/01/29 16:23:49  taurel
+// - Commit after merge with branch Release_6_1_1-bugfixes
+//
+// Revision 3.51  2009/01/23 13:58:18  taurel
+// - Print out wrong option from the command line before aborting
+//
+// Revision 3.50  2009/01/21 12:49:03  taurel
+// - Change CopyRights for 2009
+//
+// Revision 3.49  2009/01/15 13:57:20  taurel
+// - Fix bugs found by Jens
+//
+// Revision 3.48  2008/10/13 14:59:38  taurel
+// - Set the ORB verifyObjectExistsAndType option to false when creating the ORB
+//
+// Revision 3.47  2008/10/06 15:01:36  taurel
+// - Changed the licensing info from GPL to LGPL
+//
+// Revision 3.46  2008/10/03 06:53:09  taurel
+// - Add some licensing info in each files
+//
+// Revision 3.45  2008/10/02 12:24:43  taurel
+// - The user now has the possibility to install its own event loop
+//
+// Revision 3.44  2008/10/02 09:09:47  taurel
+// - First implementation of multiple polling thread(s)
+//
+// Revision 3.43  2008/09/23 14:59:35  taurel
+// - Commit after the end of DevEncoded data type implementation
+// - The new test suite is also now running fine
+//
+// Revision 3.42  2008/06/10 07:52:15  taurel
+// - Add code for the DevEncoded attribute data type
+// Revision 3.40.2.2  2009/01/29 15:17:01  taurel
+// - Add some bug fixes for server used with the -f option (database as
+// file)
+//
+// Revision 3.41  2008/05/20 12:44:20  taurel
+// - Commit after merge with release 7 branch
+// Revision 3.40.2.1  2008/05/22 12:10:53  taurel
+// - Fix bug for server started without database (the db server itself for instance)
+//
 // Revision 3.40  2008/03/26 16:51:12  taurel
 // - Change the printed message when a DS starts (on cout instead of cerr)
+// Revision 3.32.2.3  2008/05/20 06:17:46  taurel
+// - Last commit before merge with trunk
+// (start the implementation of the new DevEncoded data type)
 //
 // Revision 3.39  2008/03/25 14:59:01  taurel
 // - Fix bug in the previous change
+// Revision 3.32.2.2  2008/02/07 15:58:15  taurel
+// - First implementation of the Controlled Access done
 //
 // Revision 3.38  2008/03/25 13:32:25  taurel
 // - Change when the RootPOA CORBA object is created. This allows DS to
@@ -38,6 +125,8 @@ static const char *RcsId = "$Id: utils.cpp,v 3.40 2008/03/26 16:51:12 taurel Exp
 //
 // Revision 3.34  2008/01/15 12:29:55  taurel
 // - Change db timeout when getting data for the db cache
+// Revision 3.32.2.1  2007/12/19 15:54:48  taurel
+// - Still some work going on for the locking feature
 //
 // Revision 3.33  2007/12/12 10:17:18  taurel
 // - Db calls during DS startup has a separate timeout and some retries
@@ -313,12 +402,6 @@ static const char *RcsId = "$Id: utils.cpp,v 3.40 2008/03/26 16:51:12 taurel Exp
 // - Change the way how TangoMonitor class is implemented. It's a recursive
 //   mutex
 //
-//
-//
-// copyleft :           European Synchrotron Radiation Facility
-//                      BP 220, Grenoble 38043
-//                      FRANCE
-//
 //-=============================================================================
 
 #if HAVE_CONFIG_H
@@ -468,6 +551,14 @@ Util::Util(int argc,char *argv[]):cl_list_ptr(NULL)
 
 	effective_job(argc,argv);
 	_constructed = true;
+
+//
+// For Windows, install a console handler to ignore LOGOFF event
+//
+
+#ifdef _TG_WINDOWS_
+	install_cons_handler();
+#endif
 }
 
 void Util::effective_job(int argc,char *argv[])
@@ -500,7 +591,6 @@ void Util::effective_job(int argc,char *argv[])
 
 		DServerSignal::instance();
 
-
 //
 // Initialise CORBA ORB
 //
@@ -508,25 +598,21 @@ void Util::effective_job(int argc,char *argv[])
 #ifdef _TG_WINDOWS_
 		WORD rel = 0x0202;
 		WSADATA dat;
-		int init_res = WSAStartup(rel,&dat);
+		WSAStartup(rel,&dat);
 #endif
 
 		char h_name[80];
 		int res = gethostname(h_name,80);
 		if (res == 0)
 		{
-			char addr[180];
-			strcpy(addr,"giop:tcp:");
-			strcat(addr,h_name);
-			strcat(addr,":");
 			const char *options[][2] = {
-			{"endPoint",addr},
 			{"clientCallTimeOutPeriod",CLNT_TIMEOUT_STR},
 			{"serverCallTimeOutPeriod","5000"},
 			{"maxServerThreadPoolSize","100"},
 			{"threadPerConnectionUpperLimit","55"},
 			{"threadPerConnectionLowerLimit","50"},
 			{"supportCurrent","0"},
+			{"verifyObjectExistsAndType","0"},
 			{"giopMaxMsgSize",MAX_TRANSFER_SIZE},
 			{0,0}};
 			
@@ -542,6 +628,7 @@ void Util::effective_job(int argc,char *argv[])
 			{"threadPerConnectionUpperLimit","55"},
 			{"threadPerConnectionLowerLimit","50"},
 			{"supportCurrent","0"},
+			{"verifyObjectExistsAndType","0"},
 			{"giopMaxMsgSize",MAX_TRANSFER_SIZE},
 			{0,0}};
 			
@@ -592,6 +679,8 @@ void Util::effective_job(int argc,char *argv[])
 				display_help_message();
 			}
 		}
+		else
+			db = NULL;
 
 //
 // Create the server CORBA objects
@@ -633,15 +722,13 @@ void Util::effective_job(int argc,char *argv[])
 		create_event_supplier();
 
 //
-// Create the polling thread and start it
+// Create the heartbeat thread and start it
 //
 
-		ext->poll_th = new PollThread(ext->shared_data,ext->poll_mon);
-		ext->poll_th->start();
-		ext->poll_th_id = ext->poll_th->id();
-		cout4 << "Pollthread Id = " << ext->poll_th_id;
-								
-
+		ext->heartbeat_th = new PollThread(ext->shared_data,ext->poll_mon,true);
+		ext->heartbeat_th->start();
+		ext->heartbeat_th_id = ext->heartbeat_th->id();
+		cout4 << "Heartbeat thread Id = " << ext->heartbeat_th_id;
 								
 		cout4 << "Tango object singleton constructed" << endl;
 		
@@ -744,19 +831,19 @@ Util::Util(HINSTANCE hInst,int nCmdShow):cl_list_ptr(NULL),mon("Windows startup"
 // This method should be called from a Windows graphic program
 //
 
-        _win = true;
+	_win = true;
 
 //
 // Build UNIX like command argument(s)
 //
 
-        build_argc_argv();
+	build_argc_argv();
 
 //
 // Really do the job now
 //
 
-        effective_job(argc,argv);
+	effective_job(argc,argv);
 
 //
 // Store the nCmdShow parameter an mark the object has being completely
@@ -873,6 +960,7 @@ void Util::check_args(int argc,char *argv[])
 								set_trace_level(4);
 							else
 							{
+								cerr << "Unknown option " << argv[ind] << endl;
 								print_usage(argv[0]);
 							}
 						}
@@ -885,6 +973,7 @@ void Util::check_args(int argc,char *argv[])
 						long level = atol(&(argv[ind][2]));
 						if (level == 0)
 						{
+							cerr << "Unknown option " << argv[ind] << endl;
 							print_usage(argv[0]);
 						}
 						else
@@ -901,7 +990,10 @@ void Util::check_args(int argc,char *argv[])
 					
 				case 'n':
 					if (strcmp(argv[ind],"-nodb") != 0)
+					{
+						cerr << "Unknown option " << argv[ind] << endl;
 						print_usage(argv[0]);
+					}
 					else
 					{
 						if ( _FileDb )
@@ -913,12 +1005,15 @@ void Util::check_args(int argc,char *argv[])
 
 				case 'f':
 					if (strncmp(argv[ind],"-file=",6) != 0)
+					{
+						cerr << "Unknown option " << argv[ind] << endl;
 						print_usage(argv[0]);
+					}
 					else
 					{
 						if ( !_UseDb )
 							print_usage(argv[0]);
-						_FileDb = true;
+						Tango::Util::_FileDb = true;
 						database_file_name = argv[ind]; 
 						database_file_name.erase(0,6);
 #ifdef _TG_WINDOWS_
@@ -943,7 +1038,10 @@ void Util::check_args(int argc,char *argv[])
 					}
 					
 					if (strcmp(argv[ind],"-dlist") != 0)
+					{
+						cerr << "Unknown option " << argv[ind] << endl;
 						print_usage(argv[0]);
+					}
 					else
 					{
 						if (_UseDb == true)
@@ -1017,7 +1115,7 @@ void Util::check_args(int argc,char *argv[])
 	}
 	
 //
-// Build server name in lower-case letters
+// Build server name
 //
 
 //	long ctr;
@@ -1120,12 +1218,12 @@ void Util::display_help_message()
 
 void Util::print_usage(char *serv_name)
 {
-        TangoSys_OMemStream o;
+	TangoSys_OMemStream o;
 		
 	o << "usage :  " << serv_name << " instance_name [-v[trace level]]";
 	o << " [-file=<file_name> | -nodb [-dlist <device name list>] ]" << ends;
 	
-        print_err_message(o.str(),Tango::INFO);
+	print_err_message(o.str(),Tango::INFO);
 }
 
 //+----------------------------------------------------------------------------
@@ -1256,8 +1354,9 @@ void Util::connect_db()
 //
 // Set a timeout on the database device
 //
-	
-	db->set_timeout_millis(DB_TIMEOUT);
+
+	if (_FileDb == false)	
+		db->set_timeout_millis(DB_TIMEOUT);
 	
 //
 // Also copy this database object ptr into the ApiUtil object. Therefore,
@@ -1289,10 +1388,10 @@ void Util::connect_db()
 			{
 				ext->db_cache = new DbServerCache(db,get_ds_name(),get_host_name());
 			}
-			catch (Tango::DevFailed &e) {}
+			catch (Tango::DevFailed &) {}
 			catch (...) 
 			{
-				cerr << "Unknown exception while trying to fill database cache" << endl;
+				cerr << "Unknown exception while trying to fill database cache..." << endl;
 			}
 			db->set_timeout_millis(DB_TIMEOUT);
 		}
@@ -1425,7 +1524,8 @@ void Util::misc_init()
 //
 
 #ifdef _TG_WINDOWS_
-	main_win_text = "TANGO collaboration (ALBA / DESY / ELETTRA / ESRF / SOLEIL )\n";
+	main_win_text = "TANGO collaboration\n";
+	main_win_text = main_win_text + "(ALBA / DESY / ELETTRA / ESRF / SOLEIL )\n";
 	main_win_text = main_win_text + "CORBA based device server\n";
 	main_win_text = main_win_text + "Developped by Tango team";
 #endif
@@ -1540,22 +1640,11 @@ Util::~Util()
 #ifdef _TG_WINDOWS_
 	if (ds_window != NULL)
 	{
-		TangoMonitor &mon = get_poll_monitor();
-		PollThCmd &shared_cmd = get_poll_shared_cmd();
-
-		{	
-			omni_mutex_lock sync(mon);
-
-			shared_cmd.cmd_pending = true;
-			shared_cmd.cmd_code = POLL_EXIT;
-
-			mon.signal();
-
-		}
+		stop_all_polling_threads();
+		stop_heartbeat_thread();
+		clr_heartbeat_th_ptr();
 		
-		get_polling_thread_object()->join(0);
-		clr_poll_th_ptr();
-      	   	unregister_server();
+		unregister_server();
 		get_dserver_device()->delete_devices();
 		if (_FileDb == true)
 			delete db;
@@ -1751,7 +1840,6 @@ void Util::server_already_running()
 
 void Util::server_init(bool with_window)
 {
-
 //
 // Even if we are not in a Python DS, we have to create the per-thread
 // PyData object. For Python DS, this is done in the Python_init method
@@ -1764,7 +1852,7 @@ void Util::server_init(bool with_window)
 		omni_thread::create_dummy();
 	}
 #endif
-	
+
 	omni_thread *th = omni_thread::self();
 	if (is_py_ds() == false)
 	{
@@ -1814,7 +1902,7 @@ void Util::server_init(bool with_window)
 //
 
 		DServerClass::init();
-		
+
 //
 // Configure polling from the polling properties
 // In case of python DS, we need to release the Python GIL
@@ -1839,20 +1927,23 @@ void Util::server_init(bool with_window)
 		{
 			lock_ptr->Get();
 		}
-		
-#ifdef _TG_WINDOWS_
-	}
-#endif /* _TG_WINDOWS_ */
 
 //
 // Delete the db cache if it has been used
 //
 	
-	if (ext->db_cache != NULL)
-	{
-		delete ext->db_cache;
-		ext->db_cache = NULL;
+		if (ext->db_cache != NULL)
+		{
+		// extract sub device information before deleting cache!
+			get_sub_dev_diag().get_sub_devices_from_cache();
+		
+	
+			delete ext->db_cache;
+			ext->db_cache = NULL;
+		}
+#ifdef _TG_WINDOWS_
 	}
+#endif /* _TG_WINDOWS_ */
 }
 
 //+----------------------------------------------------------------------------
@@ -1952,7 +2043,42 @@ void Util::server_run()
 	//JM : 9.8.2005 : destroy() should be called at the exit of run()!
 	try
 	{
-		orb->run();
+		if (ext->ev_loop_func != NULL)
+		{
+			
+//
+// If the user has installed its own event management function, call it in a loop
+//
+			
+			struct timespec sleep_time;
+			sleep_time.tv_sec = 0;
+			sleep_time.tv_nsec = 20000000;
+			bool user_shutdown_server;
+			
+			while(ext->shutdown_server == false)
+			{
+				if (is_svr_shutting_down() == false)
+				{
+					if (orb->work_pending())
+						orb->perform_work();
+	
+					user_shutdown_server = (*ext->ev_loop_func)();
+					if (user_shutdown_server == true)
+					{
+						shutdown_server();
+						ext->shutdown_server = true;
+					}
+				}
+				else
+				{
+					nanosleep(&sleep_time,NULL);
+				}
+			}
+		}
+		else
+		{
+			orb->run();
+		}
 		server_cleanup();
 
 		if (th_id == 0)
@@ -1961,7 +2087,6 @@ void Util::server_run()
 			PyLock *lock_ptr = (static_cast<PyData *>(tmp_py_data))->PerTh_py_lock;
 			lock_ptr->Get();
 		}
-	
 	}
 	catch (CORBA::Exception &e)
 	{
@@ -2089,7 +2214,7 @@ DeviceImpl *Util::get_device_by_name(const string &dev_name)
 	string dev_name_lower(dev_name);
 	transform(dev_name_lower.begin(),dev_name_lower.end(),dev_name_lower.begin(),::tolower);
 
-	DeviceImpl *ret_ptr = find_device_name_core(dev_name_lower);	
+	DeviceImpl *ret_ptr = find_device_name_core(dev_name_lower);
 
 //
 // If the device is not found, may be the name we have received is an alias ?
@@ -2466,509 +2591,6 @@ void Util::print_err_message(const char *err_mess,Tango::MessBoxType type)
 #endif	
 }
 
-//+----------------------------------------------------------------------------
-//
-// method : 		Util::polling_configure()
-// 
-// description : 	This method sends command to the polling thread for
-//			all cmd/attr with polling configuration stored in db.
-//			This is done in separate thread in order to equally
-//			spread all the polled objects polling time on the
-//			smallest polling period.
-//
-//-----------------------------------------------------------------------------
-
-void Util::polling_configure()
-{
-
-//
-// Send a stop polling command to thread in order not to poll devices
-//
-
-	DServer *adm_dev = get_dserver_device();
-	adm_dev->stop_polling();
-
-	vector<DeviceClass *> &tmp_cl_list = adm_dev->get_class_list();
-	unsigned long i,j,k;
-	int upd;
-	TangoSys_MemStream s;
-
-//
-// Create the structure used to send data to the polling thread
-// and store them in a vector
-//
-
-	vector<DevVarLongStringArray *> v_poll_cmd;
-
-//
-// A loop on each class and each device in class
-//
-
-	int smallest_upd;
-	bool first_loop = true;
-
-	for (i = 0;i < tmp_cl_list.size();i++)
-	{
-		vector<DeviceImpl *> &dev_list = tmp_cl_list[i]->get_device_list();
-		for (j = 0;j < dev_list.size();j++)
-		{
-			vector<string> &poll_cmd_list = dev_list[j]->get_polled_cmd();
-			vector<string> &poll_attr_list = dev_list[j]->get_polled_attr();
-
-//
-// Check that cmd_list and attr_list have a correct syntax
-//
-
-			if ((poll_cmd_list.size() % 2) == 1)
-			{
-				TangoSys_OMemStream o;
-				o << "System property polled_cmd for device " << dev_list[j]->get_name() << " has wrong syntax" << ends;
-				Except::throw_exception((const char *)"API_BadConfigurationProperty",
-				        		o.str(),
-				        		(const char *)"Util::polling_configure");
-			}
-
-			if ((poll_attr_list.size() % 2) == 1)
-			{
-				TangoSys_OMemStream o;
-				o << "System property polled_attr for device " << dev_list[j]->get_name() << " has wrong syntax" << ends;
-				Except::throw_exception((const char *)"API_BadConfigurationProperty",
-				        		o.str(),
-				        		(const char *)"Util::polling_configure");
-			}
-
-
-//
-// A loop on each command
-//
-
-			for (k = 0;k < poll_cmd_list.size();k++)
-			{
-
-				DevVarLongStringArray *send = new DevVarLongStringArray();
-				send->lvalue.length(1);
-				send->svalue.length(3);
-
-				send->svalue[0] = CORBA::string_dup(dev_list[j]->get_name().c_str());
-				send->svalue[1] = CORBA::string_dup("command");
-				send->svalue[2] = CORBA::string_dup(poll_cmd_list[k].c_str());
-							
-//
-// Convert polling period to a number and store polling thread command params
-//
-
-				s << poll_cmd_list[k + 1];
-				s >> upd;
-				s.clear();
-				send->lvalue[0] = upd;
-
-				if (first_loop == true)
-				{
-					smallest_upd = upd;
-					first_loop = false;
-				}
-				else
-				{
-					if (upd < smallest_upd)
-						smallest_upd = upd;
-				}
-
-				v_poll_cmd.push_back(send);
-				k++;
-			}
-
-//
-// A loop on each attribute
-//
-
-			for (k = 0;k < poll_attr_list.size();k++)
-			{
-
-				DevVarLongStringArray *send = new DevVarLongStringArray();
-				send->lvalue.length(1);
-				send->svalue.length(3);
-
-				send->svalue[0] = CORBA::string_dup(dev_list[j]->get_name().c_str());
-				send->svalue[1] = CORBA::string_dup("attribute");
-				send->svalue[2] = CORBA::string_dup(poll_attr_list[k].c_str());
-							
-//
-// Convert polling period to a number ans store polling thread command params
-//
-
-				s << poll_attr_list[k + 1];
-				s >> upd;
-				s.clear();
-				send->lvalue[0] = upd;
-
-				if (first_loop == true)
-				{
-					smallest_upd = upd;
-					first_loop = false;
-				}
-				else
-				{
-					if (upd < smallest_upd)
-						smallest_upd = upd;
-				}
-
-				v_poll_cmd.push_back(send);
-				k++;
-			}
-		}
-	}
-
-
-	unsigned long nb_cmd = v_poll_cmd.size();
-
-//
-// Send command to polling thread one by one and sleep between
-// sending each command.
-// In the following computation , I try to take into account the
-// non real time aspect of our OS. I remove 15 mS from each
-// sleeping time due to thread wake-up time
-//
-
-	if (nb_cmd != 0)
-	{
-		int sleeping_time = smallest_upd / nb_cmd;
-		int delta_time = 0;
-	
-		long delta_os = 15;
-		if (delta_os < sleeping_time)
-			sleeping_time = sleeping_time - delta_os;
-
-		cout4 << "PollConfigureThread: smallest_upd = " << smallest_upd;
-		cout4 << ", delta_time = " << sleeping_time;
-		cout4 << ", nb_poll_objects = " << nb_cmd << endl;
-		
-		for (i = 0;i < nb_cmd;i++)
-		{
-			try
-			{
-				adm_dev->add_obj_polling(v_poll_cmd[i],false,delta_time);
-			}
-			catch (Tango::DevFailed &e)
-			{
-				bool throw_ex = true;
-
-				if (::strcmp(e.errors[0].reason.in(),"API_AlreadyPolled") == 0)
-				{
-					try
-					{
-						adm_dev->upd_obj_polling_period(v_poll_cmd[i],false);
-						throw_ex = false;
-					}
-					catch (Tango::DevFailed &) {}
-				}
-
-				if (throw_ex == true)
-				{
-					TangoSys_OMemStream o;
-					o << "Error when configuring polling for device " << v_poll_cmd[i]->svalue[0].in();
-					if (::strcmp(v_poll_cmd[i]->svalue[1].in(),"command") == 0)
-						o << ", cmd = ";
-					else
-						o << ", attr = ";
-					o << v_poll_cmd[i]->svalue[2].in() << ends;
-					Except::re_throw_exception(e,(const char *)"API_BadConfigurationProperty",
-								o.str(),
-								(const char *)"Util::polling_configure");
-				}
-			}
-
-
-			if (nb_cmd > 1)
-			{		
-				delta_time = delta_time + sleeping_time;
-			}
-		}
-	}
-		
-//
-// Now, start the real polling
-//
-
-	adm_dev->start_polling();
-
-//
-// Delete allocated memory
-//
-
-	for (unsigned long l = 0;l < nb_cmd;l++)
-		delete v_poll_cmd[l];
-
-}
-
-//+----------------------------------------------------------------------------
-//
-// method : 		Util::trigger_attr_polling()
-// 
-// description : 	Trigger the polling thread for polled attributes
-//			registered with a polling update period set as
-//			"externally triggered" (0 mS)
-//
-//-----------------------------------------------------------------------------
-
-void Util::trigger_attr_polling(Tango::DeviceImpl *dev,string &name)
-{
-	cout4 << "Sending trigger to polling thread" << endl;
-	
-//
-// Check that the device is polled
-//
-
-	if (dev->is_polled() == false)
-	{
-		TangoSys_OMemStream o;
-		o << "Device " << dev->get_name() << " is not polled" << ends;
-		
-		Except::throw_exception((const char *)"API_DeviceNotPolled",o.str(),
-				   	(const char *)"Util::trigger_attr_polling");
-	}
-	
-//
-// Find the wanted object in the list of device polled object
-//
-
-	string obj_name(name);
-	transform(obj_name.begin(),obj_name.end(),obj_name.begin(),::tolower);
-		
-	vector<PollObj *>::iterator ite = dev->get_polled_obj_by_type_name(Tango::POLL_ATTR,obj_name);
-
-//
-// Check that it is an externally triggered polling object. If it is not the
-// case, throw exception
-//
-
-	long tmp_upd = (*ite)->get_upd();
-	if (tmp_upd != 0)
-	{
-		TangoSys_OMemStream o;
-
-		o << "Polling for attribute ";		
-		o << name;
-		o << " (device " << dev->get_name() << ") ";
-		o << " is not externally triggered.";
-		o << ends;
-		Except::throw_exception((const char *)"API_NotSupported",o.str(),
-					(const char *)"Util::trigger_attr_polling");
-	}
-	
-//
-// Send command to the polling thread but wait in case of previous cmd 
-// still not executed
-//
-
-	TangoMonitor &mon = get_poll_monitor();
-	PollThCmd &shared_cmd = get_poll_shared_cmd();
-
-	{	
-		omni_mutex_lock sync(mon);
-		if (shared_cmd.trigger == true)
-		{
-			mon.wait();
-		}
-		shared_cmd.trigger = true;
-		shared_cmd.dev = dev;
-		shared_cmd.name = obj_name;
-		shared_cmd.type = Tango::POLL_ATTR;
-
-		mon.signal();
-		
-		cout4 << "Trigger sent to polling thread" << endl;
-		
-//
-// Wait for thread to execute command
-//
-
-/*		TangoMonitor &dev_mon = dev->get_dev_monitor();
-		omni_thread *th = omni_thread::self();
-		
-		while (shared_cmd.trigger == true)
-		{
-
-//
-// Warning: It's possible to have a deadlock here (experienced under
-// Windows) in case of this method being called from a command (or attribute
-// methods) which are rapidly sent by the client.
-// Client request cmd1 which send trigger to the polling thread
-// The polling thread wake up clear shared_cmd.trigger and try to
-// execute the command. But cmd 1 thread still owns the device monitor and
-// polling thread wait. cmd 1 finished and client immediately send the
-// command a new time. On Windows, it may happens that the polling
-// thread is not activated just after the cmd thread has released the
-// device monitor. As the client sent a new command, the device monitor
-// is immediately re-taken by the thread executing the new command sent by
-// the client. An order is sent to the polling thread and the cmd
-// thread reach this code. It will wait for polling thread to clear
-// shared_cmd.trigger. But, the polling thread is already waiting for
-// the device monitor and ..... deadlock....
-// 
-		
-			bool deadlock = false;
-			if (th->id() == dev_mon.get_locking_thread_id())
-			{
-				cout4 << "Possible deadlock detected!" << endl;
-				deadlock = true;
-				dev_mon.rel_monitor();
-				dev_mon.rel_monitor();
-			}
-			
-			interupted = mon.wait(DEFAULT_TIMEOUT);
-			if (deadlock == true)
-			{
-				dev_mon.get_monitor();
-				dev_mon.get_monitor();
-			}
-			
-			if ((shared_cmd.trigger == true) && (interupted == false))
-			{
-				cout4 << "TIME OUT" << endl;
-				Except::throw_exception((const char *)"API_CommandTimedOut",
-					        	(const char *)"Polling thread blocked !!!",
-					        	(const char *)"Util::trigger_attr_polling");
-			}
-		}*/
-	}
-	cout4 << "Thread cmd normally executed" << endl;
-}
-
-
-//+----------------------------------------------------------------------------
-//
-// method : 		Util::trigger_cmd_polling()
-// 
-// description : 	Trigger the polling thread for polled command
-//			registered with a polling update period set as
-//			"externally triggered" (0 mS)
-//
-//-----------------------------------------------------------------------------
-
-void Util::trigger_cmd_polling(Tango::DeviceImpl *dev,string &name)
-{
-	cout4 << "Sending trigger to polling thread" << endl;
-	
-//
-// Check that the device is polled
-//
-
-	if (dev->is_polled() == false)
-	{
-		TangoSys_OMemStream o;
-		o << "Device " << dev->get_name() << " is not polled" << ends;
-		
-		Except::throw_exception((const char *)"API_DeviceNotPolled",o.str(),
-				   	(const char *)"Util::trigger_cmd_polling");
-	}
-	
-//
-// Find the wanted object in the list of device polled object
-//
-
-	string obj_name(name);
-	transform(obj_name.begin(),obj_name.end(),obj_name.begin(),::tolower);
-		
-	vector<PollObj *>::iterator ite = dev->get_polled_obj_by_type_name(Tango::POLL_CMD,obj_name);
-
-//
-// Check that it is an externally triggered polling object. If it is not the
-// case, throw exception
-//
-
-	long tmp_upd = (*ite)->get_upd();
-	if (tmp_upd != 0)
-	{
-		TangoSys_OMemStream o;
-
-		o << "Polling for command ";		
-		o << name;
-		o << " (device " << dev->get_name() << ") ";
-		o << " is not externally triggered.";
-		o << ends;
-		Except::throw_exception((const char *)"API_NotSupported",o.str(),
-					(const char *)"Util::trigger_cmd_polling");
-	}
-	
-//
-// Send command to the polling thread but wait in case of previous cmd 
-// still not executed
-//
-
-	TangoMonitor &mon = get_poll_monitor();
-	PollThCmd &shared_cmd = get_poll_shared_cmd();
-
-	{	
-		omni_mutex_lock sync(mon);
-		if (shared_cmd.trigger == true)
-		{
-			mon.wait();
-		}
-		shared_cmd.trigger = true;
-		shared_cmd.dev = dev;
-		shared_cmd.name = obj_name;
-		shared_cmd.type = Tango::POLL_CMD;
-
-		mon.signal();
-		
-		cout4 << "Trigger sent to polling thread" << endl;
-		
-//
-// Wait for thread to execute command
-//
-
-/*		TangoMonitor &dev_mon = dev->get_dev_monitor();
-		omni_thread *th = omni_thread::self();
-		
-		while (shared_cmd.trigger == true)
-		{
-		
-//
-// Warning: It's possible to have a deadlock here (experienced under
-// Windows) in case of this method being called from a command (or attribute
-// methods) which are rapidly sent by the client.
-// Client request cmd1 which send trigger to the polling thread
-// The polling thread wake up clear shared_cmd.trigger and try to
-// execute the command. But cmd 1 thread still owns the device monitor and
-// polling thread wait. cmd 1 finished and client immediately send the
-// command a new time. On Windows, it may happens that the polling
-// thread is not activated just after the cmd thread has released the
-// device monitor. As the client sent a new command, the device monitor
-// is immediately re-taken by the thread executing the new command sent by
-// the client. An order is sent to the polling thread and the cmd
-// thread reach this code. It will wait for polling thread to clear
-// shared_cmd.trigger. But, the polling thread is already waiting for
-// the device monitor and ..... deadlock....
-// 
-
-			bool deadlock = false;
-			if (th->id() == dev_mon.get_locking_thread_id())
-			{
-				cout4 << "Possible deadlock detected!" << endl;
-				deadlock = true;
-				dev_mon.rel_monitor();
-				dev_mon.rel_monitor();
-			}
-
-			interupted = mon.wait(DEFAULT_TIMEOUT);
-			if (deadlock == true)
-			{
-				dev_mon.get_monitor();
-				dev_mon.get_monitor();
-			}
-			
-			if ((shared_cmd.trigger == true) && (interupted == false))
-			{
-				cout4 << "TIME OUT" << endl;
-				Except::throw_exception((const char *)"API_CommandTimedOut",
-					        	(const char *)"Polling thread blocked !!!",
-					        	(const char *)"Util::trigger_cmd_polling");
-			}
-		}*/
-	}
-	cout4 << "Thread cmd normally executed" << endl;
-}
-
 
 //+----------------------------------------------------------------------------
 //
@@ -2984,52 +2606,6 @@ long Util::get_tango_lib_release()
 	return _convert_tango_lib_release(TgLibVers);
 }
 
-//+----------------------------------------------------------------------------
-//
-// method : 		Util::clean_attr_polled_prop()
-// 
-// description : 	Clean in database the prop used to memorized which attribute
-//					are polled	
-//
-//-----------------------------------------------------------------------------
-
-void Util::clean_attr_polled_prop()
-{
-	if (Tango::Util::_UseDb == true)
-	{
-		DbData send_data;
-		DbDatum db_info("polled_attr");
-	
-		for (unsigned int loop = 0;loop < ext->polled_dyn_attr_names.size();loop++)
-		{		
-			vector<string>::iterator ite_attr = find(ext->polled_att_list.begin(),ext->polled_att_list.end(), ext->polled_dyn_attr_names[loop]);	
-			if (ite_attr != ext->polled_att_list.end())
-			{
-				ite_attr = ext->polled_att_list.erase(ite_attr);
-				ext->polled_att_list.erase(ite_attr);
-			}
-			else
-			{
-				TangoSys_OMemStream o;
-
-				o << "Polling properties for attribute " << ext->polled_dyn_attr_names[loop] << " on device " << ext->dyn_att_dev_name;
-				o << " not found device in polled attribute list!" << ends;		
-
-				Except::throw_exception((const char *)"API_MethodArgument",o.str(),
-										(const char *)"Util::clean_attr_polling_prop");
-			}
-		}		
-
-		db_info << ext->polled_att_list;
-		send_data.push_back(db_info);
-		
-		if (db_info.size() == 0)
-			db->delete_device_property(ext->dyn_att_dev_name,send_data);
-		else
-			db->put_device_property(ext->dyn_att_dev_name,send_data);
-	}	
-}
-
 
 //+----------------------------------------------------------------------------
 //
@@ -3183,6 +2759,26 @@ CoutBuf *Util::get_debug_object()
 	return ds_window ? ds_window->get_output_buffer() : 0;
 }
 
+BOOL CtrlHandler(DWORD fdwCtrlType)  
+{ 
+	switch( fdwCtrlType )  
+	{  
+// Ignore logoff event! 
+	case CTRL_LOGOFF_EVENT: 
+		return TRUE;  
+ 
+// Pass all other signals to the next signal handler 
+	default:  
+		return FALSE;  
+	}  
+}  
+
+void Util::install_cons_handler()
+{
+	if (!SetConsoleCtrlHandler((PHANDLER_ROUTINE)CtrlHandler,TRUE))
+		print_err_message("WARNING: Can't install the console handler");
+}
+
 
 //+----------------------------------------------------------------------------
 //
@@ -3238,6 +2834,19 @@ void *Util::ORBWin32Loop::run_undetached(void *ptr)
 
 
 	util->polling_configure();
+	
+//
+// Delete DB cache (if there is one)
+//
+
+	if (util->ext->db_cache != NULL)
+	{
+		// extract sub device information before deleting cache!
+		util->get_sub_dev_diag().get_sub_devices_from_cache();
+		
+		delete util->ext->db_cache;
+		util->ext->db_cache = NULL;
+	}
 
 //
 // Start the ORB
@@ -3282,6 +2891,15 @@ void clear_att_dim(Tango::AttributeValue_3 &att_val)
 	att_val.w_dim.dim_y = 0;
 }
 
+void clear_att_dim(Tango::AttributeValue_4 &att_val)
+{
+	att_val.r_dim.dim_x = 0;
+	att_val.r_dim.dim_y = 0;
+	att_val.w_dim.dim_x = 0;
+	att_val.w_dim.dim_y = 0;
+	
+	att_val.data_format = Tango::FMT_UNKNOWN;
+}
 
 //
 // The function called by the interceptor on thread creation
