@@ -1,13 +1,19 @@
- static const char *RcsId = "$Id: notifd2db.cpp,v 1.5 2008/03/03 10:22:15 taurel Exp $";
+ static const char *RcsId = "$Id: notifd2db.cpp,v 1.7 2008/09/22 15:24:36 taurel Exp $";
 
 /*
  * this program will export the Notification Factory to the TANGO database
  *
  * - andy 30nov03
  *		
- *  $Revision: 1.5 $
+ *  $Revision: 1.7 $
  *
  *  $Log: notifd2db.cpp,v $
+ *  Revision 1.7  2008/09/22 15:24:36  taurel
+ *  - Fix the main function declaration for gcc 4.3
+ *
+ *  Revision 1.6  2008/04/24 12:06:27  tiagocoutinho
+ *  fix bug 1950478: notifd2db hangs
+ *
  *  Revision 1.5  2008/03/03 10:22:15  taurel
  *  - Added retries around DB object creation and calls (in case of heavily loaded DB)
  *
@@ -47,7 +53,7 @@ void print_usage();
 void append_to_file(string &,string &);
 
 
-int main(unsigned int argc, char **argv)
+int main(int argc, char **argv)
 {
 	bool def = false;
 	vector<string> file_names;
@@ -98,6 +104,7 @@ int main(unsigned int argc, char **argv)
 			{
 				tango_db = new Tango::Database();
 				tango_db->set_timeout_millis(10000);
+				num_retries = 0;
 			}
 			catch (Tango::DevFailed &e)
 			{
